/*
DeltaChars dialogue

v1
*/

local DC = DeltaChars
local DB = DC.DialogueBox
local DF = DB.FLAGS

DB.talkers = {
	susie = {
		prefix = "SUSIE",
		default = 1,
		sound = sfx_susie
	},
	ralsei = {
		prefix = "RALSEI",
		default = 1,
		sound = sfx_ralsei
	},
	lancer = {
		prefix = "LANCER",
		default = 2,
		sound = sfx_lancer
	},
	rk = {
		prefix = "ROUXLS",
		default = 1,
		overlay = "ROUXLS_HAT"
	},
	eggman = {
		prefix = "DC_EGGMAN",
		default = 1,
		sound = sfx_dceggm
	},
	fang = {
		prefix = "DC_FANG",
		default = 1,
		sound = sfx_dcfang
	},
	toriel = {
		prefix = "TORIEL",
		default = 1,
		sound = sfx_toriel
	},
	queen = {
		prefix = "DC_QUEEN",
		default = 1,
		sound = sfx_dcquen
	},
	spamton = {
		default = 1,
		sound = sfx_dcspam
	},
	mute = {
		default = 1,
		sound = sfx_dcnull
	},
}

local vars = {}
DB.vars = vars

local function yield()
	coroutine.yield()
end

local function wait(tics)
	while tics > 0
		tics = $ - 1
		yield()
	end
end

local function talk(name, portraitNum, text)
	return {
		name = name,
		portraitNum = portraitNum,
		text = text
	}
end

local function spawnPartners(player)
	local mo = player.mo
	local skin1 = DC.GetNextSkin(player.skin)
	
	local dcSkins = {
		skins[skin1],
		skins[DC.GetNextSkin(skin1)]
	}
	
	local angle = (player.cmd.angleturn << 16) + ANGLE_90
	local diff = 72*mo.scale
	local zdiff = 256*mo.scale
	
	vars[1] = P_SpawnMobjFromMobj(mo, P_ReturnThrustX(mo, angle, diff), P_ReturnThrustY(mo, angle, diff), zdiff, MT_GFZFLOWER1)
	vars[2] = P_SpawnMobjFromMobj(mo, P_ReturnThrustX(mo, angle, -diff), P_ReturnThrustY(mo, angle, -diff), zdiff, MT_GFZFLOWER1)
	
	for i = 1, 2
		local v = vars[i]
		local skin = dcSkins[i]
		v.skin = skin.name
		v.color = skin.prefcolor
		v.angle = mo.angle
		v.state = S_PLAY_FALL
		v.shadowscale = FRACUNIT
		
		//SMS Alfredo's painful hat of death
		if player.ralseihat and v.skin == "ralsei"
			v.eflags = $ | MFE_FORCESUPER
			v.color = R_GetColorByName("darkner") or $
		end
	end
	
	while not P_IsObjectOnGround(vars[1])
		if P_IsObjectOnGround(mo)
			for i = 1, 2
				local v = vars[i]
				v.momz = $ + P_GetMobjGravity(v)
			end
		end
		yield()
	end
	
	for i = 1, 2
		local v = vars[i]
		v.state = S_PLAY_STND
	end
	return true
end

local function returnPartners(player)
	local mo = player.mo
			
	S_StartSound(mo, sfx_dcjump)
	for i = 1, 2
		local pal = vars[i]
		local speed = 3*FRACUNIT
		local dist = FixedHypot(pal.x - mo.x, pal.y - mo.y)
		pal.angle = R_PointToAngle2(pal.x, pal.y, mo.x, mo.y)
		pal.momz = FixedMul(-P_GetMobjGravity(pal), FixedDiv(dist, 2*speed))
		pal.z = $ + P_MobjFlip(pal)
		P_InstaThrust(pal, pal.angle, speed)
		pal.state = S_PLAY_ROLL
		pal.scalespeed = FRACUNIT/48
	end
	
	local timer = TICRATE*2
	while timer and not P_IsObjectOnGround(vars[1])
		timer = $ - 1
		for i = 1, 2
			local pal = vars[i]
			pal.spritexscale = $ - pal.scalespeed
			pal.spriteyscale = $ - pal.scalespeed
			DC.SpawnSpareSparkles(pal, 1)
		end
		yield()
	end
	
	for i = 1, 2
		P_RemoveMobj(vars[i])
		vars[i] = nil
	end
	return true
end

local function RouxlsLaser(x, y, z, func)
	local timer = 9*TICRATE/8
	local camAngle = R_PointToAngle(x, y)
	local laser = P_SpawnMobj(x, y, z, MT_THOK)
	local sector = laser.subsector.sector
	
	laser.state = S_DC_ROUXLSLASER
	laser.blendmode = AST_ADD
	
	P_TeleportMove(laser, x - cos(camAngle), y - sin(camAngle), P_FloorzAtPos(x, y, z, 0))
	laser.spriteyscale = sector.ceilingheight - laser.floorz
	
	while timer > 0
		timer = $ - 1
		
		if timer == 0
			if func
				func()
			end
		elseif laser.frame & FF_TRANSMASK
			laser.frame = $ - FF_TRANS10
			laser.spritexscale = $ + FRACUNIT
		end
		
		yield()
	end
	
	while laser
		if leveltime % 3 == 0
			laser.frame = $ + FF_TRANS10
		end
		
		if laser.frame & FF_TRANSMASK > FF_TRANS90
			P_RemoveMobj(laser)
			laser = nil
		else
			laser.spritexscale = FixedMul($, 4*FRACUNIT/5)
		end
		
		yield()
	end
	
	wait(TICRATE/2)
end

/*
OKAY, LISTEN UP
you want to add your own events, huh?

tag: the name of the event, used for identification and saving purposes
save: determines whether the event will only show once after finishing, true by default
zone: the title of the level this event should be available in (available in all if omitted)
act: the act number this event should be available in (available in all if omitted)
preCondition: a function that runs when the map loads.
	if it returns true, the event will be available. otherwise, it will not be.
	this function does not run if the map does not first meet the conditions of the zone and act parameters.
	if omitted, event will be available.
hookType & hookArg: the hook parameters for which this event will be polled while available.
	e.g. if hookType is set to "MobjDamage" and hookArg is set to MT_BLUECRAWALA,
	the event will attempt to activate whenever a blue crawla is damaged.
	if omitted, hookType defaults to "MapLoad".
condition: a function that is run whenever the event is polled.
	if it returns truthy, the event will activate. Otherwise, it will not.
	if omitted, the event will activate.
script: a table containing the dialogue. I will document this later because I need to try it first lol
*/

// ---------------------------------
// ---------------------------------
// GREENFLOWER
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "intro",
	zone = "greenflower",
	act = 1,
	flags = DF.STASIS,
	preCondition = function()
		return not (gamecomplete or vars.beatBrak)
	end,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 1, "..."),
		talk("susie", 12, "OK, Kris? What the HELL is this place?"),
		talk("susie", 23, "You said there would be snacks here!!"),
		talk("ralsei", 27, "C-calm down, Susie!\n(Kris, you DID tell her that...)"),
		talk("ralsei", 6, "This is Greenflower Zone. It's really pretty, don't you think?"),
		talk("susie", 11, "Maybe it would be if it had green flowers.\n(Or some snacks...)"),
		talk("susie", 8, "Hey, no way! Are those onion rings in front of us?"),
		talk("ralsei", 24, "UH, NO?????"),
		talk("ralsei", 5, "Let's not lose focus.\nKris, Susie, do you see those blue robots?"),
		talk("susie", 3, "Yeah. I'm not blind, unlike whatever loser coined \"Greenflower\"."),
		talk("ralsei", 9, "Susie, those robots are holding animals captive within their shells."),
		talk("susie", 15, "Wait, what the hell?????"),
		talk("susie", 6, "No snacks. A rubbish name. AND hostages??? This place sucks!!"),
		talk("ralsei", 7, "The situation is pretty awful. But we can help."),
		talk("ralsei", 6, "It turns out that breaking the robots open is a piece of cake."),
		talk("ralsei", 12, "If we're careful, we can set the animals free!"),
		talk("susie", 7, "Oh, I get to break stuff here? Why didn't you say so earlier?"),
		returnPartners,
		TICRATE/3,
		talk("susie", 34, "What are we waiting for? Let's save these losers.")
	}
})

DB.AddEvent({
	tag = "outro",
	zone = "greenflower",
	act = 1,
	flags = DF.STASIS,
	preCondition = function()
		return gamecomplete or vars.beatBrak
	end,
	script = {
		spawnPartners,
		TICRATE/2,
		function()
			S_StartSound(nil, sfx_dcphon)
			return true
		end,
		talk("mute", nil, "..."),
		talk(nil, nil, "Your phone is ringing?"),
		talk(nil, nil, "BEEP!"),
		talk("toriel", 8, "-ris! KRIS! Oh, thank god I got through!"),
		talk("toriel", 7, "Where have you been? Do you have any idea how worried I've been?"),
		talk("toriel", 7, "It's almost night time!"),
		talk(nil, nil, "..."),
		talk("toriel", 1, "..."),
		talk("toriel", 3, "Y-Yes, it's... good to hear your voice too, sorry I shouted."),
		talk("toriel", 7, "Where are you? You've been gone for hours!"),
		talk("toriel", 7, "Hm? With friends? Why didn't you call me?"),
		talk("toriel", 5, "Well, you're safe, that's all that matters."),
		talk("toriel", 3, "You're staying over with your friends then? Is it Susie?"),
		talk("susie", 11, "Krissss, pass the blunt!"),
		talk("toriel", 16, "...?"),
		talk("susie", 3, "Stop hogging all the girls, Kris!"),
		talk("toriel", 13, "Ahah, that old trick, hello Susie, I can hear you."),
		talk("toriel", 4, "Kris, can you put me on speaker?"),
		talk(nil, nil, "Bleep!"),
		talk("toriel", 1, "Now listen to me, Susie."),
		talk("susie", 36, "Y-Yes, ma'am!"),
		talk("toriel", 1, "You make sure Kris gets to school nice and safe tomorrow, alright?"),
		talk("toriel", 2, "It's a school night!"),
		talk("susie", 36, "Yes, ma'am, of course!"),
		talk("toriel", 4, "Alright then you two, take care!"),
		talk(nil, nil, "(click...)"),
		talk("susie", 25, "..."),
		talk("ralsei", 5, "..."),
		talk("susie", 14, "You've got a good mother, Kris."),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "switching",
	zone = "greenflower",
	act = 1,
	preCondition = function()
		return not (gamecomplete or vars.beatBrak)
	end,
	script = {
		10*TICRATE,
		talk("ralsei", 21, "Oh yeah, Kris! I have some important information for you."),
		talk("ralsei", 3, "You can change your character by pressing \130Custom 1\128 or \130Custom 3\128."),
		talk("ralsei", 17, "Try experimenting with our abilities in different situations!"),
		talk("susie", 3, "Or just play as me the whole time.")
	}
})

DB.AddEvent({
	tag = "house",
	zone = "greenflower",
	act = 1,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return DB.current == nil and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 27
	end,
	script = {
		talk("ralsei", 23, "Hey, isn't this RedXVI's house?"),
		talk("ralsei", 35, "I heard it can take you to hell... *shudder*"),
		talk("susie", 18, "LETS GO LETS GO LETS GO LETS GO LETS GO LETS GO LETS GO LETS GO\127"),
		talk("spamton", 1, "(HEY!! Can't a guy get some [$4.99 Per Night] REST???)")
	}
})

DB.AddEvent({
	tag = "picnic",
	zone = "greenflower",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and (mo.subsector.sector.tag == 435 or mo.subsector.sector.tag == 436)
	end,
	script = {
		talk("ralsei", 26, "Oh hey guys, look! We can have a picnic here!"),
		talk("ralsei", 3, "It's got a wonderful view, the air is super fresh and..."),
		talk("ralsei", 7, "...huh, that's odd. I swear I had it somewhere."),
		talk("susie", 36, "Had what?"),
		talk("ralsei", 19, "Oh y'know, just some leftover cauldron cake to share together!"),
		talk("susie", 21, "Did... it look like a big, white bag with tons of stickers on it?"),
		talk("ralsei", 21, "Yeah! Why do you ask?"),
		talk("susie", 7, "..."),
		talk("susie", 4, "...no reason."),
	}
})

DB.AddEvent({
	tag = "picnicEgg",
	zone = "greenflower",
	act = 2,
	hookType = "MobjDeath",
	hookArg = MT_EGGMAN_BOX,
	condition = function(mobj)
		return DB.current == nil and mobj.subsector and mobj.subsector.sector
		and mobj.subsector.sector.tag == 435
	end,
	script = {
		TICRATE*3/2,
		talk("susie", 6, "Argh WHO BROUGHT RAW EGGS TO A PICNIC!?"),
	}
})

// ---------------------------------
// ---------------------------------
// TECHNO HILL
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "gatorade",
	zone = "techno hill",
	act = 1,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.starpostnum == 1 --Entering the first factory
	end,
	script = {
		TICRATE,
		talk("susie", 8, "Holy crap, a Gatorade factory!?"),
		talk("ralsei", 13, "Susie, wait! If you fall in, you'll turn purple!"),
		talk("susie", 2, "....?"),
		talk("ralsei", 27, "O-oh, right.")
	}
})

//Please rewrite if it sounds stupid
DB.AddEvent({
	tag = "goop",
	zone = "techno hill",
	act = 2,
	hookType = "ThinkFrame",
	preCondition = function()
		vars.wentInGoop = 0
		return true
	end,
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		if P_IsObjectInGoop(mo)
			vars.wentInGoop = 1
		elseif vars.wentInGoop
			return true
		end
	end,
	script = {
		TICRATE,
		talk("ralsei", 5, "I really wish we didn't have to jump into the goop..."),
		talk("susie", 10, "What are you talking about?? This stuff tastes GREAT!"),
		talk("ralsei", 24, "You've been eating it!?"),
		talk("susie", 3, "You haven't?"),
	}
})

// ---------------------------------
// ---------------------------------
// DEEP SEA
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "swim",
	zone = "deep sea",
	act = 1,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 12, "W-water?"),
		talk("susie", 13, "..."),
		talk("susie", 14, "Uh, Kris, I've got a bit of a confession to make."),
		talk("susie", 4, "I uh... Can't swim."),
		talk("ralsei", 3, "Oh Susie, don't worry, I can't swim either!"),
		talk("susie", 7, "Kris, you have the biggest grin on your face, what?"),
		talk("susie", 8, "You can swim?"),
		talk("ralsei", 20, "Oh you can, Kris? That's wonderful!"),
		talk("ralsei", 21, "I bet you can swim very well, I'd like to see you do breast stroke!"),
		talk("ralsei", 6, "...hm? What's wrong, Kris? You're suddenly pale."),
		talk("susie", 3, "Hah, they're just bluffing."),
		talk("susie", 3, "You can't swim just like the rest of us dweebs!"),
		talk("susie", 36, "...H-Hey, it's just a joke, no need to look so ticked."),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "eggmanMonitor",
	zone = "deep sea",
	act = 1,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 763 and #mo.subsector.sector.lines == 4
	end,
	script = {
		TICRATE,
		talk("susie", 12, "..."),
		talk("susie", 2, "I was expecting something to be up here..."),
		talk("susie", 1, "But now I'm just disappointed."),
	}
})

DB.AddEvent({
	tag = "cirno",
	zone = "deep sea",
	act = 1,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 479
	end,
	script = {
		talk("susie", 7, "Whoa! Is that Chillno?"),
		talk("susie", 8, "Kris, you never told me she was in this game!"),
		talk("ralsei", 21, "Who?"),
		talk("susie", 10, "Y'know! The funky lil' ice idiot! From Toes!"),
		talk("ralsei", 23, "(Toes??)"),
		talk("susie", 36, "Hmm...? What's that Kris?"),
		talk("susie", 36, "I'm pronouncing her name wrong?"),
		talk("susie", 3, "Alright smarty pants, then how DO you pronounce it?"),
		talk(nil, nil, "..."),
		talk("susie", 7, "Oh!"),
		talk("susie", 4, "Well I feel foolish."),
		talk("ralsei", 23, "?????"),
	}
})

DB.AddEvent({
	tag = "spindashSwitch",
	zone = "deep sea",
	act = 2,
	hookType = "ThinkFrame",
	preCondition = function()
		vars.spindashSwitch = false
		return true
	end,
	condition = function()
		local player = consoleplayer
		if not (player and player.skin == DC.Kris.GetSkinIndex() and player.deltaChars)
			vars.spindashSwitch = false
			return
		end
		
		if player.deltaChars.flags & DC.FLAGS.SPINDASH
			vars.spindashSwitch = true
		elseif vars.spindashSwitch
			return true
		end
	end,
	script = {
		TICRATE/2,
		talk("susie", 12, "..."),
		talk("susie", 2, "Kris."),
		talk("susie", 7, "Did you just open that door with your BUTT?"),
	}
})

// ---------------------------------
// ---------------------------------
// DEEP SEA 3 BOSS FIGHT
// ---------------------------------
// ---------------------------------

function DB.ConvertRouxlsStates(rouxls)
	if (not rouxls) == (not vars.rouxlsStates)
		return // states are already set this way
	end
	
	DC.DebugPrint("Converting Rouxls states...")
	
	vars.rouxlsStates = rouxls
	
	local bossSprite
	local fakeSprite
	
	if rouxls
		bossSprite = SPR_ROUX
		fakeSprite = SPR_STWK
		states[S_EGGMOBILE3_STND].frame = B
		states[S_EGGMOBILE3_SHOCK].frame = B
		states[S_EGGMOBILE3_FLEE2].frame = K
		
		states[S_BOSSSEBH1].sprite = SPR_ROUX
		states[S_BOSSSEBH1].frame = L
		states[S_BOSSSEBH2].sprite = SPR_ROUX
		states[S_BOSSSEBH2].frame = L
	else
		bossSprite = SPR_EGGO
		fakeSprite = SPR_FAKE
		states[S_EGGMOBILE3_STND].frame = A
		states[S_EGGMOBILE3_SHOCK].frame = FF_ANIMATE
		states[S_EGGMOBILE3_FLEE2].frame = L
		
		states[S_BOSSSEBH1].sprite = SPR_SEBH
		states[S_BOSSSEBH1].frame = A
		states[S_BOSSSEBH2].sprite = SPR_SEBH
		states[S_BOSSSEBH2].frame = B
	end
	
	for state = S_EGGMOBILE3_STND, S_EGGMOBILE3_FLEE2
		states[state].sprite = bossSprite
	end
	
	for state = S_FAKEMOBILE_INIT, S_FAKEMOBILE_DIE1
		states[state].sprite = fakeSprite
	end
end

DC.AddHook("MapChange", do
	DB.ConvertRouxlsStates(false)
end)

DB.AddEvent({
	tag = "rouxlsStart",
	zone = "Deep Sea",
	act = 3,
	flags = DF.STASIS|DF.FLIP_BOX,
	preCondition = function()
		local boss
		
		for mo in mobjs.iterate()
			if mo.type == MT_EGGMOBILE3
				boss = mo
				boss.flags = $ | MF_NOTHINK
				break
			end
		end
		
		vars.boss = boss
		
		return boss
	end,
	script = {
		function()
			local boss = vars.boss
			
			wait(8)
			
			S_StartSound(nil, sfx_dceggl)
			boss.state = S_EGGMOBILE3_ROFL
			boss.tics = 3*TICRATE/2
			while boss.tics > 0
				boss.tics = $ - 1
				if boss.tics & 1
					boss.frame = $ ^^ 1
				end
				yield()
			end
			boss.state = S_SPAWNSTATE
			return true
		end,
		talk("eggman", 1, "Hohoho! Brace yourselves, lamebrains, for this time I hav\127e"),
		function()
			local boss = vars.boss
			
			S_StopMusic()
			S_StartSound(nil, sfx_rouxls)
			RouxlsLaser(boss.x, boss.y, boss.z, function()
				local eggman = P_SpawnMobjFromMobj(boss, 0, 0, 64*FRACUNIT, MT_GFZFLOWER1)
				eggman.state = S_DC_EGGMANCOLLAPSE
				P_InstaThrust(eggman, boss.angle + ANGLE_135, 5*boss.scale)
				S_StartSound(nil, sfx_dcdmg)
				DB.ConvertRouxlsStates(true)
				P_SetMobjStateNF(boss, boss.state)
			end)
			S_ChangeMusic("ROUXLS", true)
			return true
		end,
		talk("rk", 3, "\002GE HA HA HA\000!\nART THOU SURPRISETH, WORMES?"),
		talk("susie", 6, "Oh god not THIS GUY AGAIN???"),
		talk("rk", 5, "Thyne future rulere of this kingdome hath granted me the honore..."),
		talk("rk", 1, "...to strike thee downst!!"),
		talk("ralsei", 22, "Uh-- h-he did??"),
		talk("rk", 7, "..."),
		talk("rk", 6, "Yes."),
		talk("ralsei", 23, "It doesn't really look like he consented to th\127at"),
		talk("rk", 2, "AHA! BUT I SHALT PROVETH MY SUPERIORITY,"),
		talk("rk", 2, "AND HE SHALT ACKNOWLEDGETH ME AS HIS LEFTE HANDE MAN!"),
		talk("rk", 3, "Prepareth to be overwhelmed by the ULTIMATE minion!"),
		talk("rk", 4, "Toil, wormes!"),
		function()
			S_ChangeMusic("ROUXLB", true)
			vars.boss.flags = $ & ~MF_NOTHINK
			return true
		end
	}
})

DB.AddEvent({
	tag = "rouxlsStart2",
	save = false,
	zone = "Deep Sea",
	act = 3,
	flags = DF.STASIS|DF.FLIP_BOX,
	preCondition = function()
		return DB.seenDialogue["rouxlsStart"]
	end,
	script = {
		function()
			local boss = vars.boss
			
			S_FadeOutStopMusic(MUSICRATE, player)
			S_StartSound(nil, sfx_rouxls)
			RouxlsLaser(boss.x, boss.y, boss.z, function()
				local eggman = P_SpawnMobjFromMobj(boss, 0, 0, 64*FRACUNIT, MT_GFZFLOWER1)
				eggman.state = S_DC_EGGMANCOLLAPSE
				P_InstaThrust(eggman, boss.angle + ANGLE_135, 5*boss.scale)
				S_StartSound(nil, sfx_dcdmg)
				DB.ConvertRouxlsStates(true)
				P_SetMobjStateNF(boss, boss.state)
			end)
			
			S_ChangeMusic("ROUXLB", true)
			vars.boss.flags = $ & ~MF_NOTHINK
			return true
		end,
	}
})

DB.AddEvent({
	tag = "rouxlsDefeat",
	save = false,
	zone = "deep sea",
	act = 3,
	hookType = "BossDeath",
	hookArg = MT_EGGMOBILE3,
	flags = DF.FLIP_BOX,
	condition = function(mo)
		if mo.sprite == SPR_ROUX
			local statue = P_SpawnMobjFromMobj(mo, 0, 0, 23*FRACUNIT, MT_GARGOYLE)
			statue.sprite = SPR_ROUX
			statue.frame = A // it should be A anyway but let's make sure
			statue.angle = mo.angle
			statue.height = 80 * statue.scale
			S_StopMusic()
			return true
		end
		return false
	end,
	script = {
		talk("rk", 6, "\003GOD"),
		talk("rk", 6, "\003DAMMIT"),
		function()
			local map = mapheaderinfo[gamemap]
			if not map
				return false
			end
			if map and map.muspostbossname
				S_ChangeMusic(map.muspostbossname, true, nil, 0x8000 | (map.muspostbosstrack & 0x0FFF), // WHY ARE THERE NO CONSTANTS FOR THESE
					map.muspostbosspos, 0, map.muspostbossfadein)
			else
				P_RestoreMusic()
			end
			return true
		end
	}
})

DB.AddEvent({
	tag = "starwalker",
	zone = "deep sea",
	act = 3,
	hookType = "TouchSpecial",
	hookArg = MT_FAKEMOBILE,
	condition = function(item, mo)
		return item.sprite == SPR_STWK
	end,
	script = {
		talk(nil, nil, "These \130tubes\128 are \130Pissing\128 me off..."),
		talk(nil, nil, "I'm the original   \130Starwalker\128")
	}
})

// ---------------------------------
// ---------------------------------
// CASTLE EGGMAN
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "moss",
	zone = "castle eggman",
	act = 1,
	hookType = "TouchSpecial",
	hookArg = MT_DC_BANANA,
	flags = DF.STASIS,
	preCondition = function()
		if vars.cezmoss then return end
		local save = P_SpawnMobj(2404*FRACUNIT, 8744*FRACUNIT, 4000*FRACUNIT, MT_DC_BANANA)
		if save and save.valid
			save.state = S_DC_SAVE
			return true
		end
	end,
	condition = function(save, mo)
		A_ForceStop(mo)
		return true
	end,
	script = {
		talk(nil, nil, "You found the \131[Moss]\128!"),
		talk(nil, nil, "Your title has been upgraded to Moss Connoisseur."),
		talk("susie", 3, "Oh HELL yeah!! This is AWESOME!!"),
		talk("susie", 8, "HEY, SAVE SOME FOR ME KRIS!!"),
		talk("ralsei", 4, "..."),
		talk("ralsei", 18, "...I'm glad you two have something to bond over!"),
		function()
			vars.cezmoss = true
			return true
		end
	}
})

DB.AddEvent({
	tag = "bridge1",
	zone = "castle eggman",
	act = 1,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		if mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 161
			vars.bridgeSequence = true
			return true
		end
	end,
	script = {
		TICRATE,
		talk("ralsei", 13, "The ceiling is falling down...? And the walls, and\127"),
		talk("susie", 28, "Oh crap, the whole bridge is collapsing! Move your ass, Kris!"),
		talk("susie", 6, "RUN! HOLD [X] OR WHATEVER!"),
	}
})

DB.AddEvent({
	tag = "bridge2",
	zone = "castle eggman",
	act = 1,
	hookType = "KeyDown",
	preCondition = function()
		vars.bridgeSequence = false
		return true
	end,
	condition = function(keyevent)
		local player = consoleplayer
		local mo = player and player.mo
		return vars.bridgeSequence and DB.current == nil and keyevent.name == "x"
	end,
	script = {
		talk("susie", 18, "KRIS WHAT ARE YOU DOING I DIDN'T MEAN LITERALLY YOU DORK!"),
	}
})

DB.AddEvent({
	tag = "castle",
	zone = "castle eggman",
	act = 2,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 5, "Tch, a castle again? This guy sucks!"),
		talk("ralsei", 36, "W-What's wrong with castles?"),
		talk("susie", 20, "HE'S STEALING OUR THING!!"),
		talk("susie", 19, "He better not have any stupidly cool personalized rooms!!"),
		talk("ralsei", 35, "S-Stupid?"),
		talk("susie", 22, "Aaagh whatever, bet he doesn't have cauldron cooking though."),
		talk("susie", 10, "We've got him beat there, Ralsei."),
		returnPartners,
	}
})

// ---------------------------------
// ---------------------------------
// CASTLE EGGMAN 3 BOSS FIGHT (You're welcome LD)
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "allSpikeBalls",
	save = false,
	zone = "castle eggman",
	act = 3,
	hookType = "SusieDestructable",
	preCondition = function(item)
		vars.spikeBalls = 0
		for mo in mobjs.iterate()
			if mo.type == MT_EGGMOBILE4_MACE
				vars.spikeBalls = $ + 1
			end
		end
		return vars.spikeBalls
	end,
	condition = function(item)
		if item.type == MT_EGGMOBILE4_MACE
			vars.spikeBalls = $ - 1
			if not vars.spikeBalls
				for mo in mobjs.iterate()
					if mo.type == MT_EGGMOBILE4
						vars.boss = mo
						mo.flags = $ | MF_NOTHINK
						return true
					end
				end
				vars.spikeBalls = -1
			end
		end
	end,
	script = {
		function()
			S_FadeOutStopMusic(MUSICRATE)
			return true
		end,
		TICRATE,
		talk("eggman", 1, "Ummm... I think... I think I'll just leave."),
		function()
			local boss = vars.boss
			boss.flags = $ &~ MF_NOTHINK
			
			P_KillMobj(boss, consoleplayer.mo, consoleplayer.mo)
			boss.state = S_EGGMOBILE4_DIE4
			boss.flags2 = $ &~ MF2_FRET
			
			P_RestoreMusic(consoleplayer)
			return true
		end
	}
})

DB.AddEvent({
	tag = "allEggRobos",
	zone = "castle eggman",
	act = 3,
	hookType = "SusieDestructable",
	preCondition = function(item)
		vars.eggRobos = 0
		for mo in mobjs.iterate()
			if mo.type == MT_EGGROBO1
				vars.eggRobos = $ + 1
			end
		end
		return vars.eggRobos
	end,
	condition = function(item)
		if item.type == MT_EGGROBO1
			vars.eggRobos = $ - 1
			return not vars.eggRobos
		end
	end,
	script = {
		talk("ralsei", 23, "Can we... fight the doctor now?"),
		talk("susie", 4, "Huff... huff... sure, give me..."),
		talk("susie", 4, "Huff... give me a sec..."),
	}
})

DB.AddEvent({
	tag = "hitEggRobo5",
	zone = "castle eggman",
	act = 3,
	hookType = "TouchSpecial",
	hookArg = MT_EGGROBO1,
	condition = function(mobj, mo)
		if DB.current ~= nil or mo.skin ~= DC.Susie.SKIN then return end
		vars.hitEggRobo = $ and $ + 1 or 1
		return vars.hitEggRobo >= 5
	end,
	script = {
		talk("susie", 18, "JUST HIT THEM, LET ME HIT THEM OUGHH!"),
	}
})

DB.AddEvent({
	tag = "hitEggRobo1",
	zone = "castle eggman",
	act = 3,
	hookType = "TouchSpecial",
	hookArg = MT_EGGROBO1,
	condition = function()
		return vars.hitEggRobo == 1
	end,
	script = {
		talk("susie", 6, "HOW??!?"),
	}
})

DB.AddEvent({
	tag = "hitEggRobo2",
	zone = "castle eggman",
	act = 3,
	hookType = "TouchSpecial",
	hookArg = MT_EGGROBO1,
	condition = function()
		return vars.hitEggRobo == 2
	end,
	script = {
		talk("susie", 6, "STOP IT!"),
	}
})

DB.AddEvent({
	tag = "hitEggRobo3",
	zone = "castle eggman",
	act = 3,
	hookType = "TouchSpecial",
	hookArg = MT_EGGROBO1,
	condition = function()
		return vars.hitEggRobo == 3
	end,
	script = {
		talk("susie", 6, "OUGHH!"),
	}
})

DB.AddEvent({
	tag = "hitEggRobo4",
	zone = "castle eggman",
	act = 3,
	hookType = "TouchSpecial",
	hookArg = MT_EGGROBO1,
	condition = function()
		return vars.hitEggRobo == 4
	end,
	script = {
		talk("susie", 18, "STOP IT STOP IT STOP IT!"),
	}
})

DB.AddEvent({
	tag = "madEggRobo",
	zone = "castle eggman",
	act = 3,
	hookType = "SusieDestructable",
	condition = function(item)
		return DB.current == nil
		and vars.hitEggRobo and vars.hitEggRobo >= 5 and item.type == MT_EGGROBO1
	end,
	script = {
		talk("susie", 26, "WAS THAT SO HARD!?"),
	}
})

DB.AddEvent({
	tag = "revengeEggRobo",
	zone = "castle eggman",
	act = 3,
	hookType = "SusieDestructable",
	condition = function(item)
		return DB.current == nil
		and vars.hitEggRobo and item.type == MT_EGGROBO1
	end,
	script = {
		talk("susie", 28, "Not so tough now are ya? ARE YA??!!"),
	}
})

DB.AddEvent({
	tag = "oneEggRobo",
	zone = "castle eggman",
	act = 3,
	hookType = "SusieDestructable",
	condition = function(item)
		return DB.current == nil
		and item.type == MT_EGGROBO1
	end,
	script = {
		talk("susie", 35, "OUTTA THE WAY, SUCKER!"),
	}
})

// ---------------------------------
// ---------------------------------
// ARID CANYON
// ---------------------------------
// ---------------------------------	

DB.AddEvent({
	tag = "heatStroke",
	zone = "arid canyon",
	act = 1,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("ralsei", 6, "The desert... It's uh, pretty hot here..."),
		talk("susie", 12, "...You know, you could just. Take your scarf off."),
		talk("ralsei", 2, "But it's... comfy."),
		talk("ralsei", 2, "Makes me feel happy, and warm, and... dizzy, and lightheaded..."),
		talk("susie", 13, "...Okay, Ralsei is having a heat stroke. We better move, Kris."),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "cacti75",
	zone = "arid canyon",
	hookType = "SusieDestructable",
	preCondition = function(item)
		vars.destroyedcacti = $ or 0
		return vars.destroyedcacti < 75
	end,
	condition = function(item)
		if DC.MobjIsCactus(item)
			vars.destroyedcacti = $ + 1
			return vars.destroyedcacti == 75
		end
	end,
	script = {
		TICRATE,
		talk("susie", 12, "...?"),
		talk("ralsei", 6, "What's the matter, Susie?"),
		talk("ralsei", 19, "Are you getting tired from smashing all those cacti?"),
		talk("susie", 12, "No... it's, uh..."),
		talk("susie", 13, "Anyone uhh... A-Anyone hear that?"),
		talk("ralsei", 5, "...? Hear what?"),
		talk("susie", 13, "Almost like... Angry... Spanish squealing?"),
		talk("ralsei", 27, "You're probably exhausted from swinging about so much."),
		talk("ralsei", 3, "It's very hot here, be careful not to get heat stroke!"),
		talk("ralsei", 17, "Remember to hydrate!"),
		talk("susie", 1, "..."),
		talk("susie", 2, "...?"),
	}
})

DB.AddEvent({
	tag = "cacti1",
	zone = "arid canyon",
	hookType = "SusieDestructable",
	preCondition = function()
		return not vars.destroyedcacti
	end,
	condition = function()
		return vars.destroyedcacti
	end,
	script = {
		TICRATE,
		talk("susie", 6, "HA! TAKE THAT YOU PRICKLY PICKLE!!"),
	}
})

DB.AddEvent({
	tag = "door",
	zone = "arid canyon",
	act = 2,
	hookType = "SusieDestructable",
	condition = function(item)
		return item.type == MT_SALOONDOOR
	end,
	script = {
		TICRATE,
		talk("susie", 6, "Minecarts are for nerds!"),
	}
})

DB.AddEvent({
	tag = "shortMinecart",
	zone = "arid canyon",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		if player.powers[pw_carry] == CR_MINECART
			local tag = DC.GetMinecartTag(player)
			DC.DebugPrint("Minecart tag: "..tag)
			return tag == 98 or tag == 3300
		end
	end,
	script = {
		talk("ralsei", 26, "Weeeeeeeeeeeeeeee!!"),
		function(player)
			while player.powers[pw_carry] == CR_MINECART
				yield()
			end
			return player.playerstate == PST_LIVE
		end,
		talk("ralsei", 22, "oh")
	}
})

DB.AddEvent({
	tag = "finalMinecart",
	zone = "arid canyon",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.starpostnum == 5
	end,
	script = {
		TICRATE,
		talk("susie", 5, "Another minecart? What a drag, Kris!"),
		talk("susie", 3, "Ram one of those TNT barrels, that'll get the excitement going!"),
		talk("ralsei", 23, "D-Don't actually do that, Kris..."),
		function()
			vars.finalMinecart = true
			return true
		end
	}
})

DB.AddEvent({
	tag = "tntBarrel",
	zone = "arid canyon",
	act = 2,
	hookType = "MobjDamage",
	hookArg = MT_PLAYER,
	preCondition = function()
		vars.finalMinecart = false
		return true
	end,
	condition = function(mo, inf)
		return vars.finalMinecart and DB.current == nil
		and inf and inf.type == MT_TNTBARREL
	end,
	script = {
		TICRATE,
		talk("susie", 18, "YEEEEAAAAAHHHHH!!!!!!"),
		talk("ralsei", 33, "COME ON, THAT HURT!"),
		talk("susie", 3, "Wimp."),
	}
})

DB.AddEvent({
	tag = "banana",
	zone = "arid canyon",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 2510 and #mo.subsector.sector.lines == 4
	end,
	script = {
		TICRATE*2,
		talk("susie", 12, "A... banana?"),
		talk("queen", 1, "Kris Get The Banana"),
		talk("susie", 18, "WHERE THE HELL???????"),
	}
})

DB.AddEvent({
	tag = "potassium",
	zone = "arid canyon",
	act = 2,
	hookType = "TouchSpecial",
	hookArg = MT_DC_BANANA,
	preCondition = function()
		return P_SpawnMobj(1006*FRACUNIT, 3838*FRACUNIT, 3528*FRACUNIT, MT_DC_BANANA)
	end,
	script = {
		TICRATE,
		talk("queen", 1, "Potassium"),
	}
})

DB.AddEvent({
	tag = "shortcutMinecart",
	zone = "arid canyon",
	act = 2,
	hookType = "ThinkFrame",
	preCondition = function()
		vars.shortcut = 0
		return true
	end,
	condition = function()
		local player = consoleplayer
		if vars.shortcut
			vars.shortcut = $ - 1
			return player.mo and player.mo.y > 5920*FRACUNIT
		elseif player.powers[pw_carry] == CR_MINECART
		and DC.ButtonDown(player, BT_JUMP)
		and (DC.GetMinecartTag(player) == 2019 or DC.GetMinecartTag(player) == 2021)
			vars.shortcut = TICRATE
		end
	end,
	script = {
		talk("ralsei", 33, "KRIS WHAT ARE YOU DOING!?"),
		function(player)
			while not (player.mo and P_IsObjectOnGround(player.mo)
			and player.playerstate == PST_LIVE)
				yield()
			end
			return true
		end,
		talk("susie", 18, "WOOHOO TIME SAVE BABY!"),
	}
})

DB.AddEvent({
	tag = "unfinishedMinecart",
	zone = "arid canyon",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return DB.current == nil
		and consoleplayer.powers[pw_carry] == CR_MINECART
		and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 2577 and #mo.subsector.sector.lines == 4
	end,
	script = {
		TICRATE,
		talk("susie", 18, "What is it with all these roller coasters being unfinished?!!?!?"),
	}
})

DB.AddEvent({
	tag = "unusedSignpost",
	zone = "arid canyon",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 1881 and #mo.subsector.sector.lines == 8
	end,
	script = {
		talk("susie", 12, "Huh? Why's this thing in here?"),
		talk("ralsei", 21, "You mean the signpost? I assume that evil scientist put it here."),
		talk("ralsei", 20, "His face has been on all of them, after all."),
		talk("susie", 2, "No, I mean why is it HERE."),
		talk("susie", 5, "What kind of moron hides a signpost behind a signpost?"),
		talk("ralsei", 21, "To be fair, he does seem a little on the loony side."),
		talk("susie", 18, "YOU THINK?"),
		talk("susie", 6, "He straight up trapped us in an arena and made us PUSH BUTTONS."),
		talk("susie", 6, "In a SPECIFIC ORDER."),
		talk("susie", 18, "I HATE pushing buttons in a specific order!!"),
	}
})

DB.AddEvent({
	tag = "fang",
	zone = "arid canyon",
	act = 3,
	flags = DF.STASIS|DF.FLIP_BOX,
	preCondition = function()
		for mo in mobjs.iterate()
			if mo.type == MT_FANG
				vars.boss = mo
				return true
			end
		end
	end,
	script = {
		function()
			local boss = vars.boss
			S_FadeOutStopMusic(4*MUSICRATE)
			while boss.state < S_FANG_INTRO9
				yield()
			end
			boss.tics = -1
			return true
		end,
		talk("fang", 1, "Ahah! That fool Robotnik just LET me aboard, now I can\127"),
		talk("susie", 3, "Yo"),
		function()
			local boss = vars.boss
			boss.state = S_FANG_INTRO10
			boss.tics = TICRATE/2
			while boss.state < S_FANG_INTRO12
				yield()
			end
			boss.tics = -1
			return true
		end,
		TICRATE/2,
		talk("fang", 2, "Achk! The doc actually had the brains to hire some REAL muscle, eh?"),
		talk("fang", 1, "Well, you're too late!"),
		talk("fang", 1, "I ain't about to blow this gig to some barney-lookin' lacky!!"),
		talk("susie", 19, "...What'd you call me?"),
		talk("fang", 2, "-Gulp-"),
		function()
			S_ChangeMusic("VSFANG", true, nil, nil, MUSICRATE*1457/250)
			vars.boss.tics = 1
			return true
		end,
	}
})

DB.AddEvent({
	tag = "fang",
	zone = "arid canyon",
	act = 3,
	flags = DF.FLIP_BOX,
	hookType = "MobjDeath",
	hookArg = MT_FANG,
	condition = function(mobj, inf, mo)
		if mo and mo.player and mo.player.skin == DC.Susie.GetSkinIndex()
			S_StartSound(nil, sfx_fangus)
			return true
		end
	end,
	script = {
		talk("fang", 2, "AAAH HOO HOO HOOEY"),
	}
})

// ---------------------------------
// ---------------------------------
// RED VOLCANO
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "lava",
	zone = "red volcano",
	act = 1,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 8, "Oh sick, a volcano!! I always wondered how hot lava is."),
		talk("ralsei", 23, "Uh... enough to melt pretty much everything?"),
		talk("susie", 6, "The hell you mean pretty much?!"),
		talk("susie", 11, "There's gotta be something lava won't melt. Like water."),
		talk("susie", 1, "..."),
		talk("susie", 2, "wait."),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "bird",
	zone = "red volcano",
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.powers[pw_carry] == CR_PTERABYTE
	end,
	script = {
		talk("susie", 6, "God I hate birds!!"),
		talk("susie", 18, "First Berdly and now THIS?!"),
	}
})

DB.AddEvent({
	tag = "lavaRock",
	zone = "red volcano",
	hookType = "ThinkFrame",
	condition = function()
		local mo = consoleplayer.mo
		local rock = mo and mo.tracer
		return consoleplayer.powers[pw_carry] == CR_ROLLOUT
		and rock and rock.eflags & MFE_TOUCHLAVA
	end,
	script = {
		TICRATE,
		talk("ralsei", 22, "C-Careful, Kris! We don't wanna slip and fall into the lava!"),
		talk("susie", 3, "CHECK THIS OUT KRIS!! NO HANDS!!"),
	}
})

DB.AddEvent({
	tag = "rocket",
	zone = "red volcano",
	act = 1,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 1668 and #mo.subsector.sector.lines == 6
	end,
	script = {
		talk("ralsei", 21, "Oh wow... Is that a rocket ship? Maybe we can\127"),
		talk("susie", 18, "MINE MINE MINE MINE MINE MINE MINE!!"),
	}
})

// ---------------------------------
// ---------------------------------
// EGG ROCK
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "spaceStation",
	zone = "egg rock",
	act = 1,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("ralsei", 21, "We made it guys! We're in that evil man's base."),
		talk("susie", 11, "A space station, huh? Gotta admit that's pretty cool."),
		talk("susie", 3, "Too bad I'm gonna crash it down to the ground anyways!!"),
		talk("ralsei", 32, "Can... we get out before doing that though."),
		talk("susie", 3, "WE'LL SEE!!"),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "space1",
	save = false,
	zone = "egg rock",
	hookType = "ThinkFrame",
	preCondition = function()
		if vars.wentToSpace == nil
		or vars.wentToSpace < 2
			vars.wentToSpace = 0
		end
		return not vars.wentToSpace
	end,
	condition = function()
		local player = consoleplayer
		local mo = player.mo
		
		if P_InSpaceSector(mo)
			local sector = mo.subsector.sector
			if sector.floorpic == "F_SKY1"
			or sector.ceilingpic == "F_SKY1"
				vars.wentToSpace = 1
				return true
			end
		end
	end,
	script = {
		TICRATE,
		talk("susie", 35, " ")
	}
})

DB.AddEvent({
	tag = "space2",
	save = false,
	zone = "egg rock",
	hookType = "ThinkFrame",
	preCondition = function()
		return vars.wentToSpace ~= 2
	end,
	condition = function()
		local player = consoleplayer
		local mo = player.mo
		
		if vars.wentToSpace ~= 1
			return
		end
		
		if not P_InSpaceSector(mo)
			local sector = mo.subsector.sector
			if sector.floorpic ~= "F_SKY1"
			and sector.ceilingpic ~= "F_SKY1"
				vars.wentToSpace = 2
				return true
			end
		end
	end,
	script = {
		TICRATE,
		talk("ralsei", 21, "Did you say something back there, Susie? I couldn't hear you."),
		talk("susie", 22, "Heh, trust me. You don't want me to repeat myself.")
	}
})

DB.AddEvent({
	tag = "vent",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 1034 and not player.powers[pw_carry]
	end,
	script = {
		TICRATE/2,
		talk("susie", 22, "Did you just vent?"),
		talk("ralsei", 22, "...What?"),
		talk("susie", 3, "Venting AND sequence breaking?"),
		talk("susie", 3, "Seems I've had quite the influence on you!"),
		talk("ralsei", 23, "(Is that a good thing or a bad thing?)"),
	}
})

DB.AddEvent({
	tag = "disco",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 573
	end,
	script = {
		TICRATE,
		talk("susie", 12, "??"),
		talk("susie", 18, "WHAT."),
	}
})

DB.AddEvent({
	tag = "discoEggman",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and (mo.subsector.sector.tag == 519 or mo.subsector.sector.tag == 521)
	end,
	script = {
		TICRATE/2,
		talk("susie", 18, "WHY THE HELL IS HE EVEN IN HERE?!"),
	}
})

DB.AddEvent({
	tag = "discoLeave",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 6108
	end,
	script = {
		TICRATE/2,
		talk("susie", 2, "...huh. Anyone else hear that..?"),
		talk("ralsei", 21, "Hear what, Susie?"),
		talk("susie", 1, "Nothing, just... Obnoxious music?"),
	}
})

DB.AddEvent({
	tag = "erz2Shortcut",
	zone = "egg rock",
	act = 2,
	hookType = "SusieBustSector",
	hookArg = 495,
	script = {
		function()
			vars.bustedERZ2Shortcut = true
			return true
		end,
		TICRATE/2,
		talk("susie", 7, "...?!"),
		function()
			if DB.seenDialogue["splitPath4"]
				DB.nextIndex = 100
			end
			return true
		end,
		talk("susie", 12, "What the--\010\127"),
		talk("susie", 26, "Kris... did you KNOW this moving walkway was here??"),
		talk("ralsei", 17, "That's SOME intuition, Kris!"),
		[100] = talk("susie", 8, "Oh hey, we really CAN skip all of that...")
	}
})

// ---------------------------------
// ---------------------------------
// EGG ROCK SPLIT PATH (You're welcome LD)
// ---------------------------------
// ---------------------------------

local splitpaths = {
	[C] = {137, 4}, --Start of split path
	[D] = {1956, 8}, --Start of split path (Alt)
	[R] = {322, 16}, --Right path
	[L] = {327, 13}, --Left path
	[B] = {1990, 4}, --Backtrack room
}

local checkSplitPath = function(path)
	local player = consoleplayer
	local mo = player and player.mo
	local sector = mo and mo.subsector and mo.subsector.sector
	
	return sector and #sector == splitpaths[path][1] and #sector.lines == splitpaths[path][2]
end

DB.AddEvent({
	tag = "splitPath0",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if not vars.splitPath and (checkSplitPath(C) or checkSplitPath(D))
			return true
		end
	end,
	script = {
		talk("ralsei", 20, "It seems that there's two paths we can take... Which way, guys?"),
		talk("susie", 3, "Left! That evil guy isn't right, so it MUST be left!"),
		talk("ralsei", 21, "...Your choice, Kris!"),
	}
})

DB.AddEvent({
	tag = "splitPathR1",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if not vars.splitPath and checkSplitPath(R)
			vars.splitPath = 1
			DB.CutCurrentEvents()
			return true
		end
	end,
	script = {
		talk("susie", 7, "Huh? Is my hunch not good enough?"),
		talk("ralsei", 17, "Don't worry Kris, this is definitely the right path!"),
		talk("susie", 13, "Ooh look at me, Mr\001.\000 Technical! Tch, left was the better choice."),
	}
})

DB.AddEvent({
	tag = "splitPathL1",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if not vars.splitPath and checkSplitPath(L)
			vars.splitPath = -1
			DB.CutCurrentEvents()
			return true
		end
	end,
	script = {
		talk("susie", 11, "Hm! Yes, this is definitely right!"),
		talk("ralsei", 6, "Um, Susie, this is left."),
		talk("susie", 3, "That's what I said, right?"),
		talk("ralsei", 24, "No, left!"),
		talk("susie", 12, "..."),
		talk("susie", 2, "...What?"),
	}
})

DB.AddEvent({
	tag = "splitPathR2",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if vars.splitPath == -1 and checkSplitPath(R)
			vars.splitPath = 2
			DB.CutCurrentEvents()
			return true
		end
	end,
	script = {
		talk("susie", 36, "Kris? Look! Come on, there are LASERS there!!"),
		talk("ralsei", 21, "It's okay Kris! It's a tough choice!"),
	}
})

DB.AddEvent({
	tag = "splitPathL2",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if vars.splitPath == 1 and checkSplitPath(L)
			vars.splitPath = -2
			DB.CutCurrentEvents()
			return true
		end
	end,
	script = {
		talk("ralsei", 21, "It's okay, Kris! It's a tough choice!"),
		talk("susie", 8, "Left? Are we going left!?"),
	}
})

DB.AddEvent({
	tag = "splitPath3",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if vars.splitPath == -2 and checkSplitPath(R)
			vars.splitPath = 3
			DB.CutCurrentEvents()
			return true
		elseif vars.splitPath == 2 and checkSplitPath(L)
			vars.splitPath = -3
			DB.CutCurrentEvents()
			return true
		end
	end,
	script = {
		talk("susie", 6, "KRIS! WE HAVE BUTTS TO KICK!!"),
		talk("susie", 18, "JUST PICK ONE I DON'T CARE ANYMORE!"),
	}
})

DB.AddEvent({
	tag = "splitPath4",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if vars.bustedERZ2Shortcut
		and not vars.splitPath
			return false
		end
		
		if (not vars.splitPath or abs(vars.splitPath) <= 3) and checkSplitPath(B)
			if not vars.splitPath or vars.splitPath > 0
				vars.splitPath = 4
			else
				vars.splitPath = -4
			end
			return not vars.bustedERZ2Shortcut
		end
	end,
	script = {
		talk("ralsei", 21, "Oh? Look at that, Kris! A way back to where we began!"),
		talk("susie", 28, "You're telling me we could have skipped ALL of that?!"),
		talk("susie", 18, "I should've smashed that stupid door down!!"),
	}
})

DB.AddEvent({
	tag = "splitPath5",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if vars.splitPath
			if abs(vars.splitPath) == 4 and (checkSplitPath(C) or checkSplitPath(D))
				return true
			elseif (vars.splitPath == 3 and checkSplitPath(L))
			or (vars.splitPath == -3 and checkSplitPath(R))
				vars.splitPath = -$
			end
		end
	end,
	script = {
		talk("susie", 12, "And... we're here again."),
		talk("susie", 2, "Kris, shouldn't we... go back?"),
	}
})

DB.AddEvent({
	tag = "splitPathD6",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if (vars.splitPath == 4 and checkSplitPath(L))
		or (vars.splitPath == -4 and checkSplitPath(R))
			vars.splitPath = 5
			DB.CutCurrentEvents()
			return true
		end
	end,
	script = {
		talk("susie", 1, "...well I mean, at least it's a different route."),
	}
})

DB.AddEvent({
	tag = "splitPathS6",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		if (vars.splitPath == -4 and checkSplitPath(L))
		or (vars.splitPath == 4 and checkSplitPath(R))
			vars.splitPath = 5
			DB.CutCurrentEvents()
			return true
		end
	end,
	script = {
		talk("susie", 18, "REALLY, KRIS?! LAP 2 ISN'T A THING HERE!!"),
	}
})

DB.AddEvent({
	tag = "splitPathD7",
	zone = "egg rock",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		return vars.splitPath == 6 and consoleplayer.exiting
	end,
	script = {
		talk("susie", 12, "...Why."),
	}
})

DB.AddEvent({
	tag = "splitPathS7",
	zone = "egg rock",
	act = 2,
	flags = DF.STASIS,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		
		if vars.splitPath == 5
		and mo and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 6108
			vars.splitPath = 6
		elseif vars.splitPath == 6 and checkSplitPath(B)
			vars.splitPath = 7
			return true
		end
	end,
	script = {
		function(player)
			local mo = player.mo
			if not mo then return true end
			
			local angle = (player.cmd.angleturn << 16) + ANGLE_90
			local diff = 72*mo.scale
			local zdiff = 256*mo.scale
			local skin = skins[DC.Susie.GetSkinIndex()]
			local susie = (player.skin == DC.Susie.GetSkinIndex() and mo)
			or P_SpawnMobjFromMobj(mo, P_ReturnThrustX(mo, angle, diff), P_ReturnThrustY(mo, angle, diff), zdiff, MT_GFZFLOWER1)
			
			A_ForceStop(mo)
			if susie == mo
				vars.susie = nil
			else
				susie.skin = skin.name
				susie.color = skin.prefcolor
				susie.angle = mo.angle
				susie.state = S_PLAY_FALL
				susie.shadowscale = FRACUNIT
				
				while not P_IsObjectOnGround(susie)
					susie.momz = $ + P_GetMobjGravity(susie)
					yield()
				end
				
				vars.susie = susie
			end
			
			susie.state = S_TAILSOVERLAY_EDGE
			S_StartSound(susie, sfx_dctail)
			
			return true
		end,
		TICRATE,
		talk("susie", 18, "NO."),
		talk("susie", 18, "WE ARE NOT DOING THIS A THIRD TIME."),
		talk("susie", 18, "WE ARE LEAVING."),
		function(player)
			local mo = player.mo
			if not mo then return true end
			local susie = vars.susie
			
			if susie
				S_StartSound(susie, sfx_statu2)
				vars.susietimer = TICRATE/2
				local targetx = mo.x + P_ReturnThrustX(mo, ANGLE_270, mo.radius*2)
				local targety = mo.y + P_ReturnThrustY(mo, ANGLE_270, mo.radius*2)
				while vars.susietimer
					P_TryMove(susie, susie.x - (susie.x - targetx)/vars.susietimer,
					susie.y - (susie.y - targety)/vars.susietimer, true)
					vars.susietimer = $ - 1
					yield()
				end
				
				S_StopSound(susie)
				DC.Ralsei.SpawnDamageEffect(player, susie)
			end
			
			while not player.exiting
				if mo.y <= -6016*FRACUNIT
					mo.angle = ANGLE_180
				else
					mo.angle = ANGLE_270
				end
				player.drawangle = mo.angle
				
				P_InstaThrust(mo, mo.angle, 16*mo.scale)
				if susie
					susie.angle = mo.angle
					if susie.state ~= S_PLAY_WALK
						susie.state = S_PLAY_WALK
					end
					susie.tics = min($, 2)
					P_TeleportMove(susie,
						mo.x + P_ReturnThrustX(mo, mo.angle, mo.radius*2),
						mo.y + P_ReturnThrustY(mo, mo.angle, mo.radius*2),
						mo.z
					)
					susie.momx = mo.momx
					susie.momy = mo.momy
					
					if player.skin == DC.Kris.GetSkinIndex()
						if mo.state ~= S_PLAY_FALL
							mo.state = S_PLAY_FALL
						end
					elseif mo.state ~= S_PLAY_PAIN
						mo.state = S_PLAY_PAIN
					end
					player.drawangle = $ + ANGLE_180
				else
					if mo.state ~= S_PLAY_WALK
						mo.state = S_PLAY_WALK
					end
					mo.tics = min($, 2)
				end
				
				yield()
			end
			
			if susie
				S_StartSound(mo, sfx_dcdead)
				mo.frame = $ &~ FF_FRAMEMASK
				mo.sprite2 = SPR2_SHIT
				player.panim = PA_ETC
				mo.tics = -1
				
				susie.state = S_PLAY_STND
			end
			
			return true
		end,
	}
})

// ---------------------------------
// ---------------------------------
// BLACK CORE
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "metalRace",
	zone = "black core",
	act = 1,
	script = {
		TICRATE*2/3,
		talk("susie", 7, "Hey, you wanna race, don't you?"),
		talk("ralsei", 5, "Susie, I don't think\127"),
		talk("susie", 3, "CHALLENGE ACCEPTED!"),
	}
})

DB.AddEvent({
	tag = "airCut",
	zone = "black core",
	act = 1,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.powers[pw_spacetime]
	end,
	script = {
		talk("susie", 6, "Did that punk just remove all the air??"),
		talk("ralsei", 21, "Kris! Susie! We should conserve oxygen! Just a little further!"),
		talk("susie", 6, "Conserve this!"),
		talk("susie", 18, "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\127"),
	}
})

DB.AddEvent({
	tag = "computerRoom",
	zone = "black core",
	act = 2,
	flags = DF.STASIS|DF.FLIP_BOX,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.mo and consoleplayer.mo.y <= 2287*FRACUNIT
	end,
	script = {
		function(player)
			while player.mo.momx or player.mo.momy
				yield()
			end
			return true
		end,
		spawnPartners,
		talk("susie", 12, "Huh? What's that on those computer screens?"),
		talk("eggman", 1, "-esting, 1, 2- *scratch*"),
		talk("susie", 5, "Oh, wait, it's just that big fat egg guy again."),
		talk("eggman", 1, "Silence! I am Dr\001.\000 Eggman!"),
		talk("eggman", 1, "The greatest scientific genius in the world!"),
		talk("eggman", 1, "And soon to be ruler of this planet!"),
		talk("ralsei", 36, "Ruler of the planet?"),
		talk("eggman", 1, "Indeed!"),
		talk("eggman", 1, "With the power of Black Rock and the Dark Fountain combined..."),
		talk("eggman", 1, "I shall create Brak Eggman Mark II and use it to impose my rule!"),
		talk("susie", 12, "The hell's a Brak?"),
		talk("ralsei", 16, "Dr\001.\000 Eggman, you have no idea what that fountain is capable of!"),
		talk("ralsei", 15, "You need to stop what you're doing now, before it's too late!"),
		talk("eggman", 1, "I very well know what it is capable of, goat boy!"),
		function(player)
			S_StartSound(nil, sfx_wbreak)
			DC.StartQuake(player, 24*player.mo.scale, 3)
			return true
		end,
		talk("eggman", 1, "LIMITLESS POWER!!"),
		talk("eggman", 1, "And you will not stop me, not even by force!"),
		talk("eggman", 1, "So I suggest you go run back home to wherever it is you came from."),
		function()
			S_StartSound(nil, sfx_dceggl)
			return true
		end,
		TICRATE*3/2,
		talk("ralsei", 35, "Kris, what are we going to do?"),
		talk("susie", 3, "Sounds to me like some butt-kicking is in order!"),
		talk("susie", 10, "Come on! We have a world to save!"),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "fakeSign",
	zone = "black core",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return DB.current == nil and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 12
	end,
	script = {
		talk("ralsei", 21, "Hey, a signpost!"),
		talk("susie", 18, "RALSEI WATCH OUT IT'S A TRAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\127"),
		talk("ralsei", 13, "WHAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"),
	}
})

DB.AddEvent({
	tag = "brak",
	zone = "black core",
	act = 3,
	flags = DF.STASIS|DF.FLIP_BOX,
	condition = function()
		local boss
		
		for mo in mobjs.iterate()
			if mo.type == MT_CYBRAKDEMON
				boss = mo
				boss.flags = $ | MF_NOTHINK
				break
			end
		end
		
		vars.boss = boss
		
		return boss
	end,
	script = {
		function(player)
			while not P_IsObjectOnGround(player.mo)
				yield()
			end
			return true
		end,
		talk("eggman", 1, "I warned you to leave, now you will reap the consequences!"),
		talk("eggman", 1, "FACE THE WRATH OF BRAK EGGMAN MARK I!"),
		talk("susie", 18, "FACE THE WRATH OF YOUR BUTT BEING KICKED BY THE $!$? SQUAD!"),
		talk("ralsei", 24, "...Fun Gang."),
		talk("susie", 22, "That's what I said, right?"),
		function()
			vars.boss.flags = $ &~ MF_NOTHINK
			return true
		end,
	}
})

DB.AddEvent({
	tag = "acid",
	zone = "black core",
	act = 3,
	flags = DF.FLIP_BOX,
	script = {
		4*TICRATE,
		talk("susie", 20, "Come on, that electric barrier is cheating!"),
		talk("ralsei", 21, "I bet if we lead him into that acid, that will discharge it!"),
		talk("susie", 11, "It's worth a shot."),
	}
})

DB.AddEvent({
	tag = "brak",
	zone = "black core",
	act = 3,
	flags = DF.FLIP_BOX,
	hookType = "ThinkFrame",
	condition = function()
		local boss = vars.boss
		if boss and boss.valid and boss.state == S_CYBRAKDEMON_VILE_ATTACK2
		and boss.type == MT_CYBRAKDEMON
			return true
		end
	end,
	script = {
		talk("susie", 18, "KRIS! RALSEI! TAKE COVER!"),
	}
})

DB.AddEvent({
	tag = "brak",
	zone = "black core",
	act = 3,
	flags = DF.FLIP_BOX,
	hookType = "MobjDeath",
	hookArg = MT_CYBRAKDEMON,
	condition = function()
		vars.beatBrak = true
		return true
	end,
	script = {
		talk("eggman", 1, "IMPOSSIBLE...!"),
		talk("ralsei", 18, "We did it!"),
		talk("ralsei", 35, "...Huh? Is the ground shaking?"),
		talk("susie", 18, "CRAP, SHE'S GOING DOWN!! WE GOTTA GO, KRIS, HURRY!!"),
	}
})

DB.AddEvent({
	tag = "brak",
	zone = "black core",
	act = 3,
	flags = DF.FLIP_BOX,
	hookType = "MobjDeath",
	hookArg = MT_CYBRAKDEMON,
	script = {
		TICRATE*30,
		talk("susie", 18, "KRIS. GET IN THE POD!! OR ELSE WE'LL..."),
		talk("susie", 6, "we'll..."),
		talk("susie", 12, "huh."),
		talk("ralsei", 23, "...is... the space station even crashing down..?"),
		talk("eggman", 1, "...CURSES!! THEY FIGURED IT OUT!!"),
		talk("susie", 18, "WHERE EVEN ARE YOU?!"),
	}
})

// ---------------------------------
// ---------------------------------
// FROZEN HILLSIDE
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "frozen",
	zone = "frozen hillside",
	act = 0,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 5, "Sheesh. This place is freezing cold!"),
		talk("susie", 11, "Bet Noelle would love it here..."),
		talk("ralsei", 3, "That's okay Susie! You can have my scarf if you need it."),
		talk("susie", 10, "Nah, I'm fine."),
		talk("susie", 11, "Plus, Kris looks like they need it waaay more than I do."),
		talk("susie", 3, "They're all blue!!"),
		talk("ralsei", 23, "Isn't that how they normally look?"),
		talk("susie", 7, "..."),
		talk("susie", 13, "...Y-Yeah. Sure."),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "amyKris",
	zone = "frozen hillside",
	act = 0,
	flags = DF.STASIS,
	hookType = "MobjThinker",
	hookArg = MT_ROSY,
	condition = function(amy)
		local mo = amy.target
		local player = consoleplayer
		if mo ~= consoleplayer.mo then return end
		
		return amy.state == S_ROSY_HUG
		and player.skin == DC.Kris.GetSkinIndex()
	end,
	script = {
		TICRATE/2,
		talk("ralsei", 27, "Aw, look Kris! You made a new friend!"),
		talk("susie", 11, "Pssh, yeah right."),
		talk("susie", 10, "Look at the two lil' lovebirds. She's got a tight grip on 'em!"),
		talk("susie", 12, "..."),
		talk("susie", 29, "Can... Can Kris breathe, though?"),
	}
})

DB.AddEvent({
	tag = "amySusie",
	zone = "frozen hillside",
	act = 0,
	flags = DF.STASIS,
	hookType = "MobjThinker",
	hookArg = MT_ROSY,
	condition = function(amy)
		local mo = amy.target
		local player = consoleplayer
		if mo ~= consoleplayer.mo then return end
		
		return amy.state == S_ROSY_HUG
		and player.skin == DC.Susie.GetSkinIndex()
	end,
	script = {
		TICRATE/2,
		talk("ralsei", 27, "Oh, Susie! She likes you!"),
		talk("susie", 18, "WHAT THE- HEY, LET GO!!"),
		talk("susie", 24, "..."),
		talk("susie", 23, "...KRIS WIPE THAT GRIN OFF YOUR FACE!!"),
	}
})

DB.AddEvent({
	tag = "amyRalsei",
	zone = "frozen hillside",
	act = 0,
	flags = DF.STASIS,
	hookType = "MobjThinker",
	hookArg = MT_ROSY,
	condition = function(amy)
		local mo = amy.target
		local player = consoleplayer
		if mo ~= consoleplayer.mo then return end
		
		return amy.state == S_ROSY_HUG
		and player.skin == DC.Ralsei.GetSkinIndex()
	end,
	script = {
		TICRATE/2,
		talk("ralsei", 14, "!!"),
		talk("susie", 8, "HAHAHA, LOOK KRIS! RALSEI'S GOT A GIRLFRIEND!!"),
		talk("ralsei", 25, "I-I'm humbled, really, but i-i-it's t-too... too soon!"),
	}
})

// ---------------------------------
// ---------------------------------
// PIPE TOWERS
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "mario",
	zone = "pipe towers",
	act = 0,
	flags = DF.STASIS,
	script = {
		function(player)
			player.pflags = $ | PF_INVIS --Prevent Goombas from killing you
			vars.killedGoomba = false
			return true
		end,
		spawnPartners,
		TICRATE/2,
		talk("susie", 12, "...huh?"),
		talk("susie", 28, "Hey, WAIT A MINUTE! This isn't Spiky Robot Bash 2 AT ALL!"),
		talk("susie", 24, "Kris? What are you smirking at?"),
		talk("ralsei", 21, "Oh! It seems we've somehow stumbled into the Mushroom Kingdom!"),
		talk("ralsei", 5, "Looks a bit barren though..."),
		talk("ralsei", 7, "The fact there are only Goombas here is not a good sign."),
		talk("susie", 11, "Those are the bad guys, right?"),
		talk("ralsei", 6, "Well, yes, but considering they're not robots we should try and\127"),
		talk("susie", 3, "That's all I needed to hear! TIME TO KICK SOME BUTT!!!"),
		talk("ralsei", 24, "Susie...!"),
		returnPartners,
		function(player)
			player.pflags = $ &~ PF_INVIS
			return true
		end,
	}
})

DB.AddEvent({
	tag = "krisGoomba",
	zone = "pipe towers",
	act = 0,
	hookType = "MobjDeath",
	hookArg = MT_GOOMBA,
	condition = function(mobj)
		if not vars.killedGoomba
		and not mobj.ralseispared
		and consoleplayer.skin == DC.Kris.GetSkinIndex()
			vars.killedGoomba = true
			return true
		end
	end,
	script = {
		TICRATE,
		talk("ralsei", 24, "Kris! Not you too!"),
		talk("susie", 8, "Hey Kris, I just saw another batch of them over there!"),
		talk("susie", 3, "Bet you can't squash more Goombas than me!"),
	}
})

DB.AddEvent({
	tag = "susieGoomba",
	zone = "pipe towers",
	act = 0,
	hookType = "MobjDeath",
	hookArg = MT_GOOMBA,
	condition = function(mobj)
		if not vars.killedGoomba
		and not mobj.ralseispared
		and consoleplayer.skin == DC.Susie.GetSkinIndex()
			vars.killedGoomba = true
			return true
		end
	end,
	script = {
		TICRATE,
		talk("ralsei", 24, "Susie, please! Have some respect!"),
		talk("susie", 3, "What, you WANT Princess Mango to get kidnapped again?"),
		talk("ralsei", 32, "T-That's not... That's not what I meant!"),
	}
})

DB.AddEvent({
	tag = "ralseiGoomba",
	zone = "pipe towers",
	act = 0,
	hookType = "MobjDeath",
	hookArg = MT_GOOMBA,
	condition = function(mobj)
		if not vars.killedGoomba
		and not mobj.ralseispared
		and consoleplayer.skin == DC.Ralsei.GetSkinIndex()
			vars.killedGoomba = true
			return true
		end
	end,
	script = {
		TICRATE,
		talk("ralsei", 8, "Oh, these poor, poor Goombas..."),
		talk("susie", 2, "If they're so poor then why are you beating them up?"),
		talk("ralsei", 36, "U-Um... Well... I-I was..."),
		talk("susie", 3, "Come on, I know you want in on the fun. Liven up a little!"),
		talk("susie", 3, "We can always give them some of these green shrooms later."),
		talk("ralsei", 12, "... I suppose."),
	}
})

DB.AddEvent({
	tag = "sparedGoomba",
	zone = "pipe towers",
	act = 0,
	hookType = "MobjDeath",
	hookArg = MT_GOOMBA,
	condition = function(mobj)
		if not vars.killedGoomba
		and mobj.ralseispared
		and consoleplayer.skin == DC.Ralsei.GetSkinIndex()
			vars.killedGoomba = true
			return true
		end
	end,
	script = {
		TICRATE,
		talk("susie", 12, "Are you putting them to sleep?"),
		talk("ralsei", 3, "...And then the little oopling went home."),
		talk("susie", 18, "ARE YOU READING THEM A BEDTIME STORY!?"),
	}
})

DB.AddEvent({
	tag = "pipeTowersGoal",
	zone = "pipe towers",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 1003 and #mo.subsector.sector.lines == 145
	end,
	script = {
		talk("susie", 18, "WHAT DO YOU MEAN PRINCESS MANGO IS IN ANOTHER CASTLE!?"),
	}
})

// ---------------------------------
// ---------------------------------
// FOREST FORTRESS
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "fortress",
	zone = "forest fortress",
	act = 0,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 12, "...Wait a minute. Didn't we already go through this place?"),
		talk("ralsei", 6, "What do you mean, Susie?"),
		talk("susie", 1, "Y'know. Middle of the woods? Castle?"),
		talk("susie", 2, "Though this looks totally barren. And boring."),
		talk("ralsei", 5, "Maybe that evil doctor began working on his castle here and just..."),
		talk("ralsei", 5, "...left?"),
		talk("susie", 2, "..."),
		talk("susie", 5, "That's lame. He's lame. THIS PLACE IS LAME!!"),
		returnPartners,
	}
})



DB.AddEvent({
	tag = "fortressMoss",
	zone = "forest fortress",
	act = 0,
	hookType = "TouchSpecial",
	hookArg = MT_DC_BANANA,
	flags = DF.STASIS,
	preCondition = function()
		if vars.ffzmoss then return end
		local save = P_SpawnMobj(6952*FRACUNIT, 14964*FRACUNIT, 5454*FRACUNIT, MT_DC_BANANA)
		if save and save.valid
			save.state = S_DC_SAVE
			return true
		end
	end,
	condition = function(save, mo)
		A_ForceStop(mo)
		return true
	end,
	script = {
		talk("susie", 6, "GUH *hack* BLEH"),
		talk("susie", 18, "EVEN THE MOSS HERE IS LAME!!"),
		function()
			vars.ffzmoss = true
			return true
		end,
	}
})

// ---------------------------------
// ---------------------------------
// TECHNO LEGACY
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "legacy",
	zone = "techno legacy",
	act = 0,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 12, "What."),
		talk("ralsei", 23, "Huh?"),
		talk("susie", 12, "Is this a Fountain Dew factory...?"),
		talk("susie", 1, "..."),
		talk("susie", 2, "..."),
		talk("susie", 2, "This place stinks, let's leave."),
		talk("ralsei", 10, "..."),
		talk("ralsei", 10, "Agreed."),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "legacy",
	zone = "techno legacy",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 119
	end,
	script = {
		TICRATE,
		talk("susie", 18, "KRIS, WHY ARE WE STILL HERE!?"),
	}
})

// ---------------------------------
// ---------------------------------
// HAUNTED HEIGHTS
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "captainCrunch",
	zone = "haunted heights",
	act = 0,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 5, "Man. How long have we even been here for?"),
		talk("ralsei", 6, "About a day?"),
		talk("susie", 18, "A DAY?! It feels like... like TWO YEARS have passed!!"),
		talk("ralsei", 5, "Well, it seems like time moves differently in this Dark World."),
		talk("ralsei", 5, "This fountain must be much more powerful than we thought."),
		talk("susie", 14, "...and I'm already all out of crunchy captain snacks..."),
		talk("susie", 19, "WHATEVER. Let's go already Kris!!"),
		returnPartners,
	}
})

// ---------------------------------
// ---------------------------------
// AERIAL GARDEN
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "agz",
	zone = "aerial garden",
	act = 0,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 8, "Finally, we made it to the best level in the entire game!"),
		talk("ralsei", 23, "Huh? Don't you find those bee things annoying?"),
		talk("susie", 3, "You kidding??"),
		talk("susie", 3, "They infinitely respawn, meaning I get to smash them forever!"),
		talk("ralsei", 21, "Alright, but let's not stay here too long, OK?"),
		talk("ralsei", 20, "We have a fountain to take care of!"),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "agzGoal",
	zone = "aerial garden",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 840 and #mo.subsector.sector.lines == 116
	end,
	script = {
		talk("susie", 8, "Hey, before we spin the signpost, can we go back and smash more bees?"),
		talk("ralsei", 32, "If we keep doing that, we'll be here forever!"),
		talk("susie", 3, "Exactly."),
	}
})

// ---------------------------------
// ---------------------------------
// AZURE TEMPLE
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "atz",
	zone = "azure temple",
	act = 0,
	flags = DF.STASIS,
	script = {
		function(player)
			player.powers[pw_underwater] = 0
			return true
		end,
		spawnPartners,
		TICRATE/2,
		talk("susie", 13, "... I think we took a wrong turn somewhere."),
		talk("susie", 20, "Where the hell is this place??"),
		talk("susie", 18, "AND WHY IS IT ALL UNDERWATER!!!??!?!"),
		talk("ralsei", 3, "These ruins look very peaceful."),
		talk("ralsei", 18, "I'm sure if we were fish, we'd love it here!"),
		talk("susie", 31, "... OK, but we're NOT fish."),
		talk("susie", 24, "How about you get us out of here before we become fish FOOD?!?"),
		returnPartners,
		function(player)
			player.powers[pw_underwater] = underwatertics
			return true
		end
	}
})

DB.AddEvent({
	tag = "puzzle1",
	zone = "azure temple",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 623 and #mo.subsector.sector.lines == 43
	end,
	script = {
		TICRATE,
		talk("susie", 18, "A PUZZLE?!?"),
		talk("susie", 5, "As if this place couldn't get any worse."),
		talk("ralsei", 3, "I'm sure it'll be easy! We just need to push these statues around."),
		talk("susie", 24, "Do I look like a dolphin to you?"),
		talk("susie", 24, "I'm not just gonna push stuff around randomly!"),
		talk("ralsei", 6, "No, but you do kind of look like a shark."),
		talk("susie", 13, "Let me just \129Rude Buster\128 these switches before you call me a fox too."),
	}
})

DB.AddEvent({
	tag = "puzzle2",
	zone = "azure temple",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		if mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 175 and #mo.subsector.sector.lines == 162
			vars.noPuzzle = true
			return true
		end
	end,
	script = {
		TICRATE,
		talk("susie", 18, "ANOTHER PUZZLE!!???!!?"),
		talk("ralsei", 6, "I don't think that \129Rude Buster\128 trick you pulled is going to work here."),
		talk("ralsei", 21, "Well, unless there were four of you, then you could line up..."),
		talk("susie", 5, "Can we just go down the other path?"),
		talk("susie", 5, "Anything would be better than this. And I mean ANYTHING."),
	}
})

DB.AddEvent({
	tag = "puzzle3",
	zone = "azure temple",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 773 and #mo.subsector.sector.lines == 117
	end,
	script = {
		TICRATE,
		talk("susie", 1, "I'm not even going to bother acting surprised."),
		talk("susie", 5, "Let's just get this over with."),
	}
})

DB.AddEvent({
	tag = "puzzle4",
	zone = "azure temple",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and mo.subsector.sector.tag == 63
	end,
	script = {
		TICRATE,
		talk("susie", 5, "Finally! Geez, no more puzzles! I am puzzled OUT."),
		talk("ralsei", 27, "They weren't that difficult, were they?"),
		talk("susie", 6, "That's not the POINT!"),
		talk("susie", 20, "Who even designed this!?"),
	}
})

DB.AddEvent({
	tag = "noPuzzle",
	zone = "azure temple",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return vars.noPuzzle and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 672 and #mo.subsector.sector.lines == 10
	end,
	script = {
		TICRATE,
		talk("susie", 20, "Ugh, what the hell? This path sucks too!"),
		talk("susie", 19, "I can't tell if this is worse than the puzzles."),
	}
})

// ---------------------------------
// ---------------------------------
// MISC BONUS
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "tutorial",
	zone = "tutorial",
	act = 0,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 14, "Huh? Who's that rat at the bottom of the screen?"),
		talk("susie", 13, "And why is she telling me what to do?"),
		talk("susie", 36, "Kris, was this in Spiky Robot Bash 2?"),
		talk("ralsei", 21, "Oh, seems we're in some sort of digital training facility."),
		talk("susie", 12, "A what now?"),
		talk("susie", 13, "Look, I don't care where we are."),
		talk("susie", 20, "All this dialog is just giving me a massive headache!"),
		talk("susie", 19, "Let's leave before my head ends up exploding."),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "titlescreen",
	flags = DF.STASIS,
	zone = "",
	act = 0,
	condition = function()
		return gamemap == titlemap and gamemap == 99
	end,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("ralsei", 23, "...?"),
		talk("susie", 12, "...?"),
		talk("susie", 1, "Kris..."),
		talk("susie", 2, "Did you tilt your cartridge or something?"),
		talk("susie", 18, "WHERE THE HELL IS THIS? I'M STANDING ON NOTHING!"),
		talk("ralsei", 21, "Look! There's some people up there waving!"),
		talk("susie", 20, "What? Where? I don't see anyone! Are they waving at us?"),
		talk("ralsei", 6, "No, they're... sorta just in a giant ring?"),
		talk("susie", 18, "WAVE AT THIS, NERDS!"),
		talk("ralsei", 35, "... *GASP*"),
		talk("ralsei", 24, "SUSIE!"),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "eatRing",
	hookType = "ThinkFrame",
	condition = function()
		return not (maptol & TOL_NIGHTS)
		and consoleplayer.powers[pw_shield] // less likely that you'll lose them
		and consoleplayer.rings > 150
	end,
	script = {
		3*TICRATE/2,
		talk("susie", 32, "..."),
		talk("susie", 21, "Hey, Ralsei? Are you \003SURE\000 these aren't onion rings?"),
		talk("ralsei", 3, "Yes, Susie, I'm 100% certain they are not onio\127"),
		function(player)
			local hudItems = {HUD_RINGS, HUD_RINGSNUM, HUD_RINGSNUMTICS}
			
			S_StartSound(nil, sfx_susie6, player)
			for i = 1, 8
				for _, hudItem in ipairs(hudItems)
					if (i/2) & 1
						hudinfo[hudItem].x = $ + 1
					else
						hudinfo[hudItem].x = $ - 1
					end
				end
				yield()
			end
			wait(8)
			
			if player.rings == 0 // aw, what?!
				DB.nextIndex = 100
				return true
			end
			
			local hudItems = {HUD_RINGS, HUD_RINGSNUM, HUD_RINGSNUMTICS}
			P_GivePlayerRings(player, -1)
			S_StartSound(nil, sfx_antiri, player)
			for i = 1, 16
				for _, hudItem in ipairs(hudItems)
					if (i/2) & 1
						hudinfo[hudItem].y = $ + 1
					else
						hudinfo[hudItem].y = $ - 1
					end
				end
				yield()
			end
			wait(12)
			return true
		end,
		talk("ralsei", 13, "NO NO NO OH MY GOD SUSIE DON'T EAT IT DON'T EAT IT IT'S NOT AN\127"),
		function(player)
			S_StartSound(nil, sfx_dcdyin, player)
			wait(5)
			S_StartSound(nil, sfx_dcdyin, player)
			wait(3)
			S_StartSound(nil, sfx_dcdyin, player)
			wait(20)
			S_StartSound(nil, sfx_dchurt, player)
			wait(6)
			S_StartSound(nil, sfx_susieh, player)
			wait(28)
			return true
		end,
		talk("susie", 19, "EUGGHHH, that's PUTRID!!"),
		talk("ralsei", 24, "Maybe you'll listen to me from now on!!"),
		[100] = talk("susie", 6, "God DAMMIT Kris, we lost the onion rings!! I'm \002STARVING\000 out here!")
	}
})

// ---------------------------------
// ---------------------------------
// SPECIAL STAGES / EMERALDS
// ---------------------------------
// ---------------------------------

DB.AddEvent({
	tag = "token",
	hookType = "TouchSpecial",
	hookArg = MT_TOKEN,
	condition = function()
		return DB.current == nil
	end,
	script = {
		TICRATE/2,
		talk("susie", 8, "Yo, Kris! What is this, a frisbee?"),
		talk("ralsei", 20, "It sure is shiny. I wonder why was it hidden like this?"),
		talk("susie", 11, "Here, stick it in your pocket with the rest of your junk."),
		talk("susie", 11, "Maybe if we can catch some downtime later..."),
		talk("susie", 3, "We can throw it at some random passers-by."),
	}
})

DB.AddEvent({
	tag = "ss1",
	zone = "floral field",
	act = 0,
	flags = DF.STASIS,
	script = {
		TICRATE*5/2,
		function()
			S_StartSound(consoleplayer and consoleplayer.mo, sfx_lancel)
			vars.lancer = true
			DB.current.event.flags = $ & ~DF.FLIP_BOX
			return true
		end,
		TICRATE*2,
		talk("lancer", 4, "SURPRISE, NERDS!!"),
		talk("lancer", 3, "I'M HERE TO GET SOME SHINY ROCKS!!"),
		talk("susie", 8, "No way, Lancer??? What's up man???"),
		talk("lancer", 6, "Susie!!! The Lancer Fan Club is back together!!!!"),
		function()
			DB.current.event.flags = $ | DF.FLIP_BOX
			return true
		end,
		talk("ralsei", 1, "...Um, I don't mean to interrupt, but is that a timer above us...?"),
		talk("lancer", 2, "Oops, forgot about that. Gotta juice!!"),
	}
})

DB.AddEvent({
	tag = "ss2",
	zone = "toxic plateau",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.bonustime
	end,
	script = {
		TICRATE/2,
		talk("susie", 12, "Uh, I thought they were Chaos EMERALDS... y'know, \131green\128?"),
		talk("ralsei", 27, "At least purple's a cool color!\nSee? Kris thinks so too!"),
		talk("susie", 23, "Guh??"),
		talk("lancer", 5, "Woah, red is cool too! What other colors can you turn into?"),
		talk("susie", 18, "SHUTUPSHUTUP!!!!"),
	}
})

DB.AddEvent({
	tag = "ss3",
	zone = "flooded cove",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.bonustime
	end,
	script = {
		TICRATE/2,
		talk("lancer", 3, "Hey!!! This one's blue like me!!!"),
		talk("lancer", 6, "It will look great with my eyes!!!"),
		talk("ralsei", 22, "Your... eyes? But where are they...?"),
		talk("lancer", 5, "On my face, kindboy!!!"),
	}
})

DB.AddEvent({
	tag = "ss4",
	zone = "cavern fortress",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.bonustime
	end,
	script = {
		TICRATE/2,
		talk("lancer", 6, "Another blue one!! This will look awesome with my bike!!"),
		talk("ralsei", 21, "Actually, this one's cyan."),
		talk("susie", 22, "Yeah, like Lancer said. It's blue."),
		talk("ralsei", 20, "Well, technically speaking, cyan is actually a different\127"),
		talk("lancer", 14, "NERD!!"),
	}
})

DB.AddEvent({
	tag = "ss5",
	zone = "dusty wasteland",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.bonustime
	end,
	script = {
		TICRATE/2,
		talk("lancer", 8, "Is it lemon flavored? The last ones tasted nasty."),
		talk("ralsei", 24, "Wait, have you been eating these?"),
		talk("lancer", 10, "Ooh, my tummy..."),
		talk("lancer", 11, "Bad on my teeth and bad on my tummy."),
		talk("susie", 22, "Ehh, it's an acquired taste."),
		talk("ralsei", 24, "You too!?"),
	}
})

DB.AddEvent({
	tag = "ss6",
	zone = "magma caves",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.bonustime
	end,
	script = {
		TICRATE/2,
		talk("lancer", 3, "Ooh, it's red!! Just like fire!!"),
		talk("susie", 8, "Or something cooler. Like BLOOD!!"),
		talk("lancer", 5, "Or STRAWBERRIES!!"),
		talk("susie", 3, "BLOOD STRAWBERRIES!!"),
		talk("lancer", 6, "So cool!!!"),
	}
})

DB.AddEvent({
	tag = "ss7",
	zone = "egg satellite",
	act = 0,
	flags = DF.STASIS,
	script = {
		TICRATE*5/2,
		talk("ralsei", 13, "W-Watch out, Lancer! This place looks really dangerous."),
		talk("susie", 3, "Nah, dontcha worry. Lancer's got this."),
		talk("lancer", 3, "Yeah!! My bike is fueled by victory..."),
		talk("lancer", 4, "AND FRIENDSHIP!!!"),
	}
})

DB.AddEvent({
	tag = "ss7collection",
	zone = "egg satellite",
	act = 0,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.bonustime
	end,
	script = {
		TICRATE/2,
		talk("lancer", 3, "Wowie, this one's the shiniest of them all!!"),
	}
})

DB.AddEvent({
	tag = "ss8",
	zone = "black hole",
	act = 0,
	flags = DF.STASIS,
	script = {
		TICRATE*5/2,
		talk("susie", 12, "..."),
		talk("susie", 13, "Kris, where the HELL are we now?!"),
		talk("ralsei", 23, "...G-Guys..."),
		talk("ralsei", 23, "Anyone else feel an overwhelming sense of dread here..?"),
		talk("lancer", nil, "Nope!! I can't smell anyways. Let's go!!!"),
	}
})

DB.AddEvent({
	tag = "dream1",
	zone = "dream hill",
	act = 0,
	flags = DF.STASIS,
	script = {
		spawnPartners,
		TICRATE/2,
		talk("susie", 1, "...?"),
		talk("susie", 2, "Kris, did you load the wrong game?"),
		talk("susie", 20, "What is THIS?! I don't even have the controller for this game!!"),
		returnPartners,
	}
})

DB.AddEvent({
	tag = "dream2",
	zone = "dream hill",
	act = 0,
	flags = DF.STASIS,
	hookType = "ThinkFrame",
	condition = function()
		return consoleplayer.powers[pw_carry] == CR_NIGHTSMODE
	end,
	script = {
		TICRATE*5/2,
		talk("lancer", 2, "Hello."),
		talk("susie", 8, "Lancer! You're here? Is this another special stage? More rocks?"),
		talk("lancer", 6, "Nope! It's my dream, hehe!"),
		talk("susie", 12, "A dream? But... wait, Kris, didn't we fall asleep before this?"),
		talk("susie", 13, "So this is a dream... WITHIN a dream?"),
		talk("lancer", 6, "I hope it's a good dream for you!"),
		talk("lancer", 3, "If not, I'll break into your house and make you feel safe."),
		talk("ralsei", 23, "Ummm... Thanks?"),
		talk("lancer", 4, "Hell yeah!"),
		talk("ralsei", 21, "So, are we gonna clear this?"),
		talk("lancer", 5, "Hell no!"),
	}
})

DB.AddEvent({
	tag = "emeralds",
	hookType = "ThinkFrame",
	condition = function()
		return All7Emeralds(emeralds) and not G_IsSpecialStage()
		and leveltime > 2 and DB.current == nil
	end,
	script = {
		TICRATE,
		talk("ralsei", 18, "We did it!"),
		talk("ralsei", 3, "Now that evil doctor can't use these gems for his evil schemes!"),
		talk("susie", 8, "Sweet. What do we get though?"),
		talk("ralsei", 6, "Hm? What do you mean, Susie?"),
		talk("susie", 36, "Oh y'know. You get all of these stones and gather 50 hoops..."),
		talk("susie", 8, "And then that's supposed to turn Spiky into SUPER Spiky."),
		talk("susie", 10, "I can't wait to become SUPER Susie!!"),
		talk("susie", 3, "Or better yet, SUSIEZILLA!!"),
		talk("lancer", 7, "Sorry bud!! Only male rats can go super duper!!"),
		talk("susie", 18, "WHAT?!"),
		talk("susie", 6, "So we... we did all that for NOTHING?!"),
		talk("lancer", 3, "It was fun!!! I wanna do it again!!!"),
		talk("susie", 2, "..."),
		talk("susie", 5, "I'm going to eat these rocks."),
	}
})

// ---------------------------------
// ---------------------------------
// UNUSED
// ---------------------------------
// ---------------------------------

/*
DB.AddEvent({
	tag = "loveTree",
	zone = "greenflower",
	act = 2,
	hookType = "ThinkFrame",
	condition = function()
		local player = consoleplayer
		local mo = player and player.mo
		return mo and mo.subsector and mo.subsector.sector
		and #mo.subsector.sector == 819 and #mo.subsector.sector.lines == 6
	end,
	script = {
		TICRATE,
		talk("ralsei", 3, "Wow, this place is pretty!"),
		talk("ralsei", 1, "Honestly though, I'm not sure what dialog to put here."),
	}
})
*/

// christmas joke
/*DB.AddEvent({
	tag = "christmas",
	zone = "frozen hillside",
	act = 0,
	flags = DF.STASIS,
	script = {
		3*TICRATE/2,
		talk("susie", 7, "Can you believe it guys?"),
		talk("susie", 8, "lightdasher's birthday! Just a week away!"),
		talk("susie", 10, "lightdasher's birthday is in a week! Woo-hoo!"),
		talk("susie", 9, "I am so happy about this information."),
		talk("susie", 8, "lightdasher's birthday, just a week away. Oh wow."),
		talk("susie", 10, "Can you believe it? lightdasher's birthday! Just in a week!"),
		talk("susie", 7, "It got here so fast!"),
		talk("susie", 8, "lightdasher's birthday! Just a\127")
	}
})*/

// color test
/*DB.AddEvent({
	tag = "colorTest",
	hookType = "MobjDamage",
	hookArg = MT_BLUECRAWLA,
	script = {
		talk("lancer", nil, "\129Hey, \130check \131out \132all \133these \134rad \135colors \136we \137can \138use \139now! \140They \141are \142so \143cool!"),
		talk("lancer", nil, "Now Kris can eat the \131[Moss]\128.")
	}
})*/

// wrapping/cutoff test
/*DB.AddEvent({
	tag = "wrapTest",
	hookType = "MobjDamage",
	hookArg = MT_BLUECRAWLA,
	script = {
		talk("lancer", nil, "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA")
	}
})*/