/*
The Lancer Fan Club

v1
*/

local DC = DeltaChars

// ----
// vars
// ----

DC.skins = {}
DC.libraries = {}

DC.destructables = {}
DC.creditsWait = TICRATE
DC.secretHP = 0

// Abilites table for Kirby to eat
if not(kirbyabilitytable)
    rawset(_G, "kirbyabilitytable", {})
end
kirbyabilitytable["kris"] = 10 // Sword
kirbyabilitytable["susie"] = 11 // Hammer
kirbyabilitytable["ralsei"] = 15 // Sleep

// ---
// commands
// ---

DC.fun = CV_RegisterVar({
	name = "dc_fun",
	defaultvalue = "On",
	flags = CV_NETVAR|CV_CALL|CV_NOINIT,
	PossibleValue = CV_OnOff,
	func = function(var)
		if var.value
			print("You can now switch characters as DeltaChars in competitive gametypes.")
		else
			print("You no longer switch characters as DeltaChars in competitive gametypes.")
		end
	end,
})

DC.help = function(player)
	S_StartSound(nil, sfx_drtalk, player)
	CONS_Printf(player, "\136dc_dialogue - Toggles whether DeltaChars' dialog triggers in singleplayer.")
	CONS_Printf(player, "\140dc_altswap - Toggles the buttons used to swap characters from Custom 1/3 to Weapon Next/Prev.")
	CONS_Printf(player, "\140dc_altswap2 - P2 variant of the above.")
	CONS_Printf(player, "\129dc_susieaxangle - Changes which of Susie's attacks face the camera.")
	CONS_Printf(player, "\129dc_susieaxangle2 - P2 variant of the above.")
	CONS_Printf(player, "\131dc_ralseihat - Toggles Ralsei's Chapter One alterative skin.")
	CONS_Printf(player, "\131dc_ralseihat2 - P2 variant of the above.")
	CONS_Printf(player, "\133dc_fun - Toggles whether switching as DeltaChars is allowed in competitive gametypes.")
	CONS_Printf(player, "\143dc_debug - Toggles DeltaChars' debug display.")
end
COM_AddCommand("dc_help", DC.help, COM_LOCAL)

// ----
// defs
// ----

DC.FLAGS = {}
DC.numFlags = 0
function DC.RegisterFlag(name)
	assert(DC.numFlags < 32, "Hit flag cap, could not assign flag "..name.."!")
	DC.FLAGS[name] = 1 << DC.numFlags
	DC.numFlags = $ + 1
	return DC.FLAGS[name]
end

DC.RegisterFlag("DANCING")
DC.RegisterFlag("SWITCHING")
DC.RegisterFlag("FUNNY")

DC.BUTTONS = {BT_JUMP, BT_SPIN, BT_CUSTOM1, BT_CUSTOM2, BT_CUSTOM3, BT_TOSSFLAG, BT_WEAPONNEXT, BT_WEAPONPREV}

DC.DEATHS = {}
DC.DEATHS.NONE = 1
DC.DEATHS.DROWN = 2
DC.DEATHS.PIT = 3
DC.DEATHS.FIRE = 4
DC.DEATHS.SELF = 5

DC.PFLAGS_BLACKLIST = PF_SLIDING|PF_BOUNCING|PF_GLIDING|PF_STARTDASH|PF_SPINNING|PF_GAMETYPEOVER|PF_STASIS

DC.DANCE_NORMALSPEED = 2*FRACUNIT

DC.ORIG_FRICTION = (0xE8 << (FRACBITS-8))

DC.MAX_SECRET_HP = 20

// --------
// consvars
// --------

DC.friendlyFire = CV_FindVar("friendlyfire")
DC.forceSkin = CV_FindVar("forceskin")

// ---------
// functions
// ---------

function DC.Valid(mo)
	return mo and mo.valid
end

// to be used in each individual script to add members to the Fun Gang
function DC.AddMember(library)
	table.insert(DC.libraries, library)
	table.insert(DC.skins, library.SKIN)
end

// returns a table with a clean set of deltaChars variables
function DC.CreateTable(player)
	local mo = player.mo
	local buttons = {}
	
	for i, button in ipairs(DC.BUTTONS)
		buttons[button] = 0
	end
	
	player.deltaChars = {
		buttons = buttons,
		flags = 0,
		deathState = 0,
		deathType = DC.DEATHS.NONE,
		deathMomZ = 0,
		hitlag = 0,
		lockAngle = 0,
		prevState = mo.state,
		prevFrame = mo.frame,
		prevPFlags = player.pflags,
		prevGrounded = P_IsObjectOnGround(mo),
		prevCarry = player.powers[pw_carry],
		prevWater = mo.eflags & MFE_UNDERWATER,
		thrownheart = false,
		ralseifunny = 0,
	}
	return player.deltaChars
end

// removes deltaChars variables from @player
function DC.RemoveTable(player)
	local delta = player.deltaChars
	
	if delta == nil
		return // nothing to do
	end
	
	for _, library in ipairs(DC.libraries)
		library.RemoveTable(player)
	end
	
	if delta.flags & (DC.FLAGS.DANCING | DC.FLAGS.FUNNY)
		player.normalspeed = skins[player.skin].normalspeed
		P_RestoreMusic(player)
		if player.mo then player.mo.state = S_DC_TERMINATE end
	end
	
	if delta.deathType == DC.DEATHS.PIT
		player.camerascale = skins[player.skin].camerascale
	end
	player.deltaChars = nil
end

// returns true if @player is a fun gang member, and creates variables for them if they don't have any
// otherwise, destroys the variables and returns false
function DC.CheckStatus(player)
	local library = DC.IsDeltaChar(player)
	
	if not library
	or not player.mo
		DC.RemoveTable(player)
		return false
	end
	
	if not player.deltaChars
		DC.CreateTable(player)
	end
	if not player.deltaChars[library.SKIN]
		for _, library in ipairs(DC.libraries)
			// remove the table of each library
			// and if it got removed, get out of any states we shouldn't be in
			if library.RemoveTable(player) and library.CancelStates
			and library.CancelStates[player.mo.state]
				player.mo.state = S_DC_TERMINATE
			end
		end
		library.CreateTable(player)
	end
	return true
end

// if @player is playing as any member of the Fun Gang, returns their skin number
function DC.IsDeltaChar(player)
	for i, library in ipairs(DC.libraries)
		if player.skin == library.GetSkinIndex()
			return library
		end
	end
end

// returns true if in singleplayer mode
function DC.Singleplayer(switching)
	return ((switching and (gametyperules & GTR_FRIENDLY or DC.fun.value))
	or (not multiplayer
	and not modeattacking
	and not titlemapinaction
	and not players[1]))
	and not P_BuddyExFollow
end

// returns the "z movement" of @mo, using momz in the air or the slope pitch on the ground
function DC.GetMobjZMovement(mo)
	if P_IsObjectOnGround(mo)
		local slope = mo.standingslope
		
		if slope
			local angDiff = R_PointToAngle2(0, 0, mo.momx, mo.momy) - slope.xydirection
			local speed = FixedHypot(mo.momx, mo.momy)
			
			return FixedMul(FixedMul(speed, cos(angDiff)), sin(slope.zangle))
		end
		
		return 0
	end
	
	return mo.momz
end

// returns true if the @player is under the correct conditions to allow button pressing
function DC.CanPressButtons(player, button)
	if player.pflags & PF_STASIS
	or player.powers[pw_nocontrol]
	or (player.exiting and (button == BT_JUMP or button == BT_SPIN))
	or player.playerstate ~= PST_LIVE
		return false
	end
	return true
end

// returns true if @player just pressed @button
function DC.ButtonDown(player, button, hold, ignoreConditions)
	// no, don't press buttons during preticker!!
	if leveltime < 2
		return false
	end

	if not ignoreConditions
	and not DC.CanPressButtons(player, button)
		return false
	end
	
	if hold
		return player.deltaChars.buttons[button]
	else
		return player.deltaChars.buttons[button] == 1
	end
end

function DC.PreUpdatePlayer(player)
	local delta = player.deltaChars
	
	if delta.hitlag <= 0
		// update buttons
		for i, button in ipairs(DC.BUTTONS)
			delta.buttons[button] = (player.cmd.buttons & button) and $ + 1 or 0
		end
	end
	
	if player.playerstate == PST_DEAD
		DC.HandleDeath(player)
	else
		DC.Lancer.UpdatePlayer(player)
		DC.HandleFunny(player)
		DC.HandleDance(player)
	end
end

function DC.PostUpdatePlayer(player)
	DC.HandleHitlag(player)
	DC.HandleSwitching(player)
	DC.UpdateVars(player)
end

// thinker while dead
function DC.HandleDeath(player)
	local delta = player.deltaChars
	local mo = player.mo
	
	// use vanilla drown behavior
	if delta.deathType == DC.DEATHS.DROWN
		return
	end
	
	// pit behavior, zoom out
	if delta.deathType == DC.DEATHS.PIT
		if mo.fuse > 0
			player.camerascale = $ + FRACUNIT/32
		end
		if delta.deathState == 0
			mo.fuse = TICRATE
			mo.momz = 0
			mo.flags2 = $ | MF2_DONTDRAW
			delta.deathState = 1
		end
		return
	end
	
	// fire behavior
	if delta.deathType == DC.DEATHS.FIRE
		if delta.deathState == 0
			mo.fuse = TICRATE
			P_SetObjectMomZ(mo, 12*FRACUNIT, false)
			P_InstaThrust(mo, player.drawangle, 6*mo.scale)
			
			S_StartSound(mo, sfx_dcburn)
			mo.sprite2 = SPR2_BURN
			delta.deathState = 1
		elseif mo.fuse
			mo.rollangle = $ + ANGLE_22h
			mo.momz = $ + P_GetMobjGravity(mo)/2
			if P_IsObjectOnGround(mo)
				A_ForceStop(mo, 1)
			end
			A_BossScream(mo, 1, MT_SMOKE)
		end
		
		return
	end
	
	// regular death behavior
	if delta.deathState == 0
		mo.fuse = INT32_MAX
		mo.flags = $ & ~MF_NOCLIPHEIGHT
		
		if delta.deathType == DC.DEATHS.SELF and P_IsObjectOnGround(mo)
			delta.deathState = 2
			mo.state = S_DC_COLLAPSE
			S_StartSound(mo, sfx_dcdyin)
		else
			mo.state = S_PLAY_PAIN
			mo.z = $ + P_MobjFlip(mo)
			P_SetObjectMomZ(mo, 6*FRACUNIT)
			P_InstaThrust(mo, player.drawangle + ANGLE_180, 6*mo.scale)
			delta.deathState = 1
		end
	end
	
	if delta.deathState == 1
		player.deadtimer = 0
		if P_CheckDeathPitCollide(mo)
			mo.flags = $ | MF_NOCLIPHEIGHT
			delta.deathState = 0
			
			mo.momz = delta.deathMomZ
			DC.SpawnPitDummy(player)
			delta.deathType = DC.DEATHS.PIT
			
		elseif P_IsObjectOnGround(mo)
			delta.deathState = 2
			mo.momx = 0
			mo.momy = 0
			mo.state = S_DC_COLLAPSE
			S_StartSound(mo, sfx_dcdyin)
		else
			delta.deathMomZ = mo.momz
			if mo.momz < -16*mo.scale
				if player == displayplayer and camera
					camera.z = $ + mo.momz
				elseif player == secondarydisplayplayer and camera2
					camera2.z = $ + mo.momz
				end
			end
		end
	elseif delta.deathState == 2
		local timer = player.deadtimer - 2*TICRATE/3
		
		if timer < 0
			return
		end
		
		if timer < TICRATE/2
			local shake = ((timer & 1)*2 - 1) * FRACUNIT
			if timer > TICRATE/4
				shake = 2*$
			end
			mo.spritexoffset = shake
		else
			S_StartSound(mo, sfx_dcdead)
			mo.spritexoffset = 0
			mo.state = $
			delta.deathState = 3
		end
	end
end		

function DC.HandleSwitching(player)
	local delta = player.deltaChars
	
	// switch characters by pressing custom 1
	local custom1
	local custom3
	if not (player.dcaltswap)
		custom1 = DC.ButtonDown(player, BT_CUSTOM1)
		custom3 = not custom1 and DC.ButtonDown(player, BT_CUSTOM3)
	else
		custom1 = DC.ButtonDown(player, BT_WEAPONNEXT)
		custom3 = not custom1 and DC.ButtonDown(player, BT_WEAPONPREV)
	end
	if (custom1 or custom3)
	and player.powers[pw_carry] ~= CR_NIGHTSMODE
	and not P_PlayerInPain(player)
	and not (delta.flags & (DC.FLAGS.SWITCHING | DC.FLAGS.FUNNY))
	and not (multiplayer and DC.forceSkin.value ~= -1)
	and DC.Singleplayer(true)
		DC.StartSwitch(player, custom3 and DC.GetPrevSkin(player.skin) or nil)
	end
	
	local overlay = delta.switchOverlay
	
	// handle overlay
	if DC.Valid(overlay)
		local mo = player.mo
		
		if delta.flags & DC.FLAGS.SWITCHING
			if not (overlay.frame & FF_TRANSMASK)
				DC.SwitchMember(player)
			else
				overlay.frame = $ - FF_TRANS10
			end
			DC.SpawnSpareSparkles(mo, 2, 2*FRACUNIT/3, 8*FRACUNIT)
		else
			overlay.frame = $ + FF_TRANS10
			if (overlay.frame & FF_TRANSMASK) > FF_TRANS90
				P_RemoveMobj(overlay)
				delta.switchOverlay = nil
				return
			end
		end
		
		if overlay.skin ~= mo.skin
			overlay.skin = mo.skin
		end
		
		//SMS Alfredo's painful hat of death
		if player.ralseihat
			overlay.eflags = $ | MFE_FORCESUPER
		end
		overlay.sprite = mo.sprite
		overlay.sprite2 = mo.sprite2
		overlay.frame = ($ & FF_TRANSMASK) | (mo.frame & ~FF_TRANSMASK)
	end
end

function DC.StartSwitch(player, newSkin)
	local delta = player.deltaChars
	local mo = player.mo
	
	if newSkin == nil
		newSkin = DC.GetNextSkin(player.skin)
	end
	delta.switchSkin = newSkin
	
	if not DC.Valid(delta.switchOverlay)
		local overlay = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_OVERLAY)
		overlay.target = mo
		overlay.state = S_INVISIBLE
		overlay.angle = player.drawangle
		overlay.color = SKINCOLOR_DC_SPARE
		overlay.colorized = true
		overlay.renderflags = $ | RF_FULLBRIGHT
		overlay.blendmode = AST_ADD
		overlay.frame = ($ & ~FF_TRANSMASK) | (FF_TRANS10 * 10)
		
		delta.switchOverlay = overlay
	end
	S_StartSound(mo, sfx_dcswch)
	delta.flags = $ | DC.FLAGS.SWITCHING
end

function DC.GetNextSkin(skin)
	local newSkin
	for i, library in ipairs(DC.libraries)
		if skin == library.GetSkinIndex()
			if i == #DC.libraries
				newSkin = DC.libraries[1].GetSkinIndex()
			else
				newSkin = DC.libraries[i + 1].GetSkinIndex()
			end
			break
		end
	end
	return newSkin
end

function DC.GetPrevSkin(skin)
	local newSkin
	for i, library in ipairs(DC.libraries)
		if skin == library.GetSkinIndex()
			if i == 1
				newSkin = DC.libraries[#DC.libraries].GetSkinIndex()
			else
				newSkin = DC.libraries[i - 1].GetSkinIndex()
			end
			break
		end
	end
	return newSkin
end

// switches @player's skin to the next member of the Fun Gang
function DC.SwitchMember(player)
	local delta = player.deltaChars
	local newSkin = delta.switchSkin
	local oldSkin = player.skin
	
	if newSkin ~= nil
		local mo = player.mo
		local height = player.height
		local oldColor = player.skincolor
		
		R_SetPlayerSkin(player, newSkin)
		if player.powers[pw_carry] == CR_ROPEHANG
			mo.z = $ + FixedMul(height - player.height, mo.scale)
		end
		
		if player.powers[pw_shield] & SH_FIREFLOWER
			mo.color = SKINCOLOR_WHITE
		end
			
		//Set each character's prefcolors if you're using them
		//Also applies to the alt colors
		//But in Ralsei's case, his alt color is determined by his hat
		local isRalsei = newSkin == DC.Ralsei.GetSkinIndex()
		local wasRalsei = oldSkin == DC.Ralsei.GetSkinIndex()
		newSkin = skins[$]
		oldSkin = skins[$]
		local wasPref = oldColor == oldSkin.prefcolor
		local wasAlt = oldColor == oldSkin.prefcolor + 1
		
		if wasPref or wasAlt
			local change = (mo.color == player.skincolor)
			if (not isRalsei and ((not wasRalsei and wasAlt) or (wasRalsei and player.deltaAltColors)))
			or (isRalsei and player.ralseihat)
				player.skincolor = newSkin.prefcolor + 1
			else
				player.skincolor = newSkin.prefcolor
			end
			
			if (wasPref or wasAlt) and not wasRalsei
				player.deltaAltColors = wasAlt
			end
				
			if change then mo.color = player.skincolor end
		end
	end
	delta.flags = $ & ~DC.FLAGS.SWITCHING
	delta.switchSkin = nil
end

--Frame lengths for Kris' funny
DC.KrisFunnyFrames = {
	[B] = 4,
	[E] = 4,
	[F] = 1,
	[G] = 22,
	[H] = 1,
	[I] = 22,
	[J] = 4,
	[K] = 22,
	[M] = 4,
	[N] = 22,
	[O] = TICRATE,
	[R_Char2Frame("d")] = TICRATE*2/3,
	[R_Char2Frame("e")] = 5
}

DC.RalseiFunnyWait = 60 --Seconds before Ralsei dies

function DC.SpawnHeartEffect(mo, hearteffect)
	local thok = P_SpawnMobjFromMobj(mo, 0,0, FixedDiv(mo.height, mo.scale)/2, MT_THOK)
	thok.fuse = thok.tics
	thok.state = S_DC_HEART
	thok.frame = Q | FF_FULLBRIGHT | FF_ADD
	
	thok.scalespeed = FixedMul($, thok.scale*8)
	thok.scale = $/4
	thok.destscale = $*16
	
	if hearteffect and mo.player and mo.player.deltaChars
		mo.player.deltaChars.hearteffect = hearteffect
	end
end

function DC.HandleFunny(player)
	local mo = player.mo
	local delta = player.deltaChars
	local kris = player.skin == DC.Kris.GetSkinIndex()
	local ralsei = player.skin == DC.Ralsei.GetSkinIndex()
	
	--Heart effect
	if delta.hearteffect
		if delta.hearteffect & 1
			DC.SpawnHeartEffect(mo)
		end
		delta.hearteffect = $ - 1
	end
	
	--During funnies
	if delta.flags & DC.FLAGS.FUNNY
		if mo.state == S_TAILSOVERLAY_EDGE
			player.normalspeed = 0
			
			--Kris' funny is uncancellable and has special logic
			if kris
				local frame = mo.frame & FF_FRAMEMASK
				player.pflags = $ | PF_JUMPSTASIS | PF_JUMPDOWN | PF_SPINDOWN
				
				if not frame and mo.tics == states[mo.state].tics - 1
					S_FadeOutStopMusic(MUSICRATE, player)
				elseif mo.tics == states[mo.state].tics
					--Grab heart
					if frame == E
						DC.StartQuake(player, 24*mo.scale, 3)
						S_StartSound(mo, sfx_dcdmg)
						
					--Convulse
					elseif frame == F or frame == H
						DC.SetHitlag(player, TICRATE/4)
						if frame == H
							S_StartSound(mo, sfx_dchurt)
						end
						
					--Rip it out
					elseif frame == J
						DC.StartQuake(player, 24*mo.scale, 12)
						DC.SpawnHeartEffect(mo, 7)
						S_StartSound(mo, sfx_susie6)
						
					--Throw it
					elseif frame == M
						DC.StartQuake(player, 24*mo.scale, 3)
						S_StartSound(mo, sfx_swing3)
						
						local angle = (player.cmd.angleturn << 16) - ANGLE_90
						local dist = mo.radius*3/2
						local hthrust = 16*mo.scale
						local vthrust = -64*FRACUNIT
						local heart = P_SpawnMobjFromMobj(mo,
							P_ReturnThrustX(mo, angle, dist),
							P_ReturnThrustY(mo, angle, dist),
							player.height/2,
							MT_DC_HEART
						)
						delta.heart = heart
						delta.thrownheart = true
						heart.target = mo
						P_InstaThrust(heart, angle, hthrust)
						P_SetObjectMomZ(heart, vthrust, false)
					elseif frame == O
						P_PlayJingleMusic(player, "CYBERE", 0, false, JT_OTHER)
					end
					
				--End dance
				elseif frame == R_Char2Frame("e")
					if mo.tics == 1
						P_RestoreMusic(player)
						mo.state = S_DC_TERMINATE
						if not DC.Valid(delta.heart)
							P_KillMobj(mo, mo, nil)
							mo.momz = 0
						end
						delta.thrownheart = false
					end
				end
				mo.tics = min($, DC.KrisFunnyFrames[frame] or 11)
			
			--Susie and Ralsei funny is cancellable
			elseif not P_IsObjectOnGround(mo)
			or DC.ButtonDown(player, BT_CUSTOM2)
				S_StopSoundByID(mo, ralsei and sfx_dcblnt or sfx_dctail)
				mo.state = S_DC_TERMINATE
				if delta.thrownheart
					delta.thrownheart = false
				end
				
			--Ralsei smoke
			--The positioning desyncs, but hopefully that doesn't cause any issues
			elseif ralsei
				if mo.tics % 5 == 0 and delta.ralseifunny < DC.RalseiFunnyWait
					local smokeplayer = not splitscreen and displayplayer or player
					local angle = (smokeplayer.cmd.angleturn << 16) + ANGLE_112h
					local dist = mo.info.radius*5/2
					local smoke = P_SpawnMobjFromMobj(mo,
						P_ReturnThrustX(mo, angle, dist) + FRACUNIT*P_SignedRandom()/INT8_MAX,
						P_ReturnThrustY(mo, angle, dist) + FRACUNIT*P_SignedRandom()/INT8_MAX,
						player.height*3/5, MT_SMOKE
					)
					--smoke.scale = $/2
					smoke.destscale = $*8
					smoke.scalespeed = FixedMul($, smoke.scale)
					
					smoke.momx = P_SignedRandom()*mo.scale/INT8_MAX
					smoke.momy = P_SignedRandom()*mo.scale/INT8_MAX
					smoke.momz = 2*mo.scale
					
					smoke.state = S_DC_SMOKE
					if P_RandomChance(FRACUNIT/2)
						smoke.mirrored = true
					end
				end
				
				if mo.tics == 1
					delta.ralseifunny = $ + 1
					if delta.ralseifunny == DC.RalseiFunnyWait
					or delta.ralseifunny == DC.RalseiFunnyWait + 1
						mo.state = $
						mo.sprite2 = SPR2_DRWN
						
						S_StartSound(mo, sfx_dchurt)
						if delta.ralseifunny == DC.RalseiFunnyWait
							mo.tics = TICRATE/4
						end
						DC.SetHitlag(player, TICRATE/4)
						
					elseif delta.ralseifunny == DC.RalseiFunnyWait + 2
						mo.state = $
						mo.sprite2 = SPR2_TRNS
						mo.frame = $ | FF_HORIZONTALFLIP
						mo.tics = 5
						
					elseif delta.ralseifunny >= DC.RalseiFunnyWait + 3
						mo.state = $
						mo.sprite2 = SPR2_CNT1
						
						S_StartSound(mo, sfx_dcdead)
						mo.tics = -1
						DC.SetHitlag(player, 5)
					end
				end
			end
			
		--Cancel funny, not in funny state
		else
			delta.flags = $ & ~DC.FLAGS.FUNNY
			if kris
				P_RestoreMusic(player)
				if delta.thrownheart and not DC.Valid(delta.heart)
					P_DamageMobj(mo, mo, nil, 1, DMG_INSTAKILL)
					mo.momz = 0
				end
			end
			delta.heart = nil
			delta.thrownheart = false
			
			player.normalspeed = skins[player.skin].normalspeed
			player.hasnomomentum = false // xmom
		end
		return
	end
	
	if DC.CanDance(player) 
	and (mo.state == S_PLAY_STND or mo.state == S_PLAY_EDGE)
	and not (delta.flags & DC.FLAGS.SWITCHING)
	and DC.ButtonDown(player, BT_TOSSFLAG, true)
	and DC.ButtonDown(player, BT_CUSTOM2)
		mo.state = S_TAILSOVERLAY_EDGE
		delta.flags = $ | DC.FLAGS.FUNNY
		delta.ralseifunny = 0
		if not kris
			S_StartSound(mo, ralsei and sfx_dcblnt or sfx_dctail)
		end
		player.hasnomomentum = true // xmom
	end
end

function DC.HandleDance(player)
	local mo = player.mo
	local delta = player.deltaChars
	
	if mo.state == S_DC_DANCE
		if not P_IsObjectOnGround(mo)
		or DC.ButtonDown(player, BT_CUSTOM2)
			mo.state = S_DC_TERMINATE
		end
	elseif DC.ButtonDown(player, BT_CUSTOM2)
	and DC.CanDance(player)
		delta.flags = $ | DC.FLAGS.DANCING
		mo.state = S_DC_DANCE
		P_PlayJingleMusic(player, "CYBERB", 0, true, JT_OTHER)
		player.hasnomomentum = true // xmom
	end
	
	if mo.state == S_DC_DANCE
		player.normalspeed = min($, DC.DANCE_NORMALSPEED)
	elseif delta.flags & DC.FLAGS.DANCING
		delta.flags = $ & ~DC.FLAGS.DANCING
		P_RestoreMusic(player)
		player.normalspeed = skins[player.skin].normalspeed
		player.hasnomomentum = false // xmom
	end
end

// returns whether @player can start dancing
function DC.CanDance(player)
	return P_IsObjectOnGround(player.mo)
	and player.powers[pw_carry] == CR_NONE
	and not player.powers[pw_nocontrol]
	and not (player.pflags & DC.PFLAGS_BLACKLIST)
	and not (player.deltaChars.flags & (DC.FLAGS.DANCING | DC.FLAGS.FUNNY))
end

// sets up the hitlag effect for a duration of @tics
function DC.SetHitlag(player, tics)
	player.deltaChars.hitlag = max($, tics)
	player.mo.flags = $ | MF_NOTHINK
	player.pflags = $ | PF_FULLSTASIS
end

function DC.HandleHitlag(player)
	local mo = player.mo
	local delta = player.deltaChars
	
	if delta.hitlag <= 0
		return false
	end
	
	delta.hitlag = $ - 1
	if delta.hitlag > 0
		player.pflags = $ | PF_FULLSTASIS
		mo.flags = $ | MF_NOTHINK
		mo.spritexoffset = 2*delta.hitlag*FRACUNIT
		if delta.hitlag & 1
			mo.spritexoffset = -$
		end
	else
		mo.flags = $ & ~MF_NOTHINK
		mo.spritexoffset = 0
	end
	return true
end

// updates "prev" variables so they can be used next tic
function DC.UpdateVars(player)
	local delta = player.deltaChars
	local mo = player.mo
	
	delta.prevState = mo.state
	delta.prevFrame = mo.frame
	delta.prevPFlags = player.pflags
	delta.prevGrounded = P_IsObjectOnGround(mo)
	delta.prevCarry = player.powers[pw_carry]
	delta.prevWater = mo.eflags & MFE_UNDERWATER
	//delta.flags = $ & ~DC.FLAGS.RENEW_SLIDE
end

// spawns @num sparkles around @mo's hitbox
function DC.SpawnSpareSparkles(mo, num, speedFactor, extraHeight, color)
	local radius = FixedDiv(mo.radius, mo.scale)
	local height = FixedDiv(mo.height, mo.scale) + (extraHeight or 0)
	
	speedFactor = $ or 0
	
	for i = 1, (num or 1)
		local spark = P_SpawnMobjFromMobj(mo,
			FixedMul(radius << 1, P_RandomFixed()) - radius,
			FixedMul(radius << 1, P_RandomFixed()) - radius,
			FixedMul(height, P_RandomFixed()),
			MT_DC_SPARESPARKLE)
		if speedFactor
			spark.momx = FixedMul(mo.momx, speedFactor)
			spark.momy = FixedMul(mo.momy, speedFactor)
			spark.momz = FixedMul(DC.GetMobjZMovement(mo), speedFactor)
		end
		if color
			spark.color = color
			spark.colorized = true
		end
	end
end

// spawns dust from @player
function DC.SpawnDust(player)
	local mo = player.mo
	local radius = FixedDiv(mo.radius, mo.scale)
	local angle
	
	if (mo.momx - player.cmomx) or (mo.momy - player.cmomy)
		angle = DC.GetPlayerMomentumAngle(player)
	else
		angle = player.drawangle
	end
	
	local x = P_ReturnThrustX(mo, angle, -radius)
	local y = P_ReturnThrustY(mo, angle, -radius)
	local dust
	
	angle = $ + ANGLE_90
	radius = FixedMul($, P_RandomFixed()) - ($ >> 1)
	
	x = $ + P_ReturnThrustX(mo, angle, radius)
	y = $ + P_ReturnThrustY(mo, angle, radius)
	
	dust = P_SpawnMobjFromMobj(mo, x, y, 0, MT_DUST)
	dust.state = S_DC_DUST
	dust.scale = $ * 2
	return dust
end

// spawns an explosion at @mo using an object of the specified @type
// optional: @scale to set sprite scale, defaults to 2*FRACUNIT
function DC.SpawnExplosion(mo, type, noSound, scale)
	local boom = P_SpawnMobjFromMobj(mo, 0, 0, 0, type)
	
	scale = $ or 2*FRACUNIT
	boom.spritexscale = FixedMul($, scale)
	boom.spriteyscale = FixedMul($, scale)
	boom.state = S_DC_EXPLOSION
	if not noSound
		S_StartSound(mo, sfx_dcexpl)
	end
	return boom
end

// kills spikes the same way vanilla does
function DC.KillSpike(item, mo)
	local spikesToKill = {}
	
	if item.flags & MF_SOLID
		S_StartSound(mo, item.info.deathsound)
	end
	
	for obj in item.subsector.sector.thinglist()
		if (obj.type == item.type and obj.health > 0 and obj.flags & MF_SOLID
		and (obj == item or P_AproxDistance(P_AproxDistance(obj.x - item.x, obj.y - item.y), obj.z - item.z) < 56*item.scale))
			table.insert(spikesToKill, obj)
		end
	end
	
	for i, obj in ipairs(spikesToKill)
		P_KillMobj(obj, mo, mo.flags & MF_MISSILE and mo.target or mo)
	end
end

// returns true if @mo is a cactus)
function DC.MobjIsCactus(mo)
	return mo.type >= MT_CACTI1
	and mo.type <= MT_CACTISMALLSEG
end

// returns true if @mo is a destructable)
function DC.MobjIsDestructable(mo)
	return DC.MobjIsCactus(mo)
	or mo.type == MT_FALLINGROCK
	or mo.type == MT_TURRET
	or mo.type == MT_SPIKEBALL
	or mo.type == MT_BOMBSPHERE
	or mo.type == MT_SMASHINGSPIKEBALL
	or mo.type == MT_SMALLMACE
	or mo.type == MT_BIGMACE
	or mo.type == MT_EGGMOBILE4_MACE
	or mo.type == MT_EGGROBO1
	or mo.type == MT_EGGSHIELD
	or mo.type == MT_CRUSHCLAW
	or mo.type == MT_SHELL
	or mo.type == MT_SALOONDOOR
	or mo.type == MT_SALOONDOORCENTER
	or ((mo.type == MT_CRAWLASTATUE or mo.type == MT_FACESTABBERSTATUE)
	and not (mo.flags & MF_PUSHABLE))
end

// adds a destructable to the destructable list
function DC.AddDestructable(mo)
	table.insert(DC.destructables, mo)
end

// flashes the destructables Susie busts
function DC.UpdateDestructables()
	for i = #DC.destructables, 1, -1
		local destructable = DC.destructables[i]
		if not DC.Valid(destructable)
			table.remove(DC.destructables, i)
			return
		end
		destructable.flags2 = $ ^^ MF2_DONTDRAW
		destructable.momz = $ + P_GetMobjGravity(destructable)
	end
end

function DC.StartQuake(player, intensity, time)
	if player ~= displayplayer
	and player ~= secondarydisplayplayer
		return
	end
	P_StartQuake(intensity, time)
end

// returns the angle @player is travelling in
function DC.GetPlayerMomentumAngle(player)
	local mo = player.mo
	return R_PointToAngle2(0, 0, mo.momx - player.cmomx, mo.momy - player.cmomy)
end

// returns the realtime speed of @player
function DC.GetPlayerSpeed(player)
	local mo = player.mo
	return FixedHypot(mo.momx - player.cmomx, mo.momy - player.cmomy)
end

// returns the angle of @player's input
function DC.GetInputAngle(player)
	// return main angle if simple mode or 2D
	if (player.pflags&PF_DIRECTIONCHAR and player.pflags&PF_ANALOGMODE)
	or (player.mo and player.mo.flags2 & MF2_TWOD) or twodlevel
		return player.mo.angle
		
	// we're not moving, so just return the direction we're facing
	elseif not (player.cmd.forwardmove or player.cmd.sidemove)
		return player.drawangle
		
	// the actual input angle
	else 
		return (player.cmd.angleturn << 16) + R_PointToAngle2(0, 0, player.cmd.forwardmove << 16, -player.cmd.sidemove << 16)
	end
end

// sets the drawangle of @player if their control scheme benefits from it
function DC.SetPlayerDrawangle(player, angle)
	if player.pflags & PF_DIRECTIONCHAR // Standard & Simple
	and (not G_RingSlingerGametype() or player.pflags & PF_ANALOGMODE)
		player.drawangle = angle
	end
end

// adjusts @momz if @mo is underwater
function DC.MobjAdjustMomZ(mo, momz)
	if mo.eflags & MFE_UNDERWATER
		momz = FixedMul($, FixedDiv(117*FRACUNIT, 200*FRACUNIT))
	end
	return momz
end

// returns whether two players are friendly
function DC.PlayersAreFriendly(p1, p2)
	// players are never friendly if friendlyfire is on
	if DC.friendlyFire.value
		return false
	end
	
	// players are always friendly in friendly gametypes
	if gametyperules & GTR_FRIENDLY
		return true
	end
	
	// players on the same team are friendly
	if G_GametypeHasTeams()
	and p1.ctfteam == p2.ctfteam
	or G_TagGametype()
	and (p1.pflags & PF_TAGIT) == (p2.pflags & PF_TAGIT)
		return true
	end
	
	return false
end

// returns the top and bottom heights of @fof at the coordinates @x and @y
function DC.GetFOFHeightsAt(fof, x, y)
	local top
	local bottom
	
	if fof.t_slope
		top = P_GetZAt(fof.t_slope, x, y)
	else
		top = fof.topheight
	end
	
	if fof.b_slope
		bottom = P_GetZAt(fof.b_slope, x, y)
	else
		bottom = fof.bottomheight
	end
	
	return top, bottom
end

// returns true if @fof can be activated by spindash
function DC.IsSpindashFOF(fof)
	local master = fof.master
	local special = master.special
	
	if not (master.flags & ML_NOCLIMB)
		return false
	end
	
	return special >= 190 and special <= 195 // rising platforms
	or special >= 150 and special <= 153 // air bobbing platforms
	or special == 176 or special == 177 or special == 180 // crumbling floating bobbing scrimblo bimbo (what the fuck are these)
end

// returns true if @player is touching an FOF than can be activated by a spindash
function DC.TouchingSpindashFOF(player)
	local mo = player.mo
	local sector = mo.subsector.sector
	
	for fof in sector.ffloors()
		if not (fof.flags & FF_EXISTS)
		or not DC.IsSpindashFOF(fof)
			continue
		end
		
		local top, bottom = DC.GetFOFHeightsAt(fof, mo.x, mo.y)
		
		if mo.z <= top
		and mo.z + mo.height >= bottom
			return true
		end
	end
	return false
end

// returns whether or not this point is considered in front of @player
function DC.PointInFront(player, x, y)
	local mo = player.mo
	local playerX = mo.x - mo.momx
	local playerY = mo.y - mo.momy
	local itemDirection = R_PointToAngle2(playerX, playerY, x, y)
	
	return abs(player.drawangle - itemDirection) < ANGLE_90
end

// returns the closest point on a line, taking its vertices into account
function DC.ClosestPointOnLine(line, x, y)
	local v1 = line.v1
	local v2 = line.v2
	local minX = min(v1.x, v2.x)
	local maxX = max(v1.x, v2.x)
	local minY = min(v1.y, v2.y)
	local maxY = max(v1.y, v2.y)
	
	x, y = P_ClosestPointOnLine(x, y, line)
	
	x = min(max($, minX), maxX)
	y = min(max($, minY), maxY)
	
	return x, y
end

// busts any breakable blocks within @mo's height range in the sector on the other side of @line
function DC.MobjBustFOFsThroughLine(mo, line, weak)
	local sector
	
	if P_PointOnLineSide(mo.x - mo.momx, mo.y - mo.momy, line) == 1
		sector = line.frontsector
	else
		sector = line.backsector
	end
	
	if sector == nil
		return
	end
	
	DC.MobjBustFOFsInSector(mo, mo.z, mo.z + mo.height, sector, weak)
end

// has @mo bust any breakable blocks between @bottom and @top in @sector
function DC.MobjBustFOFsInSector(mo, bottom, top, sector, weak)
	for fof in sector.ffloors()
		if fof.flags & (FF_EXISTS|FF_BUSTUP) ~= (FF_EXISTS|FF_BUSTUP)
		or weak and fof.flags & FF_STRONGBUST
			continue
		end
		
		local fofTop, fofBottom = DC.GetFOFHeightsAt(fof, mo.x, mo.y)
		
		if bottom < fofTop
		and top > fofBottom
			DC.MobjBustFOF(mo, fof)
		end
	end
end

// checks for bustable FOFs above or below @player and busts them when close enough
function DC.CheckVerticalBustables(player)
	local mo = player.mo
	
	for fof in mo.subsector.sector.ffloors()
		if fof.flags & (FF_EXISTS|FF_BUSTUP) ~= (FF_EXISTS|FF_BUSTUP)
			continue
		end
		
		local topZ
		local bottomZ
		
		if fof.t_slope
			topZ = P_GetZAt(fof.t_slope, mo.x, mo.y)
		else
			topZ = fof.topheight
		end
		if fof.b_slope
			bottomZ = P_GetZAt(fof.b_slope, mo.x, mo.y)
		else
			bottomZ = fof.bottomheight
		end
		
		if mo.z + mo.momz <= topZ
		and mo.z + mo.height + mo.momz >= bottomZ
			DC.MobjBustFOF(mo, fof)
		end
	end
end

// makes @mobj crumble @fof
function DC.MobjBustFOF(mo, fof)
	local line = fof.master
	
	// bust the FOF
	EV_CrumbleChain(fof)
	
	// run a linedef executor?
	if line.flags & ML_EFFECT5
		P_LinedefExecute(P_AproxDistance(line.dx, line.dy) >> FRACBITS, mo, fof.target)
	end
end

// spawns rocks like EV_CrumbleChain from source code, but for the full area of a sector instead of the area of an FOF
function DC.CrumbleSector(sector, mobjType, spacing, lifetime, fromCenter)
	if not mobjType
		mobjType = MT_ROCKCRUMBLE1
	end
	if not spacing
		spacing = 32*FRACUNIT
	end
	if lifetime == -1
		lifetime = 0
	elseif not lifetime
		lifetime = 3*TICRATE
	end
	
	local halfSpacing = (spacing / 2)
	local topZ = sector.ceilingheight
	local bottomZ = sector.floorheight
	local spawned, centerX, centerY, centerZ, widthFactor, heightFactor
	
	// I actually think the source code's method of finding sector bounds is flawed, so this is slightly different
	local leftX = INT32_MAX
	local rightX = INT32_MIN
	local bottomY = INT32_MAX
	local topY = INT32_MIN
	for i = 0, #sector.lines - 1
		local line = sector.lines[i]
		
		for v = 1, 2
			local vertex = line["v"..v]
			
			if vertex.x < leftX
				leftX = vertex.x
			end
			if vertex.x > rightX
				rightX = vertex.x
			end
			if vertex.y < bottomY
				bottomY = vertex.y
			end
			if vertex.y > topY
				topY = vertex.y
			end
		end
	end
	
	centerX = leftX/2 + rightX/2
	centerY = topY/2 + bottomY/2
	centerZ = topZ/2 + bottomZ/2
	
	leftX = $ + halfSpacing
	topY = $ - halfSpacing
	topZ = $ - halfSpacing
	
	if fromCenter
		widthFactor = (rightX + topY - leftX - bottomY) / 8
		heightFactor = (topZ - bottomZ) / 4
	end
	
	local soundPlayer = P_SpawnMobj(centerX, centerY, centerZ, MT_RAY)
	soundPlayer.tics = 3*TICRATE
	S_StartSound(soundPlayer, mobjinfo[mobjType].activesound)
	
	for x = leftX, rightX, spacing
		for y = topY, bottomY, -spacing
			if R_PointInSubsector(x, y).sector ~= sector
				continue
			end
			
			if sector.c_slope
				topZ = P_GetZAt(sector.c_slope, x, y) - halfSpacing
			end
			if sector.f_slope
				bottomZ = P_GetZAt(sector.f_slope, x, y)
			end
			
			for z = topZ, bottomZ, -spacing
				spawned = P_SpawnMobj(x, y, z, mobjType)
				spawned.angle = P_RandomKey(36)*ANG10
				
				if fromCenter
					P_InstaThrust(spawned, R_PointToAngle2(centerX, centerY, x, y), FixedDiv(P_AproxDistance(x - centerX, y - centerY), widthFactor))
					P_SetObjectMomZ(spawned, FixedDiv(z - bottomZ, heightFactor), false)
				end
				
				spawned.fuse = lifetime
			end
		end
	end
end

// P_GetMinecartSector port
function DC.GetMinecartSector(x, y, z)
	local sector = R_PointInSubsector(x, y).sector
	
	if sector.ceilingheight - sector.floorheight < 64*FRACUNIT
		return nil
	end
	
	for fof in sector.ffloors()
		if not (fof.flags & (FF_EXISTS|FF_BLOCKOTHERS)) // (is this supposed to be a check for both flags????)
			continue
		end
		
		local top = DC.GetFOFHeightsAt(fof, x, y)
		
		if abs(z - top) <= 56*FRACUNIT
			return fof.sector, top
		end
	end
	
	local floor
	if sector.f_slope
		floor = P_GetZAt(sector.f_slope, x, y)
	else
		floor = sector.floorheight
	end
	
	if abs(z - floor) > 56*FRACUNIT
		return nil
	end
	
	return sector, floor
end

// gets the tag of the Minecart parameters line @player is currently using
function DC.GetMinecartTag(player)
	local mo = player.mo
	local cart = mo.tracer
	
	if not DC.Valid(cart)
		return 0
	end
	
	local sector = DC.GetMinecartSector(cart.x, cart.y, cart.z)
	
	if not sector
		return 0
	end
	
	local lines = sector.lines
	
	for i = 0, #lines - 1
		local line = lines[i]
		if line.special == 16 and line.tag == 0 and line.frontsector == sector
			return line.frontside.textureoffset >> FRACBITS
		end
	end
	
	if sector.tag ~= 0
	and P_FindSpecialLineFromTag(16, sector.tag)
		return sector.tag
	end
	
	return 0
end

function DC.CycleStateAnimation(mo)
    if not (mo.frame & FF_ANIMATE)
        return
    end

    mo.anim_duration = $ - 1
    if mo.anim_duration ~= 0
        return
    end

    local state = states[mo.state]
    local stateFrame = (state.frame & FF_FRAMEMASK)

    mo.anim_duration = state.var2
    mo.frame = $ + 1
    if (mo.frame & FF_FRAMEMASK) - stateFrame > state.var1
        mo.frame = ($ & ~FF_FRAMEMASK) | stateFrame
    end
end

function DC.CycleMobjState(mo)
    DC.CycleStateAnimation(mo)

    if mo.tics ~= -1
        mo.tics = $ - 1
        if mo.tics == 0
            mo.state = states[mo.state].nextstate
        end
    end
    return mo.valid
end

function DC.AdvanceState(mo, tics)
    while tics > 0
        if not DC.CycleMobjState(mo)
            return
        end
        tics = $ - 1
    end
end

// --------------
// hook functions
// --------------

function DC.ThinkFrame()
	if titlemapinaction
		return
	end
	
	if DC.creditsWait
		DC.creditsWait = $ - 1
		if not DC.creditsWait
			S_StartSound(nil, sfx_dcsave)
			print("\143DeltaChars v1\128 - \136Sprites by DrStephen, Lightdasher, and HattyBoyo\128 - \129Code by Lach, SMS Alfredo, and Frostiikin",
			"\131Use the \133dc_help \131command to see all of the mod's commands!")
		end
	end
	
	// load preference for Susie's ax angle
	DC.AxAngle.CheckConfig()
	
	// update players
	for player in players.iterate
		if not DC.CheckStatus(player)
			continue
		end
		DC.PreUpdatePlayer(player)
		for i, library in ipairs(DC.libraries)
			library.UpdatePlayer(player)
		end
		DC.PostUpdatePlayer(player)
	end
	
	// flash busted destructables
	DC.UpdateDestructables()
end

function DC.NetVars(net)
	DC.destructables = net(DC.destructables)
	DC.secretHP = net(DC.secretHP)
end

function DC.PlayerMobjDamage(mo)
	local player = mo.player
	if player and DC.CheckStatus(player)
	and not player.powers[pw_shield]
	and not player.powers[pw_flashing]
		S_StartSound(mo, sfx_dchurt)
	end
end

function DC.SpawnPitDummy(player)
	local mo = player.mo
	
	local dummy = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_GHOST)
	dummy.skin = mo.skin
	dummy.color = player.skincolor
	dummy.flags = $ & ~(MF_SCENERY|MF_NOGRAVITY) // non-scenery ghosts will not be killed by death pits. I don't know why it's coded that way.
	dummy.momx, dummy.momy, dummy.momz = mo.momx, mo.momy, mo.momz
	dummy.z = $ + dummy.momz
	dummy.fuse = 5*TICRATE
	
	//SMS Alfredo's painful hat of death
	if player.ralseihat
		dummy.eflags = $ | MFE_FORCESUPER
	end
	dummy.state = S_PLAY_DEAD
	
	S_StartSound(mo, sfx_dcfall)
end

function DC.PlayerMobjDeath(mo, inf, source, damageType)
	local player = mo.player
	
	if not player
	or not DC.CheckStatus(player)
		return
	end
	
	if damageType == DMG_DEATHPIT // killed by death pit
		DC.SpawnPitDummy(player)
		player.deltaChars.deathType = DC.DEATHS.PIT
	elseif damageType == DMG_DROWNED // drowning
	or damageType == DMG_SPACEDROWN
		player.deltaChars.deathType = DC.DEATHS.DROWN
	elseif inf == mo // killed by self
		player.deltaChars.deathType = DC.DEATHS.SELF
	elseif DC.Valid(inf) // killed by object
		player.drawangle = R_PointToAngle2(mo.x, mo.y, inf.x, inf.y)
	else // killed by sector effect
		player.drawangle = R_PointToAngle2(0, 0, mo.momx, mo.momy)
	end
	
	// killed by fire
	if damageType == DMG_FIRE or damageType == DMG_ELECTRIC
		player.deltaChars.deathType = DC.DEATHS.FIRE
	end
end

function DC.MobjDeath(item, ...) // holy shit, why
	if not item.valid
	or not (item.flags & MF_MONITOR)
		return
	end
	
	return DC.RunHook(MT_NULL, "MobjDamage", item, ...)
	// (ordinarily we should also run the hook for the mobj type, but I don't predict ever hooking to specific monitor types)
end

function DC.MapLoad()
	DC.secretHP = DC.MAX_SECRET_HP
end

// -----
// hooks
// -----

DC.AddHook("ThinkFrame", DC.ThinkFrame)
DC.AddHook("NetVars", DC.NetVars)
DC.AddHook("MapLoad", DC.MapLoad)

DC.AddHook("PlayerSpawn", DC.RemoveTable)

DC.AddHook("MobjDamage", DC.PlayerMobjDamage, MT_PLAYER)
DC.AddHook("MobjDeath", DC.PlayerMobjDeath, MT_PLAYER)

DC.AddHook("MobjDeath", DC.MobjDeath)