/*
DeltaChars hook system

v1
*/

local DC = DeltaChars

DC.hooks = {}

// runs all of the hooks associated with @hookType
DC.RunHook = function(key, hookType, ...)
	local returnValue
	for i, func in ipairs(DC.hooks[key][hookType])
		local ret = func(...)
		if ret ~= nil
			returnValue = ret
		end
	end
	return returnValue
end

// hooks @func so that it is called when @hookType is run
DC.AddHook = function(hookType, func, extraArg)
	local key = extraArg or MT_NULL
	local hookExists = false
	local hooks = DC.hooks[key]
	
	// if necessary, create hook table for this string/object type
	if not hooks
		hooks = {}
		DC.hooks[key] = hooks
	end
	
	// if necessary, create a new table in the hook table for this hook type, and hook it
	if not hooks[hookType]
		hooks[hookType] = {}
		if not DC.customHookTypes[hookType]
			addHook(hookType, function(...)
				return DC.RunHook(key, hookType, ...)
			end, extraArg)
		end
	end
	
	// add this function to the hook's list
	table.insert(hooks[hookType], func)
end

// registers a nonvanilla hook type
DC.customHookTypes = {}
function DC.RegisterCustomHook(hookType)
	DC.customHookTypes[hookType] = true
end

// attempts to run a hook; if nothing is hooked to @hookType with @key, nothing runs
// intended for running hooked functions outside of hooks, such as with custom hooks
function DC.TryRunHook(key, hookType, ...)
	local hooks = DC.hooks[key]
	if not (hooks and hooks[hookType])
		return
	end
	
	return DC.RunHook(key, hookType, ...)
end
