/*
Kris Cross Apple Sauce

v1
*/

local DC = DeltaChars
local Kris = {}

DC.Kris = Kris

// ----
// defs
// ----

Kris.SKIN = "kris"

DC.FLAGS.KRIS_MASK = DC.RegisterFlag("SLIDING")
	//| DC.RegisterFlag("RENEW_SLIDE")
	| DC.RegisterFlag("SPINDASH")
	| DC.RegisterFlag("LUNGING")

Kris.SLIDE_START_SPEED = 10*FRACUNIT
Kris.SLIDE_MIN_SPEED = FRACUNIT
Kris.SLIDE_BOOST = 16*FRACUNIT
Kris.SLIDE_BOOST_COOLDOWN = 20

Kris.CROSS_SLASHES = 2
Kris.CROSS_SLASH_FRAMES = 4
Kris.CROSS_SLASH_HOP = 5*FRACUNIT
Kris.CROSS_SLASH_HOP_TICS = 3

//States to cancel out of when switching out of this character
Kris.CancelStates = {}
local statelist = {
	S_KRIS_SLIDE, S_KRIS_LUNGE, S_KRIS_CROSSSLASH
}
for i, state in ipairs(statelist)
	Kris.CancelStates[state] = true
end

// ---------
// functions
// ---------

function Kris.GetSkinIndex()
	if not Kris.skinIndex
		Kris.skinIndex = #skins[Kris.SKIN]
	end
	return Kris.skinIndex
end

function Kris.CreateTable(player)
	player.deltaChars.kris = {
		slideCooldown = 0,
		slashes = 0,
		slashSide = 0
	}
end

function Kris.RemoveTable(player)
	local delta = player.deltaChars
	
	if delta.kris == nil
		return
	end
	
	local mo = player.mo
	
	if delta.flags & DC.FLAGS.SLIDING
		Kris.StopSlide(player)
	end
	
	if delta.flags & DC.FLAGS.FUNNY
		P_RestoreMusic(player)
	end
	
	if mo.state == S_KRIS_LUNGE
		mo.state = S_DC_TERMINATE
	end
	delta.flags = $ & ~DC.FLAGS.KRIS_MASK
	delta.kris = nil
	
	return true
end

function Kris.UpdatePlayer(player)
	if player.playerstate ~= PST_LIVE
	or player.skin ~= Kris.GetSkinIndex()
		return
	end
	
	Kris.UpdateVars(player)
	Kris.HandleSlide_ThinkFrame(player)
	Kris.HandleLunge(player)
	Kris.HandleEffects(player)
end

function Kris.UpdateVars(player)
	local mo = player.mo
	local delta = player.deltaChars
	local kris = delta.kris
	
	if not delta.prevGrounded
	and P_IsObjectOnGround(mo)
	or delta.prevCarry == CR_NONE
	and player.powers[pw_carry] ~= CR_NONE
		kris.slashes = 0
	end
end

// returns whether or not @player can slide
function Kris.CanSlide(player)
	return not (player.pflags & (DC.PFLAGS_BLACKLIST & ~(PF_STASIS|PF_STARTDASH) | PF_JUMPED))
	and player.powers[pw_carry] == CR_NONE
	and not P_PlayerInPain(player)
	and not (player.deltaChars.flags & DC.FLAGS.FUNNY)
end

// returns whether or not @player can continue sliding
function Kris.CanContinueSlide(player)
	local mo = player.mo
	local delta = player.deltaChars
	local validState = (mo.state == S_KRIS_SLIDE or mo.eflags & MFE_SPRUNG and P_IsObjectOnGround(mo))
	
	// if you're under a gap, you should UHHHH probably stay sliding
	if mo.ceilingz - mo.floorz < P_GetPlayerHeight(player)
	and validState
		return true
	end
	
	// otherwise, just make sure you're holding spin and maintaining slide status
	if not (DC.ButtonDown(player, BT_SPIN, true) or delta.kris.slideCooldown > 0)
	or not validState
	or not (delta.flags & DC.FLAGS.SPINDASH)
		and P_IsObjectOnGround(mo)
		and (not mo.standingslope or abs(mo.standingslope.zdelta) < FRACUNIT/2)
		and DC.GetPlayerSpeed(player) < FixedMul(Kris.SLIDE_MIN_SPEED, mo.scale)
	or not Kris.CanSlide(player)
		return false
	end
	return true
end

// starts a slide
function Kris.StartSlide(player)
	local mo = player.mo
	local delta = player.deltaChars
	local kris = delta.kris
	
	local speed = DC.GetPlayerSpeed(player)
	local fastEnough = speed >= FixedMul(Kris.SLIDE_MIN_SPEED, mo.scale)
	local spin = DC.ButtonDown(player, BT_SPIN, true)
	local start = spin == 1 and (kris.slideCooldown == 0 or fastEnough)
	local landed = spin and not delta.prevGrounded and fastEnough
	local canSlide = (start or landed) and P_IsObjectOnGround(mo) and Kris.CanSlide(player)
	
	if not canSlide
		return
	end
	
	// counteract halving of speed when landing on the ground without pressing controls
	if landed
	and not (player.cmd.forwardmove or player.cmd.sidemove or player.cmomx or player.cmomy)
		mo.momx = $ * 2
		mo.momy = $ * 2
	end
		
	// start slide
	if start
		local boost = FixedMul(Kris.SLIDE_BOOST, mo.scale)
		local angle
		if speed >= 5*mo.scale
			angle = DC.GetPlayerMomentumAngle(player)
		elseif fastEnough
			angle = player.drawangle
		else
			angle = mo.angle
		end
		S_StartSound(mo, sfx_dcdash)
		if not (player.gotflag
		and speed + boost >= FixedMul(player.normalspeed, mo.scale))
			P_Thrust(mo, angle, boost)
		end
		kris.slideCooldown = Kris.SLIDE_BOOST_COOLDOWN
	end
	
	delta.flags = $ | DC.FLAGS.SLIDING
	player.charflags = $ | SF_NOSKID
	mo.state = S_KRIS_SLIDE
	S_StartSound(mo, sfx_dcdyin)
	Kris.DoSliding(player)
	DC.SetPlayerDrawangle(player, DC.GetPlayerMomentumAngle(player))
	
	// don't get cockblocked by gaps right in front of you
	local height = Kris.PlayerHeight(player)
	if height ~= nil
		mo.height = height
	end
end

// resets Kris after a slide
function Kris.StopSlide(player)
	local delta = player.deltaChars
	local mo = player.mo
	local skin = skins[player.skin]
	
	delta.flags = $ & ~(DC.FLAGS.SLIDING|DC.FLAGS.SPINDASH)
	player.pflags = $ & ~(PF_STARTDASH|PF_JUMPSTASIS)
	if not DC.ButtonDown(player, BT_JUMP, true, true)
		player.pflags = $ & ~PF_JUMPDOWN
	end
	player.normalspeed = skin.normalspeed
	player.thrustfactor = skin.thrustfactor
	player.charflags = $ & ~SF_NOSKID | (skin.flags & SF_NOSKID)
	S_StopSoundByID(mo, sfx_dcslid)
	if mo.state == S_KRIS_SLIDE
		mo.state = S_DC_TERMINATE
	end
end

// excerpt from P_ButteredSlope to mimic spinning physics on slopes
function Kris.ButteredSlope(player)
	local mo = player.mo
	local slope = mo.standingslope
	
	mo.friction = FixedDiv(549*DC.ORIG_FRICTION, 500*FRACUNIT) // spinning friction
	
	if not slope
	or slope.flags & SL_NOPHYSICS
	or mo.flags & (MF_NOGRAVITY|MF_NOCLIPHEIGHT)
		return
	end
	
	local flip = P_MobjFlip(mo)
	local grav = abs(P_GetMobjGravity(mo))
	local xtra = P_AproxDistance(mo.momx, mo.momy)/16
	local frictionFactor = FixedDiv(mo.friction, DC.ORIG_FRICTION)
	
	local thrust = sin(slope.zangle) * 3 / 2 * -flip
	local appliedThrust = thrust
	
	local mult
	local angle = R_PointToAngle2(0, 0, mo.momx, mo.momy) - slope.xydirection
	
	if flip * slope.zdelta < 0
		angle = $ + ANGLE_180
	end
	
	mult = cos(angle)
	thrust = FixedMul($, FRACUNIT*2/3 + mult/8)
	
	thrust = FixedMul(FixedMul($, FRACUNIT + xtra), grav)
	
	if abs(slope.zdelta) >= FRACUNIT/4
		appliedThrust = FixedMul(FixedMul(FixedMul($, FRACUNIT + xtra), grav), frictionFactor)
		thrust = $ - appliedThrust
	end
	
	P_Thrust(mo, slope.xydirection, thrust)
end

// forces certain values during a slide
function Kris.DoSliding(player)
	player.normalspeed = 0
	player.pflags = $ | PF_JUMPSTASIS | PF_JUMPDOWN
	player.thrustfactor = 1 // minimal control
	
	Kris.ButteredSlope(player)
end

// plays sliding sounds based on @player's groundedness
function Kris.SlideSounds(player)
	local mo = player.mo
	if not P_IsObjectOnGround(mo)
		if player.deltaChars.prevGrounded
			S_StopSoundByID(mo, sfx_dcslid)
		end
	elseif not S_SoundPlaying(mo, sfx_dcslid)
		S_StartSound(mo, sfx_dcslid)
	end
end

// sliding, part 1 (for PlayerThink)
function Kris.HandleSlide_PlayerThink(player)
	local delta = player.deltaChars
	
	// not sliding? gtfo
	if not (delta.flags & DC.FLAGS.SLIDING)
		return
	end
	
	// shouldn't be sliding? stop
	if not Kris.CanContinueSlide(player)
		Kris.StopSlide(player)
		return
	end
	
	local mo = player.mo
	
	if DC.ButtonDown(player, BT_JUMP)
		if delta.flags & DC.FLAGS.SPINDASH
			P_Thrust(mo, mo.angle, FixedMul(Kris.SLIDE_START_SPEED, mo.scale))
		end
		Kris.StopSlide(player)
		Kris.Lunge(player)
		return
	end
	
	local kris = delta.kris
	
	// activate spindash platforms
	if DC.TouchingSpindashFOF(player)
		player.pflags = $ | PF_STARTDASH
		delta.flags = $ | DC.FLAGS.SPINDASH
		mo.momx = player.cmomx
		mo.momy = player.cmomy
	else
		delta.flags = $ & ~DC.FLAGS.SPINDASH
	end
	
	// skipping stone!
	if not delta.prevWater and delta.prevWater ~= mo.eflags & MFE_UNDERWATER
	and P_MobjFlip(mo)*mo.momz < 0 and player.speed/2 > abs(mo.momz)
	and ((not (mo.eflags & MFE_VERTICALFLIP) and mo.z + mo.height - mo.momz > mo.watertop)
	or ((mo.eflags & MFE_VERTICALFLIP) and mo.z - mo.momz < mo.waterbottom))
		mo.momz = -$/2

		if not (mo.eflags & MFE_VERTICALFLIP) and mo.momz > 6*mo.scale
			mo.momz = 6*mo.scale
		elseif mo.eflags & MFE_VERTICALFLIP and mo.momz < -6*mo.scale
			mo.momz = -6*mo.scale
		end
	end
	
	// force these values while sliding
	Kris.DoSliding(player)
	if DC.GetPlayerSpeed(player) >= FixedMul(Kris.SLIDE_MIN_SPEED, mo.scale)
		delta.lockAngle = DC.GetPlayerMomentumAngle(player)
	end
	DC.SetPlayerDrawangle(player, delta.lockAngle)
end

// sliding, part 2 (for ThinkFrame)
function Kris.HandleSlide_ThinkFrame(player)
	local mo = player.mo
	local delta = player.deltaChars
	local kris = delta.kris
	
	if kris.slideCooldown > 0
		kris.slideCooldown = $ - 1
	end
	
	if delta.flags & DC.FLAGS.SLIDING
		// shouldn't be sliding? stop
		if not Kris.CanContinueSlide(player)
			Kris.StopSlide(player)
		else
			local speed = DC.GetPlayerSpeed(player)
			local fastEnough = speed >= FixedMul(Kris.SLIDE_MIN_SPEED, mo.scale)
			
			// counteract halving of speed when landing on the ground without pressing controls
			if not delta.prevGrounded and P_IsObjectOnGround(mo)
			and DC.ButtonDown(player, BT_SPIN, true)
			and not (player.cmd.forwardmove or player.cmd.sidemove or player.cmomx or player.cmomy)
				mo.momx = $ * 2
				mo.momy = $ * 2
			end
			
			// don't get stuck
			if mo.ceilingz - mo.floorz < P_GetPlayerHeight(player)
			and not fastEnough
				P_Thrust(mo, mo.angle, FixedMul(Kris.SLIDE_START_SPEED, mo.scale))
			end
			
			if mo.eflags & MFE_SPRUNG
				mo.state = S_KRIS_SLIDE
			end
			player.pflags = $ & ~PF_STARTDASH
			if fastEnough
				delta.lockAngle = DC.GetPlayerMomentumAngle(player)
			end
			DC.SetPlayerDrawangle(player, delta.lockAngle)
			Kris.SlideSounds(player)
		end
		return
	end
	
	Kris.StartSlide(player)
end

function Kris.HandleEffects(player)
	local mo = player.mo
	local delta = player.deltaChars
	local kris = delta.kris
	
	// dust
	if delta.flags & DC.FLAGS.SLIDING
	and not (mo.eflags & MFE_GOOWATER)
	and (leveltime % 6 == 0)
	and P_IsObjectOnGround(mo)
		local dust = DC.SpawnDust(player)
		if delta.flags & DC.FLAGS.SPINDASH
			P_Thrust(dust, player.drawangle, -FixedMul(Kris.SLIDE_START_SPEED, mo.scale))
		end
	end
	
	// afterimages
	if kris.slideCooldown
	or delta.flags & DC.FLAGS.LUNGING
		if (leveltime % 5) == 4
			local ghost = P_SpawnGhostMobj(mo)
			ghost.colorized = true
			ghost.tics = 6
			if delta.flags & DC.FLAGS.SPINDASH
				P_Thrust(ghost, player.drawangle, -FixedMul(Kris.SLIDE_START_SPEED, mo.scale))
			end
		end
	end
end

// starts a lunge
function Kris.Lunge(player)
	local delta = player.deltaChars
	local mo = player.mo
	
	P_DoJump(player, false)
	player.pflags = $ | PF_JUMPDOWN
	delta.flags = $ | DC.FLAGS.LUNGING
	mo.state = S_KRIS_LUNGE
	S_StartSound(mo, sfx_dcjump)
	if FixedHypot(player.cmd.forwardmove, player.cmd.sidemove) > 10
		P_InstaThrust(mo, DC.GetInputAngle(player), DC.GetPlayerSpeed(player))
	end
	DC.SetPlayerDrawangle(player, DC.GetPlayerMomentumAngle(player))
end

// turns Kris towards their target during a lunge attack
function Kris.HandleLunge(player)
	local delta = player.deltaChars
	
	if not (delta.flags & DC.FLAGS.LUNGING)
		return
	end
	
	local mo = player.mo
	
	if mo.state == S_KRIS_CROSSSLASH
		DC.SetPlayerDrawangle(player, delta.lockAngle)
		return
	end
	
	if mo.state ~= S_KRIS_LUNGE
		delta.flags = $ & ~DC.FLAGS.LUNGING
	end
end

// returns true if @player is in a state that could attack @item
function Kris.CanAttack(player, item)
	local state = player.mo.state
	
	// states that require enemies to be in front of the player
	if state == S_KRIS_SLIDE
		//return item == nil or DC.PointInFront(player, item.x, item.y)
		return true // just kidding, I'll let you damage at any angle
	end
	
	// states that can damage enemies at any angle
	if state == S_KRIS_CROSSSLASH
	or state == S_KRIS_LUNGE
		return true
	end
	
	return false
end

// returns true if @player can bust fofs
// if @x and @y are given, takes the coordinates into account
function Kris.CanBust(player, x, y)
	local state = player.mo.state
	
	if state == S_KRIS_SLIDE
	or state == S_KRIS_CROSSSLASH
		return y == nil or DC.PointInFront(player, x, y)
	end
	
	return false
end

function Kris.SpawnDamageEffect(player, item)
	local mo = player.mo
	local effect = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_THOK)
	effect.state = S_DC_SLASH
	effect.spritexscale = 2*$
	effect.spriteyscale = effect.spritexscale
	if player.deltaChars.kris.slashSide
		effect.renderflags = $ | RF_HORIZONTALFLIP
	end
	P_TeleportMove(effect, mo.x/2 + item.x/2, mo.y/2 + item.y/2, effect.z)
	S_StartSound(mo, sfx_dcdmg)
end

// --------------
// hook functions
// --------------

function Kris.PlayerThink(player)
	if player.skin ~= Kris.GetSkinIndex()
	or not DC.CheckStatus(player)
	or player.playerstate ~= PST_LIVE
		return
	end
	
	Kris.HandleSlide_PlayerThink(player)
end

function Kris.AbilitySpecial(player)
	if player.skin ~= Kris.GetSkinIndex()
	or not DC.CheckStatus(player)
	or player.charability ~= skins[player.skin].ability
		return
	end
	
	local mo = player.mo
	local delta = player.deltaChars
	local kris = delta.kris
	
	if not (player.pflags & PF_THOKKED)
		kris.slashes = Kris.CROSS_SLASHES
		player.pflags = $ & ~PF_SPINNING | PF_THOKKED | PF_NOJUMPDAMAGE
	end
	
	// obviously, no cross slash if you have none left to use
	if kris.slashes == 0
		return true
	end
	
	if mo.state == S_KRIS_CROSSSLASH
		if (mo.frame & FF_FRAMEMASK) % Kris.CROSS_SLASH_FRAMES == (Kris.CROSS_SLASH_FRAMES - 1)
			mo.state = S_PLAY_FALL
		else
			return true
		end
	end
	
	mo.state = S_KRIS_CROSSSLASH
	kris.slashes = $ - 1
	delta.flags = $ & ~DC.FLAGS.LUNGING
	
	return true
end

function Kris.PlayerHeight(player)
	local mo = player.mo
	if player.mo.state == S_KRIS_SLIDE
		return P_GetPlayerSpinHeight(player)
	end
end

function Kris.PlayerCanDamage(player, item)
	if not item.valid
		return
	end
	
	if Kris.CanAttack(player, item)
		return true
	end
end

function Kris.PlayerMobjCollide(mo, item) // (disabled)
	local player = mo.player
	
	if not item.valid
	or not player
	or player.skin ~= Kris.GetSkinIndex()
	or not DC.CheckStatus(player)
	or mo.z > item.z + item.height
	or item.z > mo.z + mo.height
		return
	end
	
	if item.flags & MF_SPRING
	and DC.ButtonDown(player, BT_SPIN, true)
		player.deltaChars.flags = $ | DC.FLAGS.RENEW_SLIDE
	end
end

function Kris.PlayerMobjLineCollide(mo, line)
	local player = mo.player
	
	if not player
	or player.skin ~= Kris.GetSkinIndex()
	or not Kris.CanBust(player)
		return
	end
	
	local x, y = DC.ClosestPointOnLine(line, mo.x, mo.y)
	
	if not Kris.CanBust(player, x, y)
		return
	end
	
	DC.MobjBustFOFsThroughLine(mo, line, true)
end

function Kris.MobjDamage(item, mo)
	local player = DC.Valid(mo) and mo.player
	
	if not player
	or player.skin ~= Kris.GetSkinIndex()
	or not DC.CheckStatus(player)
	or not item.valid
		return
	end
	
	local delta = player.deltaChars
	
	if mo.state == S_KRIS_LUNGE
		mo.state = S_KRIS_CROSSSLASH
	end
	
	if mo.state == S_KRIS_CROSSSLASH
		if delta.flags & DC.FLAGS.LUNGING
			delta.lockAngle = R_PointToAngle2(mo.x, mo.y, item.x, item.y)
		end
		Kris.SpawnDamageEffect(player, item)
	end
end

// -----
// hooks
// -----

DC.AddHook("PlayerThink", Kris.PlayerThink)
DC.AddHook("PlayerHeight", Kris.PlayerHeight)
DC.AddHook("PlayerCanEnterSpinGaps", Kris.PlayerHeight)
DC.AddHook("PlayerCanDamage", Kris.PlayerCanDamage)
DC.AddHook("AbilitySpecial", Kris.AbilitySpecial)

//DC.AddHook("MobjCollide", Kris.PlayerMobjCollide, MT_PLAYER)
//DC.AddHook("MobjMoveCollide", Kris.PlayerMobjCollide, MT_PLAYER)
DC.AddHook("MobjLineCollide", Kris.PlayerMobjLineCollide, MT_PLAYER)

DC.AddHook("MobjDamage", Kris.MobjDamage)

DC.AddMember(Kris)