/*
Lancer mode

v1
*/

local DC = DeltaChars
local Lancer = {}

DC.Lancer = Lancer

// ---------
// constants
// ---------

Lancer.TOP_SPEED = FRACUNIT >> 1 - 1
Lancer.BOOST_SPEED = FRACUNIT >> 3

// ---------
// functions
// ---------

// returns whether @player is in a NiGHTS capsule
function Lancer.InCapsule(player)
	local capsule = player.capsule
	
	return DC.Valid(capsule)
		and capsule.reactiontime > 0
		and player == players[capsule.reactiontime - 1]
end

// returns the camera angle during NiGHTS mode
function Lancer.GetCameraAngle(player)
	if player.pflags & PF_TRANSFERTOCLOSEST
		local axis1, axis2 = player.axis1, player.axis2
		if axis1 and axis2
			return R_PointToAngle2(axis1.x, axis1.y, axis2.x, axis2.y) + ANGLE_90
		end
	end
	
	local mo = player.mo
	local target = mo.target
	if mo.target
		if target.flags2 & MF2_AMBUSH
			return R_PointToAngle2(target.x, target.y, mo.x, mo.y)
		end
		return R_PointToAngle2(mo.x, mo.y, target.x, target.y)
	end
	
	return mo.angle + ANGLE_90
end

// returns whether Lancer can use the explosion boost
function Lancer.CanBoost(player)
	if player.powers[pw_flashing] > 2*flashingtics/3
	or Lancer.InCapsule(player)
		return false
	end
	return player.drillmeter >= 20
end

function Lancer.MobjThinker(mo)
	local player = mo.target and mo.target.player
	if player and player.powers[pw_carry] == CR_NIGHTSMODE and not player.exiting
		P_RemoveMobj(mo)
		return true
	end
	
	if P_IsObjectOnGround(mo)
		if not (mo.flags2 & MF2_SLIDEPUSH)
			mo.flags2 = $ | MF2_SLIDEPUSH
			mo.state = mo.info.painstate
			mo.spriteyscale = $ >> 1
			mo.spritexscale = $ << 1
		end
	else
		mo.flags2 = $ & ~MF2_SLIDEPUSH
	end
	
	if mo.spritexscale > FRACUNIT
		mo.spritexscale = max(FRACUNIT, $ - (FRACUNIT >> 4))
	end
	if mo.spriteyscale < FRACUNIT
		mo.spriteyscale = min(FRACUNIT, $ + (FRACUNIT >> 4))
	end
end

function Lancer.PlayerThink(player)
	if player.powers[pw_carry] ~= CR_NIGHTSMODE
	or Lancer.InCapsule(player)
	or not DC.CheckStatus(player)
	or player.speed == 0
		return
	end
	
	// uncaps vertical speed during NiGHTS mode
	local mo = player.mo
	local flyAngle = FixedAngle(player.flyangle*FRACUNIT)
	local speed = FixedDiv(player.speed*FRACUNIT, 50*FRACUNIT)
	local yspeed = FixedMul(sin(flyAngle), speed)/11
	
	if abs(yspeed) > abs(mo.momz)
		mo.momz = yspeed
	end
end

function Lancer.UpdatePlayer(player)
	if not (maptol & TOL_NIGHTS)
		return
	end
	
	local delta = player.deltaChars
	local mo = player.mo
	local carry = player.powers[pw_carry]
	
	// stage failed
	if carry == CR_NIGHTSFALL or player.exiting
		// explobe
		if (carry ~= CR_NIGHTSMODE and delta.prevCarry == CR_NIGHTSMODE)
		or (carry == CR_NIGHTSMODE and player.exiting == (14*TICRATE)/5 + 1)
			local lancer = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_DC_LANCER)
			
			local hthrust = 4*lancer.scale
			local vthrust = 2*FRACUNIT
			local angle = ANG30
			if carry == CR_NIGHTSMODE
				hthrust = $*4
				vthrust = $*8
			else
				angle = $ * (2*P_RandomKey(2) - 1)
			end
			P_InstaThrust(lancer, mo.angle + angle, hthrust)
			P_SetObjectMomZ(lancer, vthrust)
			
			lancer.target = mo
			lancer.shadowscale = FRACUNIT
			lancer.momx = $ + mo.momx
			lancer.momy = $ + mo.momy
			lancer.momz = $ + mo.momz
			DC.SpawnExplosion(mo, MT_THOK)
		end
		
		// when exiting, force the character's fall sprite, not Lancer
		// this happens every frame because the game continuously switches you back to S_PLAY_NIGHTS_FLOAT
		if mo.state == S_PLAY_NIGHTS_FLOAT
			mo.state = S_PLAY_FALL
			mo.frame = ($ &~ FF_FRAMEMASK) | ((leveltime/2) & 1)
		end
		return
	end
	
	// delay the transformation and spawn the explosion
	if mo.state == S_PLAY_NIGHTS_TRANS6
		if delta.prevState == S_PLAY_NIGHTS_TRANS5
			mo.tics = TICRATE
		end
		
		// when done transforming, throw a dummy sprite of the character backwards
		if mo.tics == 6
			DC.SpawnExplosion(mo, MT_TAILSOVERLAY)
			
			local dummy = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_GFZFLOWER1)
			dummy.flags = $ | MF_NOCLIPHEIGHT
			dummy.fuse = TICRATE
			dummy.skin = mo.skin
			dummy.state = S_PLAY_PAIN
			dummy.sprite2 = SPR2_BURN
			dummy.angle = Lancer.GetCameraAngle(player) - ANGLE_90
			P_InstaThrust(dummy, dummy.angle, -12*dummy.scale)
			P_SetObjectMomZ(dummy, 6*FRACUNIT)
			
			if DC.DialogueBox.current and DC.Singleplayer()
				mo.frame = $ &~ FF_FRAMEMASK
				mo.sprite2 = SPR2_NFLT
			else
				mo.state = S_PLAY_NIGHTS_FLY
				player.speed = FRACUNIT >> 1 - 1
				DC.SetHitlag(player, 6)
			end
		end
		
		// BLAST LANCER AWAY
		if mo.tics == 1
			if DC.DialogueBox.current and DC.Singleplayer()
				mo.tics = 2
				
				if S_SoundPlaying(mo, sfx_lancel)
					mo.sprite2 = SPR2_NSTD
					mo.frame = ($ &~ FF_FRAMEMASK) + ((leveltime / 4) & 1)
				elseif mo.sprite2 == SPR2_NSTD
					mo.frame = $ &~ FF_FRAMEMASK
					mo.sprite2 = SPR2_NFLT
				end
			else
				DC.SpawnExplosion(mo, MT_TAILSOVERLAY)
				mo.state = S_PLAY_NIGHTS_FLY
				player.speed = FRACUNIT >> 1 - 1
				DC.SetHitlag(player, 6)
			end
		end
		
		return
	end
	
	if carry == CR_NIGHTSMODE
		player.drilldelay = max($, 2)
		
		if DC.ButtonDown(player, BT_JUMP)
		and Lancer.CanBoost(player)
			DC.SpawnExplosion(mo, MT_THOK)
			player.drillmeter = max(0, $ - 5*20)
			player.speed = min($ + Lancer.BOOST_SPEED, Lancer.TOP_SPEED)
			DC.StartQuake(player, 12*mo.scale, 5)
		end
	end
end

// -----
// hooks
// -----

DC.AddHook("MobjThinker", Lancer.MobjThinker, MT_DC_LANCER)
DC.AddHook("PlayerThink", Lancer.PlayerThink)