/*
DeltaChars objects

v1
*/

local DC = DeltaChars

// spare sparkles

DC.AddHook("MobjSpawn", function(mo)
	mo.color = SKINCOLOR_DC_SPARE
	mo.colorized = true
	mo.blendmode = AST_ADD
end, MT_DC_SPARESPARKLE)

DC.AddHook("MobjThinker", function(mo)
	if mo.state ~= mo.info.spawnstate
		mo.rollangle = $ + ANG10
		if mo.tics < 10
			mo.frame = $ + FF_TRANS10
		end
	end
end, MT_DC_SPARESPARKLE)

// Susie's projectile

function DC.Susie.ProjectileCollide(beam, item)
	if not item.valid
	or not beam.valid
		return
	end
	
	if (item.type == MT_SPIKE or item.type == MT_WALLSPIKE or item.type == MT_TARGET)
	and item.z <= beam.z + beam.height
	and beam.z <= item.z + item.height
		if item.type == MT_TARGET
			P_DamageMobj(item, beam, beam.target or beam)
		else
			DC.KillSpike(item, beam)
		end
		return false
	end
end

DC.AddHook("MobjCollide", DC.Susie.ProjectileCollide, MT_SUSIE_PROJECTILE)
DC.AddHook("MobjMoveCollide", DC.Susie.ProjectileCollide, MT_SUSIE_PROJECTILE)

DC.AddHook("MobjLineCollide", function(beam, line)
	if not beam.valid
		return
	end
	
	DC.MobjBustFOFsThroughLine(beam, line)
	
	if beam.flags & MF_NOCLIPHEIGHT
		return false
	end
end, MT_SUSIE_PROJECTILE)

DC.AddHook("MobjThinker", function(beam)
	if not beam.valid
	or beam.flags2 & MF2_DONTRESPAWN
		return
	end
	
	if beam.state == beam.info.deathstate
		local angle = ANGLE_45
		for i = 1, 4
			for speed = 8*FRACUNIT, 12*FRACUNIT, 4*FRACUNIT
				local part = P_SpawnMobjFromMobj(beam, 0, 0, 0, MT_SUSIE_PROJECTILE_DEATHPART)
				part.rollangle = angle
				part.color = beam.color
				part.colorized = beam.colorized
				part.destscale = 0
				part.scalespeed = part.scale/20
				part.extravalue1 = speed
			end
			angle = $ + ANGLE_90
		end
		beam.flags2 = $ | MF2_DONTRESPAWN
		return
	end
	
	local ghost = P_SpawnMobjFromMobj(beam, 0, 0, 0, MT_SUSIE_PROJECTILE_TRAIL)
	ghost.frame = beam.frame
	ghost.angle = beam.angle
	ghost.color = beam.color
	ghost.colorized = beam.colorized
	ghost.blendmode = beam.blendmode
	
	// shrink when nearing the end of lifespan
	if beam.fuse == 8
		beam.destscale = 0
		beam.scalespeed = beam.scale / beam.fuse
	end
end, MT_SUSIE_PROJECTILE)

DC.AddHook("MobjSpawn", function(ghost)
	ghost.fuse = ghost.info.reactiontime
end, MT_SUSIE_PROJECTILE_TRAIL)

DC.AddHook("MobjThinker", function(ghost)
	local tics = ghost.info.reactiontime
	ghost.frame = ($ & ~FF_TRANSMASK) | ((10 * (tics - ghost.fuse) / tics) << FF_TRANSSHIFT)
	ghost.spriteyscale = $ - (FRACUNIT / tics)
	ghost.spriteyoffset = FixedDiv(ghost.info.height - FixedMul(ghost.info.height, ghost.spriteyscale), 2*ghost.spriteyscale)
end, MT_SUSIE_PROJECTILE_TRAIL)

DC.AddHook("MobjThinker", function(part)
	if leveltime & 1
		part.frame = $ + FF_TRANS10
	end
	
	if part.frame & FF_TRANSMASK > FF_TRANS90
		P_RemoveMobj(part)
		return
	end
	
	part.extravalue2 = $ + FixedMul(part.extravalue1, part.scale)
	
	part.spritexoffset = -FixedDiv(FixedMul(cos(part.rollangle), part.extravalue2), part.scale)
	part.spriteyoffset = FixedDiv(FixedMul(sin(part.rollangle), part.extravalue2), part.scale)
end, MT_SUSIE_PROJECTILE_DEATHPART)

// Heart

DC.AddHook("MobjThinker", function(mobj)
	--Get shadow
	if not mobj.threshold
		mobj.shadowscale = FRACUNIT*3/5
		mobj.threshold = 1
		return
	end
	
	--Delete if no player
	local mo = mobj.target
	local player = mo and mo.player
	if not player or player.skin ~= DC.Kris.GetSkinIndex()
		P_SpawnGhostMobj(mobj)
		P_RemoveMobj(mobj)
		return
	end
	
	--Home towards player if dance is finished
	if mobj.fuse or mo.state ~= S_TAILSOVERLAY_EDGE
	or (mo.frame & FF_FRAMEMASK) == R_Char2Frame("e")
		if not mobj.fuse
			mobj.flags = DC.MF_INTANGIBLE | MF_NOGRAVITY
			mobj.fuse = 6
			mobj.angle = R_PointToAngle2(mobj.x, mobj.y, mo.x, mo.y)
		end
		
		local factor = mobj.fuse
		P_TeleportMove(mobj, mobj.x - (mobj.x - mo.x)/factor,
		mobj.y - (mobj.y - mo.y)/factor,
		mobj.z - (mobj.z - mo.z - mo.height/2)/factor)
		
		if mobj.fuse == 1
			DC.StartQuake(player, 24*mo.scale, 3)
			DC.SpawnHeartEffect(mo, 7)
			DC.SetHitlag(player, TICRATE/4)
			
			local oldflash = player.powers[pw_flashing]
			P_DoPlayerPain(player, mobj, mobj)
			
			P_InstaThrust(mo, mobj.angle, FixedHypot(mo.momx, mo.momy))
			mo.momz = $/2
			mo.state = oldflash and S_PLAY_PAIN or S_PLAY_STUN
			mo.sprite2 = SPR2_DEAD
			player.powers[pw_flashing] = oldflash
			
			S_StartSound(mo, sfx_susie6)
			S_StartSound(mo, sfx_dchurt)
		end
		
		return
	end
	
	--Check for input
	if abs(player.cmd.forwardmove) > 35 or abs(player.cmd.sidemove) > 35
		mobj.reactiontime = $ + 1
	elseif mobj.reactiontime
		mobj.reactiontime = 0
	end
	
	--Move heart
	local angle = DC.GetInputAngle(player)
	if P_IsObjectOnGround(mobj)
		mobj.friction = FRACUNIT/2
		if mobj.reactiontime == 1
			P_InstaThrust(mobj, angle, mobj.scale/2)
		end
	end
end, MT_DC_HEART)