/*
susie was here
	by susie!!!!

v1
*/

local DC = DeltaChars
local Ralsei = {}

DC.Ralsei = Ralsei

// ----
// defs
// ----

Ralsei.SKIN = "ralsei"

Ralsei.SING_CHANCE = FRACUNIT/4 // FRACUNIT = 100%
Ralsei.TUNES = {sfx_ralss1, sfx_ralss2}
Ralsei.BEATS = 7
Ralsei.BEAT_TICS = 12
Ralsei.FERMATA = 3*TICRATE/2
Ralsei.SWING_SOUND_TIMING = TICRATE
Ralsei.SWING_LOSS_FACTOR = 2 //Originally 8
Ralsei.PACIFY_MIN = TICRATE/2
Ralsei.PACIFY_MAX = Ralsei.PACIFY_MIN*3/2
Ralsei.PACIFY_DELAY = Ralsei.PACIFY_MAX
Ralsei.PACIFY_COLOR1 = SKINCOLOR_AQUA
Ralsei.PACIFY_COLOR2 = SKINCOLOR_KETCHUP
Ralsei.SLEEP_TIME = TICRATE*10

//States to cancel out of when switching out of this character
Ralsei.CancelStates = {}
local statelist = {
	S_RALSEI_BLUSH, S_RALSEI_SWIPE, S_RALSEI_CAST, S_PLAY_SPINDASH
}
for i, state in ipairs(statelist)
	Ralsei.CancelStates[state] = true
end

//Sleeping frames for each enemy that has one
//Note that the order matters here!
Ralsei.SLEEP_FRAMES = {}
local framelist = {
	MT_BLUECRAWLA, MT_REDCRAWLA, MT_GFZFISH, MT_SNAILER, MT_SKIM,
	MT_CRAWLACOMMANDER, MT_VULTURE, MT_GSNAPPER, MT_JETTBOMBER, MT_JETTGUNNER,
	MT_JETJAW, MT_MINUS, MT_POINTY, MT_UNIDUS, MT_SPRINGSHELL, MT_YELLOWSHELL,
	MT_PENGUINATOR, MT_POPHAT, MT_CRUSHSTACEAN, MT_BANPYURA, MT_SPINCUSHION,
	MT_HIVEELEMENTAL, MT_BUMBLEBORE, MT_SPINBOBERT, MT_HANGSTER, MT_CANARIVORE,
	MT_PYREFLY, MT_PTERABYTE, MT_BUGGLE, MT_DRAGONBOMBER, MT_EGGGUARD,
	MT_ROBOHOOD, MT_GOOMBA, MT_BLUEGOOMBA
}
for i, enemy in ipairs(framelist)
	Ralsei.SLEEP_FRAMES[enemy] = i
end

function Ralsei.GetSkinIndex()
	if not Ralsei.skinIndex
		Ralsei.skinIndex = #skins[Ralsei.SKIN]
	end
	return Ralsei.skinIndex
end

function Ralsei.CreateTable(player)
	player.deltaChars.ralsei = {
		krisTics = 0,
		singTics = 0,
		tune = 1,
		swipeangle = 0,
		swipesound = 0,
		swipefire = false,
		swipespeed = 0,
		pacify = 0,
	}
end

function Ralsei.RemoveTable(player)
	if player.deltaChars.ralsei == nil
		return
	end
	Ralsei.HandlePacify(player, true)
	player.deltaChars.ralsei = nil
	
	return true
end

function Ralsei.UpdatePlayer(player)
	if player.skin ~= Ralsei.GetSkinIndex()
		return
	end
	
	Ralsei.HandlePacify(player)
	if player.playerstate ~= PST_LIVE
		return
	end
	
	Ralsei.HandleBlush(player)
	Ralsei.HandleSing(player)
	Ralsei.HandleSwipe(player)
end

// if a Ralsei player is touching a Kris player, their idle state is replaced with a blush
function Ralsei.HandleBlush(player)
	local ralsei = player.deltaChars.ralsei
	local mo = player.mo
	
	if player.panim ~= PA_IDLE
		ralsei.krisTics = 0
		return
	end
	
	local skin = DC.Kris.GetSkinIndex()
	local krisNearby = false
	
	for player2 in players.iterate
		local mo2 = player2.mo
		
		if player2.skin ~= skin
		or not mo2
			continue
		end
		
		if FixedHypot(mo.x - mo2.x, mo.y - mo2.y) < mo.radius + mo2.radius
		and mo.z <= mo2.z + mo2.height
		and mo2.z <= mo.z + mo.height
			krisNearby = true
			break
		end
	end
	
	// no blushing if there's no kris :(
	if not krisNearby
		if mo.state == S_RALSEI_BLUSH
			mo.state = S_PLAY_STND
		end
		ralsei.krisTics = 0
		return
	end
	
	// already blushing
	if mo.state == S_RALSEI_BLUSH
		return
	end
	
	ralsei.krisTics = $ + 1
	
	if mo.state == S_PLAY_STND // case 1: we're about to enter idle anims, but Kris is nearby
		and mo.tics == 1
	or ralsei.krisTics >= states[S_PLAY_STND].tics // case 2: we idled for long enough near Kris
		mo.state = S_RALSEI_BLUSH
	end
end

// random chance that Ralsei's idle animation audibly sings
function Ralsei.HandleSing(player)
	local mo = player.mo
	local delta = player.deltaChars
	local ralsei = delta.ralsei
	
	// check whether we just entered our idle state
	// if so, random chance the singing activates!
	if mo.state == S_PLAY_WAIT
	and delta.prevState ~= mo.state
	and not DC.DialogueBox.current
	and P_RandomChance(Ralsei.SING_CHANCE)
		S_StartSound(mo, Ralsei.TUNES[ralsei.tune])
		ralsei.tune = ($ % #Ralsei.TUNES) + 1
		ralsei.singTics = (Ralsei.BEATS - 1) * Ralsei.BEAT_TICS
	end
	
	// while singing, play the idle animation faster
	if ralsei.singTics > 0
		ralsei.singTics = $ - 1
		if ralsei.singTics % Ralsei.BEAT_TICS == 0
			mo.state = $
			if ralsei.singTics == 0
				mo.anim_duration = Ralsei.FERMATA
			end
		end
	end
end

//Start Scarf Swipe
function Ralsei.AbilitySpecial(player)
	if player.skin ~= Ralsei.GetSkinIndex()
	or not DC.CheckStatus(player)
	or player.charability ~= skins[player.skin].ability
	or (player.pflags & PF_THOKKED
	and not (player.charflags & SF_MULTIABILITY))
		return
	end
	
	local mo = player.mo
	local delta = player.deltaChars
	local ralsei = delta.ralsei
	
	if mo.state == S_RALSEI_SWIPE then return true end
	
	//Speed - Less analog push means less power
	local actionspd = FixedMul(player.actionspd, mo.scale)
	if mo.eflags & MFE_UNDERWATER then actionspd = $/2 end
	ralsei.swipespeed = actionspd
	
	local thrust = min(FixedHypot(player.cmd.forwardmove, player.cmd.sidemove), 50)
	actionspd = $*thrust/50
	
	//Vertical momentum
	//The fire version triggers if we use this during pacify
	ralsei.swipefire = 0
	if ralsei.pacify
		//Super jump!
		player.pflags = $ &~ PF_JUMPED
		P_DoJump(player, false)
		player.pflags = $ &~ PF_STARTJUMP
		
		if ralsei.pacify >= Ralsei.PACIFY_MAX
			mo.momz = $*5/2
			ralsei.swipefire = 2
		else
			mo.momz = $*7/4
			ralsei.swipefire = 1
		end
		
		//Burst effect
		local ghost = P_SpawnGhostMobj(mo)
		ghost.fuse = $*2
		ghost.sprite = SPR_FLAM
		ghost.frame = M
		ghost.blendmode = AST_ADD
		
		ghost.scalespeed = FixedMul($*2, ghost.scale)
		ghost.scale = $/4
		ghost.destscale = $*16
		ghost.z = $ + mo.height/2*P_MobjFlip(mo)
		
		if ralsei.pacify >= Ralsei.PACIFY_MAX
			ghost.color = Ralsei.PACIFY_COLOR2
		else
			ghost.color = Ralsei.PACIFY_COLOR1
		end
		ghost.colorized = true
	else
		//Undo PF_STARTJUMP speed cut
		if mo.state == S_PLAY_JUMP and player.pflags & PF_JUMPED
		and not (player.pflags & PF_STARTJUMP)
			mo.momz = $*2
		end
		player.pflags = $ | P_GetJumpFlags(player)
		
		//Momentum boost
		local grav = min(abs(P_GetMobjGravity(mo)), FixedMul(gravity, mo.scale))*P_MobjFlip(mo)
		if mo.momz*P_MobjFlip(mo) < 0
			mo.momz = 16*grav
		else
			mo.momz = $ + 16*grav
		end
	end
	
	//Horiztonal thrust
	local speed = DC.GetPlayerSpeed(player)
	
	//Funny speed loss!
	if ralsei.swipefire
		mo.momx = $/2
		mo.momy = $/2
		
	//Only thrust by half the actionspd, actionspd is our cap
	elseif speed < actionspd
		P_Thrust(mo, DC.GetInputAngle(player), actionspd/2)
		
		if DC.GetPlayerSpeed(player) > actionspd
			P_InstaThrust(mo, DC.GetPlayerMomentumAngle(player), actionspd)
		end
		
	//If we're above our actionspd, let's at least let us redirect momentum a little
	elseif speed > actionspd
		P_InstaThrust(mo, DC.GetPlayerMomentumAngle(player), speed - actionspd)
		P_Thrust(mo, DC.GetInputAngle(player), actionspd)
		ralsei.swipespeed = speed
	end
	
	//State and angle
	mo.state = S_RALSEI_SWIPE
	ralsei.swipeangle = player.drawangle
	player.pflags = $ | PF_THOKKED | PF_JUMPED | PF_NOJUMPDAMAGE &~ PF_SPINNING
	
	//Sound and effect
	ralsei.swipesound = 0
	if ralsei.swipefire
		S_StartSound(mo, sfx_dcslic)
		S_StartSound(mo, sfx_s3k48)
		DC.SetHitlag(player, 4)
	else
		S_StartSound(mo, sfx_swing3)
		DC.SetHitlag(player, 1)
	end

	return true
end

//During Scarf Swipe
function Ralsei.HandleSwipe(player)
	local mo = player.mo
	local delta = player.deltaChars
	local ralsei = delta.ralsei
	
	//Not in swipe
	if mo.state != S_RALSEI_SWIPE
		//Activate swipe even without jumping first!
		if DC.ButtonDown(player, BT_JUMP) and not player.gotflag
		and not (player.pflags & (DC.PFLAGS_BLACKLIST|PF_JUMPED|PF_THOKKED))
		and (ralsei.pacify or not P_IsObjectOnGround(mo))
		and not player.powers[pw_carry]
		and not P_PlayerInPain(player)
			if not Ralsei.AbilitySpecial(player)
				return
			end
		else
			return
		end
	end
	
	//Speed
	local actionspd = FixedMul(player.actionspd, mo.scale)
	if mo.eflags & MFE_UNDERWATER then actionspd = $/2 end
	actionspd = max($, ralsei.swipespeed)
	
	//Speed loss factor
	local factor = Ralsei.SWING_LOSS_FACTOR
	local factor2 = factor - 1
	
	//Slow down
	actionspd = $/factor + $*mo.tics/states[mo.state].tics*factor2/factor
	actionspd = min($, DC.GetPlayerSpeed(player))  //No speeding up!
	P_InstaThrust(mo, DC.GetPlayerMomentumAngle(player), actionspd)
	
	//Angle
	ralsei.swipeangle = $ - mo.tics*ANG2
	player.drawangle = ralsei.swipeangle
	
	//Sound
	if not ralsei.swipesound
		S_StartSound(mo, sfx_dcbrst)
		ralsei.swipesound = Ralsei.SWING_SOUND_TIMING
	else
		ralsei.swipesound = max($ - mo.tics, 0)
	end
	
	//Circling particles
	if mo.tics > states[mo.state].tics/4 and not delta.hitlag
		local dust
		if ralsei.swipefire
			dust = P_SpawnMobjFromMobj(mo, 0,0,0, MT_THOK)
			dust.fuse = 16
			dust.spriteyscale = FRACUNIT/2
			dust.state = S_RALSEI_FIRE
			
			if ralsei.swipefire > 1
				dust.color = Ralsei.PACIFY_COLOR2
				dust.colorized = true
			end
		else
			dust = DC.SpawnDust(player)
			dust.scale = $/2
		end
		P_TeleportMove(dust,
		mo.x + P_ReturnThrustX(mo, player.drawangle, mo.radius*3),
		mo.y + P_ReturnThrustY(mo, player.drawangle, mo.radius*3),
		mo.z + mo.height/2)
		
		dust.momx = mo.momx/2
		dust.momy = mo.momy/2
		dust.momz = mo.momz/2
		
		//Fire
		if ralsei.swipefire
			P_SpawnMobjFromMobj(mo,
			FixedMul(mo.info.radius << 1, P_RandomFixed()) - mo.info.radius,
			FixedMul(mo.info.radius << 1, P_RandomFixed()) - mo.info.radius,
			FixedMul(mo.info.height, P_RandomFixed()),
			MT_THOK).state = S_DC_SMOKE
		end
	end
	
	//Go forward to simulate the hitbox
	local dist = mo.radius*3
	local x = mo.x
	local y = mo.y
	local z = mo.z
	local momx = mo.momx
	local momy = mo.momy
	local momz = mo.momz
	local flags = mo.flags
	local godmode = player.pflags & PF_GODMODE
	local flashing = player.powers[pw_flashing]
	mo.flags = $ | MF_NOCLIPTHING | MF_NOCLIP | MF_NOCLIPHEIGHT
	player.pflags = $ | PF_GODMODE
	player.powers[pw_flashing] = TICRATE
	if srb2p then player.dashmode = 10*TICRATE end
	P_TryMove(mo, mo.x + P_ReturnThrustX(nil, player.drawangle, dist),
	mo.y + P_ReturnThrustY(nil, player.drawangle, dist), true)
	
	mo.flags = $ &~ MF_NOCLIPTHING &~ MF_NOCLIP
	P_CheckPosition(mo, mo.x, mo.y, mo.z)
	local grounded = P_IsObjectOnGround(mo)
	
	//Go back
	mo.flags = $ | MF_NOCLIPTHING | MF_NOCLIP
	if srb2p then player.dashmode = 0 end
	P_TryMove(mo, x, y, true)
	mo.z = z
	if grounded and not P_IsObjectOnGround(mo)
	and not (mo.eflags & MFE_SPRUNG)
		mo.momx = momx
		mo.momy = momy
		mo.momz = momz
	end
	mo.flags = flags
	player.pflags = ($ &~ PF_GODMODE) | godmode
	player.powers[pw_flashing] = flashing
end

function Ralsei.CircleSpawn(mo, color)
	//Circle effect
	local thok = P_SpawnMobjFromMobj(mo, 0,0,0, MT_DC_SPARE)
	thok.spritexscale = FRACUNIT/3
	thok.spriteyscale = thok.spritexscale
	
	thok.sprite = SPR_OILF
	thok.flags2 = $ | MF2_SPLAT
	thok.blendmode = AST_ADD
	
	thok.color = color or Ralsei.PACIFY_COLOR1
	thok.colorized = true
	DC.SpawnSpareSparkles(mo, 16, 2*FRACUNIT/3, -32*FRACUNIT, thok.color)
	
	thok.height = 1
	thok.target = mo
	thok.threshold = 1
end

//Start Pacify
function Ralsei.SpinSpecial(player)
	if player.skin ~= Ralsei.GetSkinIndex()
	or player.pflags & (DC.PFLAGS_BLACKLIST | PF_SPINDOWN)
	or P_PlayerInPain(player)
	or player.exiting
	or player.playerstate ~= PST_LIVE
	or player.mo.state == S_RALSEI_SWIPE
	or player.weapondelay
	or not player.deltaChars
	or not player.deltaChars.ralsei
	or player.deltaChars.flags & DC.FLAGS.SWITCHING
	or player.deltaChars.ralsei.pacify
		return
	end
	
	local mo = player.mo
	local delta = player.deltaChars
	local ralsei = delta.ralsei
	
	//Start the move
	ralsei.pacify = 1
	S_StartSound(mo, sfx_ralss3)
	player.pflags = $ | PF_JUMPSTASIS
	
	//Set the state
	mo.state = S_PLAY_SPINDASH
	player.panim = PA_DASH
	mo.tics = -1
	
	//Slow down!
	mo.momx = $/2
	mo.momy = $/2
	mo.momz = 0
	
	//Charge effect
	if ralsei.charge and ralsei.charge.valid then P_KillMobj(ralsei.charge) end
	local charge = P_SpawnMobjFromMobj(mo, 0,0,0, MT_OVERLAY)
	charge.target = mo
	charge.color = Ralsei.PACIFY_COLOR1
	charge.colorized = true
	charge.state = S_JETFUMEFLASH
	charge.spritexscale = FRACUNIT*2
	charge.spriteyscale = charge.spritexscale
	ralsei.charge = charge
	
	//Circle effect
	Ralsei.CircleSpawn(mo)
end

//During Pacify
function Ralsei.HandlePacify(player, delete)
	local mo = player.mo
	local delta = player.deltaChars
	local ralsei = delta and delta.ralsei
	local charge = ralsei and ralsei.charge
	
	if not ralsei then return end
	if not ralsei.pacify
		if ralsei.charge and ralsei.charge.valid
			P_KillMobj(ralsei.charge)
			ralsei.charge = nil
		end
	end
	if not mo or mo.state != S_PLAY_SPINDASH
	or player.pflags & DC.PFLAGS_BLACKLIST
	or delta.flags & DC.FLAGS.SWITCHING or delete
		ralsei.pacify = 0
		if mo.state == S_PLAY_SPINDASH
			mo.state = S_DC_TERMINATE
		end
		if ralsei.charge and ralsei.charge.valid
			P_KillMobj(ralsei.charge)
			ralsei.charge = nil
		end
		return
	end
	
	//Tick up...
	ralsei.pacify = $ + 1
	player.weapondelay = max($, Ralsei.PACIFY_DELAY)
	mo.movefactor = 0
	
	//No jumping during the beginning
	if ralsei.pacify < Ralsei.PACIFY_MIN/4
		player.pflags = $ | PF_JUMPSTASIS
	end
	
	//No jumping on the ground
	if P_IsObjectOnGround(mo)
		player.pflags = $ | PF_JUMPDOWN
		
	//Low gravity
	else
		mo.momz = $ - P_GetMobjGravity(mo)*4/5
	end
	
	//Release!
	if not (player.pflags & PF_SPINDOWN) and ralsei.pacify >= Ralsei.PACIFY_MIN
		local proj
		local charged = ralsei.pacify >= Ralsei.PACIFY_MAX
		local target = charged and P_LookForEnemies(player, false, true)
		if target
			proj = P_SpawnMissile(mo, target, MT_RALSEI_PROJECTILE)
		else
			proj = P_SPMAngle(mo, MT_RALSEI_PROJECTILE, mo.angle, 0)
		end
		
		if proj
			if charged
				proj.destscale = $*2
				proj.color = Ralsei.PACIFY_COLOR2
				proj.fuse = proj.info.reactiontime*3/2
			else
				proj.color = Ralsei.PACIFY_COLOR1
				proj.fuse = proj.info.reactiontime
			end
			proj.colorized = true
			
			if target
				proj.momz = ((target.z + target.height/2) - (mo.z + mo.height/2))
				/ max(P_AproxDistance(mo.x - target.x, mo.y - target.y) / FixedMul(proj.info.speed/2, proj.scale), 1)
			end
		end
		
		mo.state = S_RALSEI_CAST
		S_StartSound(proj or mo, sfx_ralss5)
		
		if charge and charge.valid
			charge.state = S_JETFUMEFLASH
			charge.fuse = 3
		end
		
		mo.momx = $/2
		mo.momy = $/2
		mo.momz = 0
		
		ralsei.pacify = 0
	end
	
	//Charge effect
	if charge and charge.valid
		charge.frame = ($ &~ FF_TRANSMASK) | FF_TRANS70
		if ralsei.pacify == Ralsei.PACIFY_MAX
			S_StartSound(mo, sfx_ralss4)
			charge.color = Ralsei.PACIFY_COLOR2
			charge.state = S_JETFUMEFLASH
			
			//Circle effect
			Ralsei.CircleSpawn(mo, ralsei.charge.color)
		end
	end
end

//Projectile Thinker
function Ralsei.ProjectileThink(mobj)
	if not mobj.health or not (mobj.flags & MF_MISSILE)
		if not mobj.cusval
			P_RadiusAttack(mobj, mobj.target or mobj, mobj.info.radius*8, 0, false)
			S_StopSoundByID(mobj, sfx_ralss5)
			mobj.cusval = $ + 1
		end
		return
	end
	
	//Floaty
	mobj.threshold = $ + 1
	local dist = FixedMul(cos(mobj.threshold*ANG20), mobj.destscale)
	local angle = mobj.angle + ANGLE_90
	P_TryMove(mobj,
	mobj.x + P_ReturnThrustX(mobj, angle, dist),
	mobj.y + P_ReturnThrustY(mobj, angle, dist),
	true)
	mobj.z = $ + FixedMul(sin(mobj.threshold*ANG20), mobj.destscale*2)
	
	//Transparent
	mobj.frame = ($ &~ FF_TRANSMASK) | FF_TRANS20
	
	//Slow
	if FixedHypot(mobj.momx, mobj.momy) > FixedMul(mobj.info.speed, mobj.destscale/4)
		mobj.momx = $*7/8
		mobj.momy = $*7/8
	end
	
	//Effect
	if not (mobj.threshold % 4)
		local thok = P_SpawnMobjFromMobj(mobj, 0,0,0, MT_THOK)
		thok.state = S_RALSEI_MIST
		thok.blendmode = AST_ADD
		
		thok.color = mobj.color
		thok.colorized = mobj.colorized
	end
end

//Sleep stuff
//This should probably be in SOC.lua...but come on, it's easier to find here!
DC.SafeFreeslot("MT_DC_SLEEPINGENEMY", "MT_DC_SPARE",
"S_DC_SLEEPINGENEMY", "S_DC_SLEEPZ", "SPR_DCSL")

mobjinfo[MT_DC_SLEEPINGENEMY] = {
	spawnstate = S_INVISIBLE,
	spawnhealth = 1,
	seestate = S_INVISIBLE,
	painstate = S_INVISIBLE,
	meleestate = S_INVISIBLE,
	missilestate = S_INVISIBLE,
	raisestate = S_INVISIBLE,
	deathstate = S_XPLD1,
	xdeathstate = S_XPLD1,
	speed = 3,
	radius = 24*FRACUNIT,
	height = 32*FRACUNIT,
	flags = MF_ENEMY|MF_SPECIAL|MF_SHOOTABLE,
}

//Set the sleeping enemy type and sprites
function DC.actions.SleepingEnemy(mobj, var1, var2)
	if not mobj.ralseitype
		//Reset the Pyrefly for when we wake up
		if mobj.type == MT_PYREFLY
			mobj.flags2 = $ &~ MF2_BOSSNOTRAP
			mobj.extravalue2 = 0
			
		//Get rid of the crab claws
		elseif (mobj.type == MT_CRUSHSTACEAN or mobj.type == MT_BANPYURA)
		and mobj.tracer
			if mobj.type == MT_CRUSHSTACEAN
				local chain = mobj.tracer.target
				local chainnext
				while (chain)
					chainnext = chain.target
					P_RemoveMobj(chain)
					chain = chainnext
				end
			end
			S_StopSound(mobj.tracer)
			P_KillMobj(mobj.tracer, mo, src, dmgtype)
			mobj.tracer = nil
		end
		
		//Set the type
		mobj.ralseitype = mobj.type
		mobj.type = MT_DC_SLEEPINGENEMY
	end
	
	//Sprites
	local frame = DC.Ralsei.SLEEP_FRAMES[mobj.ralseitype]
	if mobj.ralseitype == MT_GSNAPPER
		mobj.sprite = states[mobjinfo[mobj.ralseitype].spawnstate].sprite
		if frame and mobj.tracer
			mobj.tracer.sprite = SPR_DCSL
			mobj.tracer.frame = frame
		end
	elseif frame
		mobj.sprite = SPR_DCSL
		mobj.frame = frame
	else
		mobj.sprite = states[mobjinfo[mobj.ralseitype].spawnstate].sprite
	end
end

//Sleeping state
states[S_DC_SLEEPINGENEMY] = {
	sprite = SPR_DCSL,
	frame = A,
	tics = 1,
	nextstate = S_DC_SLEEPINGENEMY,
	action = DC.actions.SleepingEnemy,
}

//Wake up from sleep
function Ralsei.WakeUp(mobj, state)
	//Reset our type
	if mobj.ralseitype
		mobj.type = mobj.ralseitype
		mobj.ralseitype = nil
	end
	
	//Reset snoring squish
	mobj.spritexscale = FRACUNIT
	mobj.spriteyscale = FRACUNIT
	if mobj.type == MT_GSNAPPER and mobj.tracer
		mobj.tracer.spritexscale = FRACUNIT
		mobj.tracer.spriteyscale = FRACUNIT
		mobj.tracer.state = $
	end
	
	//Reset our state
	if state
		if mobj.type == MT_GSNAPPER
			mobj.state = S_GSNAPPER_STND
		else
			mobj.state = mobj.info.spawnstate
		end
		
		/*
		//Dumb stupid claw regaining (disabled because it's kinda funny)
		if (mobj.type == MT_CRUSHSTACEAN or mobj.type == MT_BANPYURA)
		and not mobj.tracer
			local bigmeatyclaw = P_SpawnMobjFromMobj(mobj, 0,0,0,
			mobj.type == MT_CRUSHSTACEAN and MT_CRUSHCLAW or MT_BANPSPRING)
			
			bigmeatyclaw.angle = mobj.angle + ((mobj.flags2 & MF2_AMBUSH) and ANGLE_90 or ANGLE_270)
			mobj.tracer = bigmeatyclaw
			bigmeatyclaw.tracer = mobj
			mobj.reactiontime = mobj.info.reactiontime >> 1
		end
		*/
	end
	
	//Reset timer
	mobj.ralseisleeping = 0
end

//Sleeping Enemy thinker
DC.AddHook("MobjThinker", function(mobj)
	//No type to return to?
	if not mobj.ralseitype then P_RemoveMobj(mobj) return end
	
	//Exit sleeping
	if not mobj.ralseisleeping or mobj.ralseisleeping >= Ralsei.SLEEP_TIME
	or mobj.state != S_DC_SLEEPINGENEMY
		Ralsei.WakeUp(mobj, true)
		return
	end
	
	//No acting while sleeping please!
	if mobj.fuse
		mobj.fuse = $ + 1
	end
	mobj.target = nil
	
	//Jokes on you, this is also used for sparing!
	if mobj.ralseisleeping < 0
		Ralsei.WakeUp(mobj)
		DC.SpawnSpare(mobj)
		mobj.ralseispared = true
		
		P_KillMobj(mobj, mobj.ralseiinf, mobj.ralseisrc)
		S_StopSound(mobj)
		P_RemoveMobj(mobj)
		return
	end
	
	//Sleeping effects
	if mobj.ralseisleeping < (Ralsei.SLEEP_TIME - TICRATE)
		//Snoring
		mobj.spritexscale = FRACUNIT - sin(mobj.ralseisleeping*ANG1*5)/12
		mobj.spriteyscale = FRACUNIT + sin(mobj.ralseisleeping*ANG1*5)/12
	
		//Spawn Zs
		if not ((mobj.ralseisleeping - 1) % TICRATE)
			local thok = P_SpawnMobjFromMobj(mobj, 0,0, mobj.info.height, MT_THOK)
			thok.state = S_DC_SLEEPZ
		end
		
	//Undo snoring when waking up
	else
		mobj.spritexscale = ($ - FRACUNIT)*15/16 + FRACUNIT
		mobj.spriteyscale = ($ - FRACUNIT)*15/16 + FRACUNIT
	end
	mobj.ralseisleeping = $ + 1
	
	//Match squish to snapper head
	if mobj.ralseitype == MT_GSNAPPER and mobj.tracer
		mobj.tracer.spritexscale = mobj.spritexscale
		mobj.tracer.spriteyscale = mobj.spriteyscale
	end
	
	//Fall even with no gravity
	if mobj.flags & MF_NOGRAVITY and not P_IsObjectOnGround(mobj)
		mobj.momz = $ - FixedMul(gravity*P_MobjFlip(mobj), mobj.scale)
	end
end, MT_DC_SLEEPINGENEMY)

//Spare effect
mobjinfo[MT_DC_SPARE] = {
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radius = mobjinfo[MT_PLAYER].radius,
	height = mobjinfo[MT_PLAYER].height,
	dispoffset = 1,
	flags = MF_SCENERY|MF_NOGRAVITY|MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT
}

//Spawn spawner
function DC.SpawnSpare(mobj, flashonly)
	//Normal fade
	if not flashonly
		local ghost = P_SpawnGhostMobj(mobj)
		ghost.fuse = $*4
		ghost.frame = $ &~ FF_TRANSMASK
		
		ghost.height = mobj.height
		ghost.radius = mobj.radius
		ghost.shadowscale = mobj.shadowscale
	end
	
	//White fade
	local spare = P_SpawnGhostMobj(mobj)
	spare.fuse = 0
	S_StartSound(spare, sfx_spare)
	
	spare.type = MT_DC_SPARE
	spare.frame = ($ &~ FF_TRANSMASK) | FF_TRANS90
	
	spare.color = SKINCOLOR_DC_SPARE
	spare.colorized = true
	spare.renderflags = $ | RF_FULLBRIGHT
	spare.blendmode = AST_ADD
	
	spare.height = mobj.height
	spare.radius = mobj.radius
end

//Spare thinker
DC.AddHook("MobjThinker", function(mobj)
	if not mobj.threshold
		if not (mobj.frame & FF_TRANSMASK)
			mobj.threshold = 1
		elseif leveltime & 1 
			mobj.frame = $ - FF_TRANS10
		end
		DC.SpawnSpareSparkles(mobj, 2, 2*FRACUNIT/3, 8*FRACUNIT)
		
	elseif leveltime & 1
		mobj.frame = $ + FF_TRANS10
		if (mobj.frame & FF_TRANSMASK) > FF_TRANS90
			P_RemoveMobj(mobj)
			return
		end
	end
	
	if mobj.target then A_CapeChase(mobj) end
end, MT_DC_SPARE)

//Sleep Zs
function DC.actions.SleepZ(mobj, var1, var2)
	if not mobj.fuse
		mobj.fuse = TICRATE*2
	end
	mobj.spritexscale = FRACUNIT + sin(mobj.fuse*ANG2*4)/2
	mobj.spriteyscale = FRACUNIT - sin(mobj.fuse*ANG2*4)/2
	
	mobj.spritexoffset = -32*FRACUNIT + -16*cos(mobj.fuse*ANG2*4)
	mobj.momz = mobj.scale
	
	//Fade out when nearing the end of fuse...
	if ((mobj.frame & FF_TRANSMASK) >> FF_TRANSSHIFT) < (NUMTRANSMAPS-1) - mobj.fuse / 2
		mobj.frame = ($ & ~FF_TRANSMASK) | (((NUMTRANSMAPS-1) - mobj.fuse / 2) << FF_TRANSSHIFT)
	end
end

states[S_DC_SLEEPZ] = {
	sprite = SPR_DCSL,
	frame = A,
	tics = 1,
	nextstate = S_DC_SLEEPZ,
	action = DC.actions.SleepZ,
}

//Spawn slap damage effect
function Ralsei.SpawnDamageEffect(player, item)
	local mo = player.mo
	local effect = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_THOK)
	effect.state = S_DC_SLAP
	effect.spritexscale = $*3/2
	effect.spriteyscale = effect.spritexscale
	P_TeleportMove(effect, mo.x/2 + item.x/2, mo.y/2 + item.y/2, effect.z)
	S_StartSound(mo, sfx_dcdmg)
end

//Do Ralsei's damage effect
//Also with the projectile put an enemy to sleep or spare them if they're already asleep
function Ralsei.MobjDamage(mobj, inf, src)
	//Damage effect stuff
	if inf and inf.player
	and inf.player.skin == Ralsei.GetSkinIndex()
	and DC.CheckStatus(inf.player)
	and inf.state == S_RALSEI_SWIPE
		Ralsei.SpawnDamageEffect(inf.player, mobj)
	end
	
	//You're asleep!
	if mobj.state == S_DC_SLEEPINGENEMY
		//Get spared
		if inf and mobj.health <= 1 and inf.type == MT_RALSEI_PROJECTILE
			mobj.ralseisleeping = -1
			mobj.ralseiinf = inf
			mobj.ralseisrc = src
			mobj.state = S_DC_SLEEPINGENEMY
			
			P_ExplodeMissile(inf)
			return true
			
		//Wake up, you're getting wacked!
		elseif mobj.ralseitype
			Ralsei.WakeUp(mobj)
		end
		
	//Fall asleep
	elseif mobj.flags & MF_ENEMY
	and inf and inf.type == MT_RALSEI_PROJECTILE
		local sleep = P_SpawnMobjFromMobj(mobj, 0,0, mobj.info.height/2, MT_THOK)
		sleep.state = S_RALSEI_SLEEP
		sleep.spritexscale = FRACUNIT*2
		sleep.spriteyscale = sleep.spritexscale
		
		mobj.ralseisleeping = 1
		mobj.state = S_DC_SLEEPINGENEMY
		S_StartSound(mobj, sfx_ralss6)
		
		P_ExplodeMissile(inf)
		return true
	end
end

//No hurting others while sleeping!
function Ralsei.ShouldDamage(mobj, inf)
	if mobj.state == S_DC_SLEEPINGENEMY and mobj.ralseisleeping < TICRATE/2 then return false end
	if inf and inf.state == S_DC_SLEEPINGENEMY then return false end
end

// -----
// hooks
// -----

DC.AddHook("AbilitySpecial", Ralsei.AbilitySpecial)
DC.AddHook("SpinSpecial", Ralsei.SpinSpecial)
DC.AddHook("MobjThinker", Ralsei.ProjectileThink, MT_RALSEI_PROJECTILE)

DC.AddHook("MobjDamage", Ralsei.MobjDamage)
DC.AddHook("ShouldDamage", Ralsei.ShouldDamage)

//Prevent the camera from going haywire in Simple
DC.AddHook("PlayerThink", function(player)
	if player.mo and player.mo.state == S_RALSEI_SWIPE
	and not (player.cmd.forwardmove or player.cmd.sidemove)
	and player.pflags & PF_DIRECTIONCHAR and player.pflags & PF_ANALOGMODE
		player.mo.angle = player.cmd.angleturn<<16
	end
end)

//DC.AddHook("MobjDamage", Ralsei.MobjDamage)

DC.AddMember(Ralsei)