/*
DeltaChars SOC

v1
*/

local DC = DeltaChars

DC.MF_INTANGIBLE = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOCLIPTHING

// To prevent duplicate freeslots
local function CheckSlot(item) // this function deliberately errors when a freeslot does not exist
	if _G[item] == nil // this will error by itself for states and objects
		error() // this forces an error for sprites, which do actually return nil for some reason
	end
end

function DC.SafeFreeslot(...)
	local item = ...
	if item == nil
		return
	end
	
	if pcall(CheckSlot, item)
		print("\131NOTICE:\128 " .. item .. " was not allocated, as it already exists.")
	else
		freeslot(item)
	end
	
	DC.SafeFreeslot(select(2, ...))
end

// ------
// colors
// ------

local function MakeSuperColor(name, map, ...)
	local prefix = "SKINCOLOR_SUPER" .. (name:upper())
	
	for i = 1, 5
		local const = prefix..i
		DC.SafeFreeslot(const)
		skincolors[_G[const]].name = "Super " .. name .. " " .. i
	end
	
	local n = 1
	
	for i = 1, 5
		local ramp = {}
		for j = 1, 16
			ramp[j] = select(n, ...)
			if map[j]
			or j == 16
				n = $ + 1
			end
		end
		skincolors[_G[prefix..i]].ramp = ramp
	end
	
	return ramp
end

DC.SafeFreeslot("SKINCOLOR_DC_SPARE")

MakeSuperColor("Knight",
	{false, false, true, false, true, true, true, false, true, true, false, true, true, true, true, true},
	255,132,134,135,172,173,174,175,253,254,31,
	129,255,132,135,134,172,173,174,175,253,254,
	255,255,255,134,134,134,173,171,172,173,174,
	129,129,255,255,255,134,134,134,137,138,138,
	129,129,129,129,129,255,255,255,255,134,135
)

MakeSuperColor("Rude",
	{false, false, true, false, false, false, true, false, true, true, false, true, true, true, true, true},
	179,182,184,185,186,198,199,28,31,
	178,181,182,184,185,196,187,199,28,
	177,178,181,182,182,183,185,186,187,
	177,177,181,181,182,182,183,184,186,
	176,179,179,180,180,180,181,182,183
)

MakeSuperColor("Fluffy",
	{true, true, false, true, false, false, true, true, false, false, false, false, false, true, true, true},
	208,209,211,214,215,45,46,47,
	208,85,224,243,246,247,248,249,
	82,89,90,243,77,69,70,71,
	81,89,89,101,115,118,119,110,
	0,88,89,113,113,115,118,119
)
	

skincolors[SKINCOLOR_DC_SPARE] = {
	name = "DC_Spare",
	ramp = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	accessible = false
}

// ------
// sounds
// ------

DC.SafeFreeslot("sfx_dcswch", "sfx_dchurt", "sfx_dcjump", "sfx_dcdyin", "sfx_dcdead", "sfx_dcfall", "sfx_dcburn",
	"sfx_dcslid", "sfx_dcdash", "sfx_dcatk", "sfx_dcdmg", "sfx_dcslam", "sfx_dcbrst", "sfx_dcslic",
	"sfx_susiel", "sfx_susieh", "sfx_susie6", "sfx_rudeba", "sfx_rudebh", "sfx_redba", "sfx_dcatkc", "sfx_dcexpl",
	"sfx_dcspla", "sfx_dctail", "sfx_dcblnt",
	"sfx_ralss1", "sfx_ralss2", "sfx_ralss3", "sfx_ralss4", "sfx_ralss5", "sfx_ralss6",
	"sfx_spare", "sfx_swing3", "sfx_dcsave",
	"sfx_dceggl", "sfx_rouxls", "sfx_dcbana", "sfx_fangus", "sfx_dcphon", "sfx_dcnull",
	"sfx_drtalk", "sfx_susie", "sfx_ralsei", "sfx_lancer", "sfx_lancel",
	"sfx_dceggm", "sfx_dcfang", "sfx_toriel", "sfx_dcquen", "sfx_dcspam")

sfxinfo[sfx_dcswch].caption = "/"
sfxinfo[sfx_drtalk].caption = "/"
sfxinfo[sfx_susie].caption = "/"
sfxinfo[sfx_ralsei].caption = "/"
sfxinfo[sfx_lancer].caption = "/"
sfxinfo[sfx_lancel].caption = "Laughing"
sfxinfo[sfx_dceggm].caption = "/"
sfxinfo[sfx_dcfang].caption = "/"
sfxinfo[sfx_fangus].caption = "/"
sfxinfo[sfx_dcphon].caption = "Phone Ringing"
sfxinfo[sfx_dcnull].caption = "/"
sfxinfo[sfx_toriel].caption = "/"
sfxinfo[sfx_dcquen].caption = "/"
sfxinfo[sfx_dcbana].caption = "Collect"
sfxinfo[sfx_dcspam].caption = "/"
sfxinfo[sfx_dcjump].caption = "Jump"
sfxinfo[sfx_dchurt].caption = "/"
sfxinfo[sfx_dcdyin].caption = "/"
sfxinfo[sfx_dcdead].caption = "Thud"
sfxinfo[sfx_dcfall].caption = "Falling"
sfxinfo[sfx_dcburn].caption = "Burning"
sfxinfo[sfx_susiel].caption = "Laughing"
sfxinfo[sfx_susieh].caption = "Grunt"
sfxinfo[sfx_susie6].caption = "Thrash"
sfxinfo[sfx_ralss1].caption = "Singing"
sfxinfo[sfx_ralss2].caption = "Singing"
sfxinfo[sfx_ralss3].caption = "Charging"
sfxinfo[sfx_ralss4].caption = "Charged"
sfxinfo[sfx_ralss5].caption = "Spell cast"
sfxinfo[sfx_ralss6].caption = "Sleeping"
sfxinfo[sfx_ralss6].singular = true
sfxinfo[sfx_spare].caption = "Spared"
sfxinfo[sfx_spare].singular = true
sfxinfo[sfx_swing3].caption = "Swinging"
sfxinfo[sfx_dcslid].caption = "/"
sfxinfo[sfx_dcdash].caption = "Dash"
sfxinfo[sfx_dcatk].caption = "Attack"
sfxinfo[sfx_dcdmg].caption = "Damage"
sfxinfo[sfx_dcslam].caption = "Impact"
sfxinfo[sfx_dcbrst].caption = "/"
sfxinfo[sfx_dceggl].caption = "Laughing"
sfxinfo[sfx_rouxls].caption = "Warping"
sfxinfo[sfx_dcslic].caption = "Slice"
sfxinfo[sfx_dcslic].flags = $ | SF_TOTALLYSINGLE
sfxinfo[sfx_rudeba].caption = "Buster"
sfxinfo[sfx_redba].caption = "Buster"
sfxinfo[sfx_rudebh].caption = "/"
sfxinfo[sfx_rudebh].singular = true
sfxinfo[sfx_rudebh].flags = $ | SF_X2AWAYSOUND
sfxinfo[sfx_dcatkc].caption = "Attack"
sfxinfo[sfx_dcexpl].caption = "Explosion"
sfxinfo[sfx_dcspla].caption = "Splat"
sfxinfo[sfx_dcsave].caption = "/"

// -------
// sprites
// -------

DC.SafeFreeslot("SPR_DCEX", "SPR_DCFX", "SPR_DCSP", "SPR_ROUX", "SPR_STWK",
	"SPR_RUDB", "SPR_LDED", "SPR_DCBN",
	"SPR2_CLPS", "SPR2_BURN",
	"SPR2_FKIK", "SPR2_SHIT")

// -------
// actions
// -------

DC.actions = {}

function DC.actions.SetPAnim(mo, var1, var2)
	local player = mo and mo.player
	if not player return end
	player.panim = var2
end

function DC.actions.AnimSpeedAdjust(mo, var1, var2)
	if not mo then return end
	local player = mo.player
	local speed
	
	if player
		speed = FixedDiv(player.speed, mo.scale)
	else
		speed = FixedDiv(FixedHypot(mo.momx, mo.momy), mo.scale)
	end
	
	if speed > 52*FRACUNIT
		mo.tics = 1
	elseif speed > 30*FRACUNIT
		mo.tics = 2
	elseif speed > 12*FRACUNIT
		mo.tics = 3
	end
	DC.actions.SetPAnim(mo, var1, var2)
end

function DC.actions.FallAnimSpeedAdjust(mo, var1, var2)
	if not mo then return end
	local speed = FixedDiv(abs(mo.momz), mo.scale)
	
	if (speed < 10<<FRACBITS)
		mo.tics = 4
	elseif (speed < 20<<FRACBITS)
		mo.tics = 3
	elseif (speed < 30<<FRACBITS)
		mo.tics = 2
	else
		mo.tics = 1
	end
end

function DC.actions.Terminate(mo, var1, var2)
	local player = mo and mo.player
	
	if not player
	or not (P_IsObjectOnGround(mo) or player.powers[pw_carry] == CR_ROLLOUT)
		return
	end
	
	if player.speed > FixedMul(player.runspeed, mo.scale)
		mo.state = S_PLAY_RUN
	elseif player.speed
		mo.state = S_PLAY_WALK
	else
		mo.state = S_PLAY_STND
	end
end

function DC.actions.CrossSlash(mo, var1, var2)
	local player = mo and mo.player
	
	if not player
	or player.skin ~= DC.Kris.GetSkinIndex()
	or not DC.CheckStatus(player)
		mo.state = S_DC_TERMINATE
		return
	end
	
	local frame = mo.frame & FF_FRAMEMASK
	local numFrames = DC.Kris.CROSS_SLASH_FRAMES
	
	if frame == A
		local kris = player.deltaChars.kris
		if kris.slashSide
			mo.frame = $ + numFrames
		end
		kris.slashSide = ($ + 1) & 1
		mo.tics = 4
		S_StartSound(mo, sfx_dcatk)
	elseif frame % numFrames == 1 and player.cmd.buttons & BT_JUMP
	and not (player.deltaChars.flags & DC.FLAGS.LUNGING)
		P_SetObjectMomZ(mo, DC.MobjAdjustMomZ(mo, DC.Kris.CROSS_SLASH_HOP), false)
	elseif frame % numFrames == 3
		mo.tics = 8
	elseif frame % numFrames == 0
		mo.state = var1
		return
	end
	
	DC.actions.SetPAnim(mo, var1, var2)
end

function DC.actions.Lunge(mo, var1, var2)
	local player = mo and mo.player
	
	if not player
	or player.skin ~= DC.Kris.GetSkinIndex()
	or not DC.CheckStatus(player)
		return
	end
	
	if not (player.deltaChars.flags & DC.FLAGS.LUNGING)
		mo.state = S_PLAY_FALL
		return
	end
	
	DC.actions.AnimSpeedAdjust(mo, var1, var2)
end

function DC.actions.AxCrash(mo, var1, var2)
	if not mo then return end
	local frame = mo.frame & FF_FRAMEMASK
	
	if frame == A
		S_StartSound(mo, sfx_dcdyin)
		mo.tics = 6
	elseif frame == B
		if mo.state == S_SUSIE_REDCRASH
			S_StartSound(mo, sfx_dcatkc)
		else
			S_StartSound(mo, sfx_dcatk)
		end
	end
	
	DC.actions.SetPAnim(mo, var1, var2)
end

function DC.actions.AxCrashLanding(mo, var1, var2)
	if not mo then return end
	local frame = mo.frame & FF_FRAMEMASK
	local targetFrame = skins[mo.skin].sprites[mo.sprite2 or SPR2_MLEL].numframes - 2
	
	if frame < targetFrame
		mo.frame = $ & ~FF_FRAMEMASK | targetFrame
	end
	
	DC.actions.SetPAnim(mo, var1, var2)
end

function DC.actions.RudeAttack(mo, var1, var2)
	local player = mo and mo.player
	
	if not player
	or player.skin ~= DC.Susie.GetSkinIndex()
	or not DC.CheckStatus(player)
		return
	end
	
	local frame = mo.frame & FF_FRAMEMASK
	
	if frame == A
		player.deltaChars.susie.attackEndTics = 0
		S_StartSound(mo, sfx_dcatk)
	elseif frame == B
		if P_IsObjectOnGround(mo)
			DC.Susie.StartQuake(player)
		end
	end
	
	DC.actions.SetPAnim(mo, var1, var2)
end

function DC.actions.RudeBusterFall(mo, var1, var2)
	DC.actions.AxCrashLanding(mo, var1, var2)
	DC.actions.FallAnimSpeedAdjust(mo, var1, var2)
end

// ------
// states
// ------

DC.SafeFreeslot("S_DC_SPARESPARKLE1", "S_DC_SPARESPARKLE2",
	"S_DC_EXPLOSION", "S_DC_DUST", "S_DC_SLASH", "S_DC_SLAM", "S_DC_SLAP", 'S_DC_SMOKE',
	"S_DC_LANCER_FALL", "S_DC_LANCER_SPLAT", "S_DC_ROUXLSLASER",
	"S_DC_EGGMANCOLLAPSE", "S_DC_BANANA", "S_DC_SAVE", "S_DC_HEART",
	"S_DC_COLLAPSE", "S_DC_DANCE", "S_DC_TERMINATE",
	"S_KRIS_SLIDE", "S_KRIS_LUNGE", "S_KRIS_CROSSSLASH",
	"S_SUSIE_AXCRASH", "S_SUSIE_AXCRASH_LANDING", "S_SUSIE_ATTACK",
	"S_SUSIE_RUDEBUSTER", "S_SUSIE_RUDEBUSTER_FALL", "S_SUSIE_REDCRASH",
	"S_SUSIE_PROJECTILE", "S_SUSIE_PROJECTILE_DEATHPART",
	"S_RALSEI_BLUSH", "S_RALSEI_SWIPE", "S_RALSEI_CAST", "S_RALSEI_MIST", "S_RALSEI_SLEEP", "S_RALSEI_FIRE")

states[S_DC_SPARESPARKLE1] = {
	sprite = SPR_NSPK,
	frame = FF_FULLBRIGHT|D,
	tics = 3,
	nextstate = S_DC_SPARESPARKLE2
}

states[S_DC_SPARESPARKLE2] = {
	sprite = SPR_NSPK,
	frame = FF_FULLBRIGHT|FF_ANIMATE,
	tics = 12,
	var1 = 3,
	var2 = 3
}

states[S_DC_EXPLOSION] = {
	sprite = SPR_DCEX,
	frame = FF_ANIMATE|FF_FULLBRIGHT,
	tics = 34,
	var1 = 16,
	var2 = 2
}

states[S_DC_DUST] = {
	sprite = SPR_DCFX,
	frame = FF_ANIMATE|FF_TRANS30,
	tics = 12,
	var1 = 3,
	var2 = 3
}

states[S_DC_SLASH] = {
	sprite = SPR_DCFX,
	frame = E|FF_ANIMATE|FF_FULLBRIGHT,
	tics = 9,
	var1 = 2,
	var2 = 3
}

states[S_DC_SLAM] = {
	sprite = SPR_DCFX,
	frame = H|FF_ANIMATE|FF_FULLBRIGHT,
	tics = 9,
	var1 = 2,
	var2 = 3
}

states[S_DC_SLAP] = {
	sprite = SPR_DCFX,
	frame = L|FF_ANIMATE|FF_FULLBRIGHT,
	tics = 12,
	var1 = 3,
	var2 = 3
}

states[S_DC_SMOKE] = {
	sprite = SPR_DCSP,
	frame = F|FF_ANIMATE,
	tics = 12,
	var1 = 3,
	var2 = 3
}

states[S_DC_LANCER_FALL] = {
	sprite = SPR_LDED,
	frame = FF_ANIMATE,
	var1 = 1,
	var2 = 2
}

states[S_DC_LANCER_SPLAT] = {
	sprite = SPR_LDED,
	frame = B,
	action = A_Pain
}

states[S_DC_ROUXLSLASER] = {
	sprite = SPR_ROUX,
	frame = M|FF_FULLBRIGHT|(10 << FF_TRANSSHIFT)
}

states[S_DC_EGGMANCOLLAPSE] = {
	sprite = SPR_ROUX,
	frame = N
}

states[S_DC_BANANA] = {
	sprite = SPR_DCBN,
	frame = FF_ANIMATE|FF_FULLBRIGHT,
	tics = -1,
	var1 = H,
	var2 = 2
}

states[S_DC_SAVE] = {
	sprite = SPR_DCBN,
	frame = FF_ANIMATE|FF_FULLBRIGHT|I,
	tics = -1,
	var1 = N - I,
	var2 = 2
}

states[S_DC_HEART] = {
	sprite = SPR_DCFX,
	frame = P,
	tics = -1,
}

states[S_DC_COLLAPSE] = {
	sprite = SPR_PLAY,
	frame = SPR2_CLPS
}

states[S_DC_DANCE] = {
	sprite = SPR_PLAY,
	frame = SPR2_FKIK,
	tics = 7,
	nextstate = S_DC_DANCE,
	action = DC.actions.SetPAnim,
	var2 = PA_DASH
}

states[S_DC_TERMINATE] = {
	sprite = SPR_PLAY,
	tics = 0,
	action = DC.actions.Terminate,
	nextstate = S_PLAY_FALL
}

states[S_KRIS_SLIDE] = {
	sprite = SPR_PLAY,
	frame = SPR2_GLID,
	tics = 4,
	action = DC.actions.AnimSpeedAdjust,
	var2 = PA_DASH,
	nextstate = S_KRIS_SLIDE
}

states[S_KRIS_LUNGE] = {
	sprite = SPR_PLAY,
	frame = SPR2_FRUN,
	tics = 4,
	action = DC.actions.Lunge,
	var2 = PA_ABILITY,
	nextstate = S_KRIS_LUNGE
}

states[S_KRIS_CROSSSLASH] = {
	sprite = SPR_PLAY,
	frame = SPR2_TWIN|FF_SPR2ENDSTATE,
	tics = 2,
	action = DC.actions.CrossSlash,
	var1 = S_KRIS_LUNGE,
	var2 = PA_ABILITY,
	nextstate = S_KRIS_CROSSSLASH
}

states[S_SUSIE_AXCRASH] = {
	sprite = SPR_PLAY,
	frame = SPR2_TWIN|FF_SPR2ENDSTATE,
	tics = 3,
	action = DC.actions.AxCrash,
	var1 = S_SUSIE_AXCRASH,
	var2 = PA_ABILITY,
	nextstate = S_SUSIE_AXCRASH
}

states[S_SUSIE_REDCRASH] = {
	sprite = SPR_PLAY,
	frame = SPR2_TAL0|FF_SPR2ENDSTATE,
	tics = 3,
	action = DC.actions.AxCrash,
	var1 = S_SUSIE_REDCRASH,
	var2 = PA_ABILITY,
	nextstate = S_SUSIE_REDCRASH
}

states[S_SUSIE_AXCRASH_LANDING] = {
	sprite = SPR_PLAY,
	frame = SPR2_MLEL|FF_SPR2ENDSTATE,
	tics = 6,
	action = DC.actions.AxCrashLanding,
	var1 = S_DC_TERMINATE,
	var2 = PA_ABILITY,
	nextstate = S_SUSIE_AXCRASH_LANDING
}

states[S_SUSIE_ATTACK] = {
	sprite = SPR_PLAY,
	frame = SPR2_MLEL|FF_SPR2ENDSTATE,
	tics = 3,
	action = DC.actions.RudeAttack,
	var1 = S_DC_TERMINATE,
	var2 = PA_ABILITY,
	nextstate = S_SUSIE_ATTACK
}

states[S_SUSIE_RUDEBUSTER] = {
	sprite = SPR_PLAY,
	frame = SPR2_MLEE|FF_SPR2ENDSTATE,
	tics = 3,
	action = DC.actions.SetPAnim,
	var1 = S_SUSIE_RUDEBUSTER_FALL,
	var2 = PA_ABILITY,
	nextstate = S_SUSIE_RUDEBUSTER
}

states[S_SUSIE_RUDEBUSTER_FALL] = {
	sprite = SPR_PLAY,
	frame = SPR2_MLEE,
	tics = 4,
	action = DC.actions.RudeBusterFall,
	var2 = PA_FALL,
	nextstate = S_SUSIE_RUDEBUSTER_FALL
}

states[S_SUSIE_PROJECTILE] = {
	sprite = SPR_RUDB,
	frame = FF_ANIMATE|FF_FULLBRIGHT,
	tics = -1,
	var1 = 1,
	var2 = 3
}

states[S_SUSIE_PROJECTILE_DEATHPART] = {
	sprite = SPR_RUDB,
	frame = C|FF_ANIMATE|FF_FULLBRIGHT,
	var1 = 1,
	var2 = 3
}

states[S_RALSEI_BLUSH] = {
	sprite = SPR_PLAY,
	frame = SPR2_MLEL,
	action = DC.actions.SetPAnim,
	var2 = PA_IDLE
}

states[S_RALSEI_SWIPE] = {
	sprite = SPR_PLAY,
	frame = SPR2_FLY,
	tics = TICRATE*3/2,
	action = DC.actions.SetPAnim,
	var2 = PA_ABILITY,
	nextstate = S_DC_TERMINATE
}

states[S_RALSEI_CAST] = {
	sprite = SPR_PLAY,
	frame = SPR2_FIRE,
	tics = 7,
	action = DC.actions.SetPAnim,
	var2 = PA_DASH,
	nextstate = S_DC_TERMINATE
}

states[S_RALSEI_MIST] = {
	sprite = SPR_DCSP,
	frame = A|FF_ANIMATE|FF_FULLBRIGHT|FF_TRANS20,
	tics = 5,
	var1 = 4,
	var2 = 1
}

states[S_RALSEI_SLEEP] = {
	sprite = SPR_DCSP,
	frame = J|FF_ANIMATE|FF_FULLBRIGHT|FF_TRANS50,
	tics = 21,
	var1 = 6,
	var2 = 3
}

states[S_RALSEI_FIRE] = {
	sprite = SPR_DCSP,
	frame = Q|FF_ANIMATE|FF_FULLBRIGHT,
	tics = -1,
	var1 = 7,
	var2 = 3
}

// -------
// objects
// -------

DC.SafeFreeslot("MT_DC_SPARESPARKLE", "MT_DC_LANCER", "MT_DC_BANANA", "MT_DC_HEART",
	"MT_SUSIE_PROJECTILE", "MT_SUSIE_PROJECTILE_TRAIL", "MT_SUSIE_PROJECTILE_DEATHPART",
	"MT_RALSEI_PROJECTILE")

mobjinfo[MT_DC_SPARESPARKLE] = {
	spawnstate = S_DC_SPARESPARKLE1,
	spawnhealth = 1000,
	radius = 4*FRACUNIT,
	height = 8*FRACUNIT,
	flags = DC.MF_INTANGIBLE|MF_NOGRAVITY|MF_SCENERY
}

mobjinfo[MT_DC_LANCER] = {
	spawnstate = S_DC_LANCER_FALL,
	painstate = S_DC_LANCER_SPLAT,
	painsound = sfx_dcspla,
	radius = 16*FRACUNIT,
	height = 32*FRACUNIT,
	flags = MF_NOBLOCKMAP|MF_SCENERY
}

mobjinfo[MT_DC_BANANA] = {
	spawnstate = S_DC_BANANA,
	deathstate = S_CYBRAKDEMONNAPALMBOMBSMALL_DIE5,
	deathsound = sfx_dcbana,
	radius = 16*FRACUNIT,
	height = 24*FRACUNIT,
	flags = MF_SLIDEME|MF_SPECIAL|MF_NOGRAVITY|MF_NOCLIPHEIGHT
}

mobjinfo[MT_DC_HEART] = {
	spawnstate = S_DC_HEART,
	deathstate = S_CYBRAKDEMONNAPALMBOMBSMALL_DIE5,
	deathsound = sfx_dcbana,
	speed = 8*FRACUNIT,
	radius = 8*FRACUNIT,
	height = 16*FRACUNIT,
	flags = MF_NOBLOCKMAP|MF_BOUNCE|MF_GRENADEBOUNCE|MF_PUSHABLE
}

mobjinfo[MT_SUSIE_PROJECTILE] = {
	spawnstate = S_SUSIE_PROJECTILE,
	spawnhealth = 1000,
	height = 79*FRACUNIT,
	radius = 16*FRACUNIT,
	speed = 45*FRACUNIT,
	reactiontime = 5*TICRATE/2, // lifetime (fuse)
	painchance = 512*FRACUNIT, // homing detection range
	deathsound = sfx_rudebh,
	deathstate = S_FBOMB_EXPL6,
	flags = MF_NOGRAVITY|MF_MISSILE|MF_NOCLIPHEIGHT
}

mobjinfo[MT_SUSIE_PROJECTILE_TRAIL] = {
	spawnstate = S_SUSIE_PROJECTILE,
	spawnhealth = 1000,
	height = 76*FRACUNIT,
	radius = 16*FRACUNIT,
	speed = 45*FRACUNIT,
	reactiontime = 10,
	flags = DC.MF_INTANGIBLE|MF_NOGRAVITY|MF_SCENERY
}

mobjinfo[MT_SUSIE_PROJECTILE_DEATHPART] = {
	spawnstate = S_SUSIE_PROJECTILE_DEATHPART,
	spawnhealth = 1000,
	height = 128*FRACUNIT,
	radius = 64*FRACUNIT,
	flags = DC.MF_INTANGIBLE|MF_NOGRAVITY|MF_SCENERY
}

mobjinfo[MT_RALSEI_PROJECTILE] = {
	spawnstate = S_JETFUMEFLASH,
	spawnhealth = 1000,
	height = 16*FRACUNIT,
	radius = 16*FRACUNIT,
	speed = 32*FRACUNIT,
	reactiontime = 2*TICRATE, // lifetime (fuse)
	deathsound = sfx_s3k81,
	deathstate = S_CACOFIRE_EXPLODE1,
	flags = MF_NOGRAVITY|MF_MISSILE
}