/*
Susan

v1
*/

local DC = DeltaChars
local AA = DC.AxAngle
local Susie = {}

DC.Susie = Susie

// ----
// defs
// ----

Susie.SKIN = "susie"

DC.FLAGS.SUSIE_MASK = DC.RegisterFlag("FALLING")
	| DC.RegisterFlag("CRASHING")
	| DC.RegisterFlag("QUEUE_ATTACK")
	| DC.RegisterFlag("RUDE_BUSTER")
	| DC.RegisterFlag("RED_BUSTER")
	| DC.RegisterFlag("SECRET_COOLDOWN")

Susie.HURT_SOUND_CHANCE = FRACUNIT/4 // FRACUNIT = 100%

Susie.AX_CRASH_HOP = 5*FRACUNIT

Susie.RUDE_BUSTER_JUMP = 16*FRACUNIT

Susie.RED_BUSTER_CONE = ANG20
Susie.RED_BUSTER_COLOR = SKINCOLOR_FLAME

Susie.SHOCKWAVE_RANGE = 512*FRACUNIT
Susie.SHOCKWAVE_POWER = 5*FRACUNIT

Susie.AX_WIDTH = 32*FRACUNIT
Susie.REACH_FLAGS = {
	RADIAL = 1 << 0,
	EXCLUDE_HEIGHT = 1 << 1
}
Susie.AX_REACH = {
	[SPR2_MLEL] = {
		[A] = {h = 126*FRACUNIT, v = 150*FRACUNIT},
		[B] = {h = 132*FRACUNIT, v = 146*FRACUNIT},
		[C] = {h = 100*FRACUNIT, v = 0} // 0 = use her radius/height, which I think is fair for anything that would otherwise be smaller than her visual
		// omitting any frames will use the last entry's reach, in this case D has the same reach as C
	},
	[SPR2_TWIN] = {
		[A] = {h = -78*FRACUNIT, v = 0},
		[B] = {h = 110*FRACUNIT, v = 158*FRACUNIT},
		[C] = {h = 116*FRACUNIT, v = 154*FRACUNIT},
		[D] = {h = 84*FRACUNIT, v = 0}
	},
	[SPR2_MLEE] = {
		[A] = {h = 81*FRACUNIT, v = 135*FRACUNIT, flags = Susie.REACH_FLAGS.RADIAL},
		[B] = {h = 81*FRACUNIT, v = 137*FRACUNIT, flags = Susie.REACH_FLAGS.RADIAL},
		[C] = {h = 53*FRACUNIT, v = 115*FRACUNIT},
		[D] = {h = 53*FRACUNIT, v = 113*FRACUNIT},
		[E] = {h = 61*FRACUNIT, v = 99*FRACUNIT, flags = Susie.REACH_FLAGS.EXCLUDE_HEIGHT}
	}
}

Susie.AX_REACH[SPR2_TAL0] = Susie.AX_REACH[SPR2_TWIN]

//States to cancel out of when switching out of this character
Susie.CancelStates = {}
local statelist = {
	S_SUSIE_AXCRASH, S_SUSIE_REDCRASH, S_SUSIE_AXCRASH_LANDING, S_SUSIE_ATTACK,
	S_SUSIE_RUDEBUSTER, S_SUSIE_RUDEBUSTER_FALL
}
for i, state in ipairs(statelist)
	Susie.CancelStates[state] = true
end

// ----
// vars
// ----

Susie.reach = {}

// ---------
// functions
// ---------

function Susie.GetSkinIndex()
	if not Susie.skinIndex
		Susie.skinIndex = #skins[Susie.SKIN]
	end
	return Susie.skinIndex
end

function Susie.CreateTable(player)
	player.deltaChars.susie = {
		axCrashButton = BT_JUMP
	}
end

function Susie.RemoveTable(player)
	local delta = player.deltaChars
	
	if delta.susie == nil
		return
	end
	
	delta.flags = $ & ~DC.FLAGS.SUSIE_MASK
	delta.susie = nil
	
	return true
end

function Susie.UpdatePlayer(player)
	if player.playerstate ~= PST_LIVE
	or player.skin ~= Susie.GetSkinIndex()
		return
	end
	
	if player.deltaChars.hitlag <= 0
		Susie.UpdateVars(player)
		Susie.HandleAxCrash(player)
		Susie.HandleAxCrashLanding(player)
		Susie.HandleRudeBuster(player)
	end
	Susie.HandleAxDamage(player)
	Susie.HandleDebugHitbox(player)
end

function Susie.UpdateVars(player)
	local mo = player.mo
	local delta = player.deltaChars
	
	// flags Susie as falling so she can ignore crawla bounces while using her axe
	if P_IsObjectOnGround(mo)
	or P_MobjFlip(mo)*mo.momz > 0
		delta.flags = $ & ~DC.FLAGS.FALLING
	else
		delta.flags = $ | DC.FLAGS.FALLING
	end
end

// returns which angle Susie should use for her attack
function Susie.GetAxAngle(player, type)
	local AA = DC.AxAngle
	local value = player[AA.PLAYER_VARIABLE]
	
	if value == nil
		value = AA.DEFAULT
	end
	
	if value == AA.ALL
	or value & type
		return player.mo.angle
	end
	
	return player.drawangle
end

// finds a mobj nearby @mo that is targetable by Rude Buster (unused)
function Susie.LookForEnemy(mo)
	local range = FixedMul(mobjinfo[MT_SUSIE_PROJECTILE].painchance, mo.scale)
	Susie.reach.h = range
	Susie.foundMobj = nil
	searchBlockmap("objects", DC.Susie.ProjectileFindTarget, mo,
		mo.x - range, mo.x + range, mo.y - range, mo.y + range)
	return Susie.foundMobj
end

// returns whether or not Susie can use her attacks
function Susie.CanStartAttack(player)
	return not (player.pflags & DC.PFLAGS_BLACKLIST)
	and not P_PlayerInPain(player)
end

// handles Susie's attacks
function Susie.HandleRudeBuster(player)
	local mo = player.mo
	local delta = player.deltaChars
	local susie = delta.susie
	
	// press spin to start an attack
	if (DC.ButtonDown(player, BT_SPIN) or delta.flags & DC.FLAGS.QUEUE_ATTACK)
	and not Susie.CanAttack(player)
		local carry = player.powers[pw_carry]
		delta.flags = $ & ~DC.FLAGS.QUEUE_ATTACK
		if Susie.CanStartAttack(player)
		and (carry == CR_ROLLOUT or carry == CR_NONE and P_IsObjectOnGround(mo))
			DC.SetPlayerDrawangle(player, Susie.GetAxAngle(player, AA.GROUND))
			mo.state = S_SUSIE_ATTACK
			delta.flags = $ & ~(DC.FLAGS.RED_BUSTER|DC.FLAGS.SECRET_COOLDOWN) | DC.FLAGS.RUDE_BUSTER
		end
	end
	
	// while in the attack state...
	if mo.state == S_SUSIE_ATTACK
		if player.gotflag
			// no buster with the flag!
			
		// ...hold spin all the way through to activate Rude Buster
		elseif delta.flags & DC.FLAGS.RUDE_BUSTER
			if not DC.ButtonDown(player, BT_SPIN, true)
				delta.flags = $ & ~DC.FLAGS.RUDE_BUSTER
			else
				if mo.tics == 1
				and mo.frame & FF_FRAMEMASK == skins[player.skin].sprites[mo.sprite2].numframes - 1
					mo.state = S_SUSIE_RUDEBUSTER
					
					/*Susie.LookForEnemy(mo)
					if Susie.foundMobj
						DC.SetPlayerDrawangle(player, R_PointToAngle2(mo.x, mo.y, Susie.foundMobj.x, Susie.foundMobj.y))
					end*/
					DC.SetPlayerDrawangle(player, Susie.GetAxAngle(player, AA.BUSTER))
					Susie.SpawnProjectile(player, Susie.foundMobj)
					
					delta.lockAngle = player.drawangle + ANGLE_180
					DC.SetPlayerDrawangle(player, delta.lockAngle)
					
					P_DoJump(player, false)
					P_SetObjectMomZ(mo, DC.MobjAdjustMomZ(mo, Susie.RUDE_BUSTER_JUMP))
					player.pflags = $ & ~PF_STARTJUMP
					delta.flags = $ & ~DC.FLAGS.RUDE_BUSTER
					return
				end
				
				/*if Susie.LookForEnemy(mo)
					P_SpawnLockOn(player, Susie.foundMobj, S_LOCKON1)
				end*/
			end
		// ...or press spin again to perform another attack
		elseif DC.ButtonDown(player, BT_SPIN)
			delta.flags = $ | DC.FLAGS.QUEUE_ATTACK
		end
		
		// disable movement (but keep SOME speed, jeez)
		player.pflags = $ | PF_FULLSTASIS | PF_JUMPDOWN | PF_SPINDOWN
		mo.friction = ($ + FRACUNIT) / 2
	end
	
	// while in the Rude Buster state, rotate 180 degrees
	if mo.state == S_SUSIE_RUDEBUSTER
		DC.SetPlayerDrawangle(player, delta.lockAngle)
	end
end

// starts Ax Crash
function Susie.StartAxCrash(player, button)
	local mo = player.mo
	local delta = player.deltaChars
	
	delta.susie.axCrashButton = button
	delta.flags = $ & ~DC.FLAGS.SECRET_COOLDOWN | DC.FLAGS.CRASHING
	player.pflags = $ & ~PF_SPINNING | PF_THOKKED | PF_NOJUMPDAMAGE
	
	P_SetObjectMomZ(mo, Susie.AX_CRASH_HOP)
	if mo.state == S_SUSIE_RUDEBUSTER
	or mo.state == S_SUSIE_RUDEBUSTER_FALL
		delta.flags = $ | DC.FLAGS.RED_BUSTER
		mo.state = S_SUSIE_REDCRASH
	else
		delta.flags = $ & ~DC.FLAGS.RED_BUSTER
		mo.state = S_SUSIE_AXCRASH
	end
end

// handles Ax Crash
function Susie.HandleAxCrash(player)
	local mo = player.mo
	local delta = player.deltaChars
	
	// press jump or spin in midair to activate ax crash
	if not (player.pflags & PF_THOKKED)
	and ((player.gotflag and (not (player.pflags & PF_STARTJUMP) or not DC.ButtonDown(player, BT_JUMP)))
	or not (player.pflags & PF_JUMPED))
	and not P_IsObjectOnGround(mo)
	and player.powers[pw_carry] == CR_NONE
	and Susie.CanStartAttack(player)
		if DC.ButtonDown(player, BT_SPIN)
			Susie.StartAxCrash(player, BT_SPIN)
		elseif DC.ButtonDown(player, BT_JUMP)
			Susie.StartAxCrash(player, BT_JUMP)
		end
	end
	
	if delta.flags & DC.FLAGS.CRASHING
		// landed on the ground - THUD!
		if P_IsObjectOnGround(mo)
			delta.flags = $ & ~(DC.FLAGS.CRASHING|DC.FLAGS.SECRET_COOLDOWN)
			mo.state = S_SUSIE_AXCRASH_LANDING
			Susie.StartQuake(player)
			if delta.flags & DC.FLAGS.RED_BUSTER
				DC.SetPlayerDrawangle(player, Susie.GetAxAngle(player, AA.BUSTER|AA.AIR))
				Susie.SpawnProjectile(player)
				delta.flags = $ & ~DC.FLAGS.RED_BUSTER
			else
				DC.SetPlayerDrawangle(player, Susie.GetAxAngle(player, AA.AIR))
			end
			return
		end
		
		// we got cancelled on twitter.com
		if not Susie.InCrashState(player)
			delta.flags = $ & ~(DC.FLAGS.CRASHING|DC.FLAGS.RED_BUSTER)
			return
		end
		
		// let go of jump - start freefall
		if not DC.ButtonDown(player, delta.susie.axCrashButton, true)
			delta.flags = $ & ~(DC.FLAGS.CRASHING|DC.FLAGS.RED_BUSTER)
		end
	end
	
	if Susie.InCrashState(player)
		local frame = mo.frame & FF_FRAMEMASK
		
		DC.SetPlayerDrawangle(player, Susie.GetAxAngle(player, AA.AIR))
		
		// we just started this animation, don't do anything
		if frame == A
			return
		end
		
		// while holding jump, accelerate to the ground
		if delta.flags & DC.FLAGS.CRASHING
			if not (mo.eflags & MFE_GOOWATER)
				mo.momz = $ - P_GetMobjGravity(mo)
				P_SetObjectMomZ(mo, -2*FRACUNIT, true)
			end
			
			// animate last 2 frames
			if mo.tics == 1
			and frame == skins[player.skin].sprites[mo.sprite2].numframes - 1
				mo.frame = $ - 1
				mo.tics = states[mo.state].tics - 1
			end
			
			// spawn Red Buster ghosts
			if delta.flags & DC.FLAGS.RED_BUSTER
			and leveltime % 3 == 0
				local ghost = P_SpawnGhostMobj(mo)
				if G_GametypeHasTeams()
					ghost.color = player.skincolor
				else
					ghost.color = Susie.RED_BUSTER_COLOR
				end
				ghost.colorized = true
				ghost.tics = 6
			end
			
			// bust floors (no longer needed since she uses CA_TWINSPIN
			//DC.CheckVerticalBustables(player)
			
			return
		end
		
		// if we let go of jump, just finish the animation
		if mo.tics == 1
		and frame == skins[player.skin].sprites[mo.sprite2].numframes - 1
			mo.state = S_PLAY_FALL
		end
	end
end

// handles Ax Crash's landing state
function Susie.HandleAxCrashLanding(player)
	local mo = player.mo
	if mo.state == S_SUSIE_AXCRASH_LANDING
		if DC.ButtonDown(player, BT_SPIN)
			player.deltaChars.flags = $ | DC.FLAGS.QUEUE_ATTACK
		end
		player.pflags = $ | PF_FULLSTASIS
		if P_IsObjectOnGround(mo)
		and mo.tics == states[mo.state].tics - 1
			Susie.DustRing(player)
		end
	end
end

// returns the appropriate ax reach table
function Susie.GetReach(player, frame)
	local mo = player.mo
	local info = Susie.AX_REACH[mo.sprite2 & ~FF_SPR2SUPER]
	
	if not info
		return false
	end
	
	if frame == nil
		frame = mo.frame & FF_FRAMEMASK
	end
	
	local reach = Susie.reach
	
	info = $[frame] or $[#$]
	
	reach.h = FixedMul(info.h, mo.scale) or mo.radius
	reach.v = FixedMul(info.v, mo.scale) or mo.height
	reach.flags = info.flags or 0
	return true
end

// searches the blockmap for nearby objects and damages them if Susie's ax can reach them
function Susie.HandleAxDamage(player)
	if not Susie.GetReach(player)
		return
	end
	
	local mo = player.mo
	local delta = player.deltaChars
	local frame = mo.frame & FF_FRAMEMASK
	local prevFrame = delta.prevFrame & FF_FRAMEMASK
	
	local reach = abs(Susie.reach.h)
	
	searchBlockmap("lines", Susie.TryAxBust, mo,
		mo.x - reach, mo.x + reach,
		mo.y - reach, mo.y + reach)
	
	searchBlockmap("objects", Susie.TryAxAttack, mo,
		mo.x - reach, mo.x + reach,
		mo.y - reach, mo.y + reach)
	
	if mo.state ~= delta.prevState // if Susie just changed state, only ignore springs while in the first frame
		if frame == A
			return
		end
	elseif frame ~= prevFrame // if we are in the same state, use the previous frame's reach data to hit springs
		if frame == A // no springing things on the first tic in the state
		or not Susie.GetReach(player, prevFrame) // should never happen, but worth avoiding warnings if other scripts do funky state things
			return
		end
		
		reach = abs(Susie.reach.h)
	end
	
	searchBlockmap("objects", Susie.TryAxSpring, mo,
		mo.x - reach, mo.x + reach,
		mo.y - reach, mo.y + reach)
end

// springs @mo from @item if it is within range of susie's ax
// Susie.GetReach MUST be called and return true before this can be called
function Susie.TryAxSpring(mo, item)
	local player = mo.player
	
	if not Susie.AxSprungByMobj(player, item)
	or not Susie.AxTouchesMobj(player, item)
		return
	end
	
	P_DoSpring(item, mo)
end

// damages @item if it is damageable and within range of susie's ax
// Susie.GetReach MUST be called and return true before this can be called
function Susie.TryAxAttack(mo, item)
	if not Susie.MobjIsAxable(item)
		return
	end
	
	local player = mo.player
	
	if not Susie.AxTouchesMobj(player, item)
	or Susie.AxSprungByMobj(player, item) // leave this for TryAxSpring
		return
	end
	
	// bust spikes
	if item.type == MT_SPIKE
	or item.type == MT_WALLSPIKE
		DC.KillSpike(item, mo)
		return
	end
	
	// spin out Metal Sonic
	if item.type == MT_METALSONIC_BATTLE
	and item.state == item.info.spawnstate
		item.flags2 = $ | MF2_CLASSICPUSH
		return
	end
	
	// other breakable objects
	if Susie.AxSpecial(player, item)
		return
	end
	
	// default behavior: damage object
	P_DamageMobj(item, item.player and mo, mo)
end

function Susie.AxTouchesMobj(player, item)
	local mo = player.mo
	local reach = Susie.reach
	
	// height checks
	if mo.eflags & MFE_VERTICALFLIP
		local momz = max(mo.momz, 0) - (reach.flags & Susie.REACH_FLAGS.EXCLUDE_HEIGHT and mo.height or 0)
		if mo.z + mo.height + momz < item.z
		or mo.z + mo.height - reach.v > item.z + item.height
			return false
		end
	else
		local momz = min(mo.momz, 0) + (reach.flags & Susie.REACH_FLAGS.EXCLUDE_HEIGHT and mo.height or 0)
		if mo.z + momz > item.z + item.height
		or mo.z + reach.v < item.z
			return false
		end
	end
	
	// horizontal checks
	local dist = FixedHypot(mo.x - item.x, mo.y - item.y)
	
	if reach.flags & Susie.REACH_FLAGS.RADIAL
		return dist <= reach.h
	end
	
	local angle = R_PointToAngle2(mo.x, mo.y, item.x, item.y) - player.drawangle + ANGLE_90
	local x, y = FixedMul(cos(angle), dist), FixedMul(sin(angle), dist) // transform the object's coordinates to be relative to the player's position and angle
	
	local xDist = abs(x) - item.radius
	
	if xDist > FixedMul(Susie.AX_WIDTH, mo.scale)
		return false // missed
	end
	
	local yDist = abs(y) - item.radius
	
	if yDist > abs(reach.h)
		return false // ax not long enough
	end
	
	if yDist > 0
	and FixedMul(reach.h, y) < 0
		return false // facing the wrong way
	end
	
	return true
end

function Susie.AxSprungByMobj(player, item)
	if item.health <= 0
		return false
	end
	
	if item.type == MT_SPRINGSHELL
	or item.type == MT_YELLOWSHELL // I really didn't need to do this LOL
		local mo = player.mo
		if mo.eflags & MFE_VERTICALFLIP
			if mo.z + mo.height - mo.momz < item.z
			or mo.z + mo.height < item.z + 8*item.scale
				return true
			end
		else
			if mo.z - mo.momz > item.z + item.height
			or mo.z > item.z + item.height - 8*item.scale
				return true
			end
		end
	end
	
	return item.flags & MF_SPRING
end

// busts fofs within range of susie's ax
// Susie.reach.h and Susie.reach.v MUST be set before this can be called
function Susie.TryAxBust(mo, line)
	local reach = Susie.reach 
	
	if reach.flags & Susie.REACH_FLAGS.RADIAL
		local x, y = DC.ClosestPointOnLine(line, mo.x, mo.y)
		local dist = FixedHypot(mo.x - x, mo.y - y)
		
		if dist > reach.h
			return
		end
	else
		local drawangle = mo.player.drawangle
		local v1, v2 = line.v1, line.v2
		local x1, y1, x2, y2 = v1.x, v1.y, v2.x, v2.y
		
		local dist1 = FixedHypot(mo.x - x1, mo.y - y1)
		local dist2 = FixedHypot(mo.x - x2, mo.y - y2)
		local angle1 = R_PointToAngle2(mo.x, mo.y, x1, y1) - drawangle + ANGLE_90
		local angle2 = R_PointToAngle2(mo.x, mo.y, x2, y2) - drawangle + ANGLE_90
		
		// transform the vertex coordinates to be relative to the player's position and angle
		x1, y1 = FixedMul(cos(angle1), dist1), FixedMul(sin(angle1), dist1)
		x2, y2 = FixedMul(cos(angle2), dist2), FixedMul(sin(angle2), dist2)
		
		if not Susie.AxIntersection(mo, x1, y1, x2, y2)
			return
		end
	end
	
	// ok, now bust the fofs in both sectors
	local bottom
	local top
	
	if mo.eflags & MFE_VERTICALFLIP
		bottom = mo.z + mo.height - reach.v
		if reach.flags & Susie.REACH_FLAGS.EXCLUDE_HEIGHT
			top = mo.z
		else
			top = mo.z + mo.height
		end
	else
		if reach.flags & Susie.REACH_FLAGS.EXCLUDE_HEIGHT
			bottom = mo.z + mo.height
		else
			bottom = mo.z
		end
		top = mo.z + reach.v
	end
	
	if line.frontsector
		local sector = line.frontsector
		DC.TryRunHook(sector.tag, "SusieAxSector", mo.player, sector, bottom, top)
		DC.MobjBustFOFsInSector(mo, bottom, top, sector)
	end
	
	if line.backsector
		local sector = line.backsector
		DC.TryRunHook(sector.tag, "SusieAxSector", mo.player, sector, bottom, top)
		DC.MobjBustFOFsInSector(mo, bottom, top, sector)
	end
end

// x1, x2, y1, y2 MUST be projected relative to @mo's position and the hitbox's angle
// (i.e. do NOT feed raw map coordinates into this function)
function Susie.AxIntersection(mo, x1, y1, x2, y2)
	local reach = Susie.reach
	local width = FixedMul(Susie.AX_WIDTH, mo.scale)
	local length = abs(reach.h)
	
	// if both coordinates are outside the hitbox and on the same side of the hitbox, intersection is impossible
	if x1 > width
	and x2 > width
	or x1 < -width
	and x2 < -width
		return false
	end
	
	if y1 > length
	and y2 > length
	or y1 < -length
	and y2 < -length
		return false
	end
	
	// if both y coordinates are on the wrong side of Susie's ax, the line doesn't intersect
	if FixedMul(y1, reach.h) < 0
	and FixedMul(y2, reach.h) < 0
		return false
	end
	
	// at this point, any perfectly vertical or horizontal line is guaranteed to intersect
	if x1 == x2
	or y1 == y2
		return true
	end
	
	// if either coordinate is inside the hitbox, intersection is guaranteed
	if abs(x1) <= width
	and abs(y1) <= length
	and FixedMul(y1, reach.h) >= 0
		return true
	end
	
	if abs(x2) <= width
	and abs(y2) <= length
	and FixedMul(y2, reach.h) >= 0
		return true
	end
	
	// ok, so both coordinates are outside the hitbox, but intersection is still possible
	// we'll find the slope of the line;
	// if it's positive, we'll compare it with the top left and bottom right hitbox coordinates
	// if it's negative, we'll compare it with the top right and bottom left hitbox coordinates
	// if the two chosen coordinates are on opposite sides of the line, then we have intersected it
	
	local left, right = -width, width
	local top, bottom
	
	if reach.h < 0
		top = 0
		bottom = reach.h
	else
		top = reach.h
		bottom = 0
	end
	
	if ((x2 - x1) > 0) == ((y2 - y1) > 0) // positive gradient
		return P_PointOnLineSide(left, top, x1, y1, x2, y2) ~= P_PointOnLineSide(right, bottom, x1, y1, x2, y2)
	else // negative gradient
		return P_PointOnLineSide(right, top, x1, y1, x2, y2) ~= P_PointOnLineSide(left, bottom, x1, y1, x2, y2)
	end
end

// returns whether or not @item is attackable by ax
function Susie.MobjIsAxable(item)
	if item.health <= 0
	// or item.player <- LAME AND UNFUN LACH!
		return false
	end
	
	return (item.flags & MF_SHOOTABLE or item.type == MT_METALSONIC_BATTLE)
		and not (item.flags & (MF_ENEMY|MF_BOSS) and item.flags2 & MF2_FRET)
	or item.type == MT_SPIKE or item.type == MT_WALLSPIKE
	or DC.MobjIsDestructable(item)
end

// checks for and performs special actions when @player touches @item during an attack
function Susie.AxSpecial(player, item)
	local mo = player.mo
	
	// bust destructables
	if DC.MobjIsDestructable(item)
		Susie.KillDestructable(item, mo)
		return true
	end
	
	// knock mines away
	if item.type == MT_BIGMINE
		if item.flags & MF_SPECIAL
			local speed = FixedHypot(mo.momx, mo.momy, mo.momz)
			if speed ~= 0
				item.momx = 24*FixedDiv(mo.momx, speed)
				item.momy = 24*FixedDiv(mo.momy, speed)
				item.momz = 24*FixedDiv(mo.momz, speed)
				mo.momx = -$ / 8
				mo.momy = -$ / 8
				mo.momz = -$ / 8
				item.flags = $ & ~MF_SPECIAL
				S_StartSound(item, item.info.activesound)
				item.tracer = mo
				
				if item.state < item.info.meleestate
					item.state = item.info.meleestate
				end
				S_StartSound(mo, sfx_bsnipe)
				DC.SetHitlag(player, 6)
			end
		end
		return true
	end
end

// spawns a hitbox plane
function Susie.SpawnDebugPlane(player, x, y, z)
	local plane = P_SpawnMobj(x, y, z, MT_THOK)
	plane.sprite = SPR_DCFX
	plane.frame = K | FF_TRANS50
	plane.tics = 1
	plane.renderflags = $ | RF_FULLBRIGHT | RF_NOCOLORMAPS
	plane.color = player.skincolor
	return plane
end

// Displays hitbox bounds for the ax
function Susie.HandleDebugHitbox(player)
	if not DC.debug.value
	or netgame
	or not Susie.GetReach(player)
		return
	end
	
	local mo = player.mo
	local reach = Susie.reach
	local bottomZ, topZ, x, y
	
	// get top and bottom positions
	if mo.eflags & MFE_VERTICALFLIP
		local momz = max(mo.momz, 0) - (reach.flags & Susie.REACH_FLAGS.EXCLUDE_HEIGHT and mo.height or 0)
		topZ = mo.z + mo.height + momz
		bottomZ = mo.z + mo.height - reach.v
	else
		local momz = min(mo.momz, 0) + (reach.flags & Susie.REACH_FLAGS.EXCLUDE_HEIGHT and mo.height or 0)
		bottomZ = mo.z + momz
		topZ = mo.z + reach.v
	end
	
	if reach.flags & Susie.REACH_FLAGS.RADIAL
		local plane = Susie.SpawnDebugPlane(player,
			mo.x, mo.y, bottomZ)
		plane.spriteyscale = topZ - bottomZ
		plane.spritexscale = reach.h
	else
		local vScale = topZ - bottomZ
		local angle = player.drawangle
		local axWidth = FixedMul(Susie.AX_WIDTH, mo.scale)
		
		// draw front and back planes
		local hScale = axWidth
		local xThrust = FixedMul(reach.h, cos(angle))
		local yThrust = FixedMul(reach.h, sin(angle))
		for i = FRACUNIT >> 8, FRACUNIT, FRACUNIT - (FRACUNIT >> 8)
			local plane = Susie.SpawnDebugPlane(player,
				mo.x + FixedMul(i, xThrust), mo.y + FixedMul(i, yThrust), bottomZ)
			plane.spriteyscale = vScale
			plane.spritexscale = hScale
			plane.angle = angle + ANGLE_90
			plane.renderflags = $ | RF_PAPERSPRITE
		end
		
		// draw side planes
		xThrust = $ / 2
		yThrust = $ / 2
		hScale = abs(reach.h)/2
		for i = -1, 1, 2
			local plane = Susie.SpawnDebugPlane(player,
				mo.x + xThrust + i*FixedMul(axWidth, cos(angle + ANGLE_90)),
				mo.y + yThrust + i*FixedMul(axWidth, sin(angle + ANGLE_90)),
				bottomZ)
			plane.spriteyscale = vScale
			plane.spritexscale = hScale
			plane.angle = angle
			plane.renderflags = $ | RF_PAPERSPRITE
		end
	end
end

// blockmap search function for projectile target (unused)
function Susie.ProjectileFindTarget(mo, item)
	// can we hit and target this thing?
	if item.flags & MF_NOCLIPTHING
	or not (item.flags & MF_SHOOTABLE and item.flags & (MF_MONITOR|MF_ENEMY|MF_BOSS)) == not (item.flags2 & MF2_INVERTAIMABLE)
	or item.health <= 0
	or item.flags2 & MF2_FRET
		return
	end
	
	// is it in front of us?
	if abs(R_PointToAngle2(mo.x, mo.y, item.x, item.y) - mo.angle) > ANGLE_90
		return
	end
	
	// is it within range?
	local dist = FixedHypot(FixedHypot(mo.x - item.x, mo.y - item.y),
		(mo.z + (mo.height >> 1)) - (item.z + (item.height >> 1)))
	
	if dist > Susie.reach.h
		return
	end
	
	// can we "see" it?
	if not P_CheckSight(mo, item)
		return
	end
	
	Susie.reach.h = dist
	Susie.foundMobj = item
end

// spawns a Rude Buster/Red Buster projectile
function Susie.SpawnProjectile(player, target)
	local mo = player.mo
	local delta = player.deltaChars
	
	local angle = player.drawangle
	local teams = G_GametypeHasTeams()
	local red = delta.flags & DC.FLAGS.RED_BUSTER
	local startAngle, endAngle
	local color
	
	if red
		startAngle = -Susie.RED_BUSTER_CONE
		endAngle = Susie.RED_BUSTER_CONE
	else
		startAngle = 0
		endAngle = 0
	end
	
	for addAngle = startAngle, endAngle, Susie.RED_BUSTER_CONE
		local beam = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_SUSIE_PROJECTILE)
		local angle = angle + addAngle
		local radius = mo.radius + beam.radius
		local x = P_ReturnThrustX(mo, angle, radius)
		local y = P_ReturnThrustY(mo, angle, radius)
		
		beam.target = mo
		if red
		and addAngle == 0
			beam.flags2 = $ | MF2_STRONGBOX // double damage
		end
		P_TryMove(beam, mo.x + x, mo.y + y, true)
		
		if not beam.valid
			continue
		end
		
		beam.momx = mo.momx
		beam.momy = mo.momy
		P_Thrust(beam, angle, FixedMul(beam.info.speed, beam.scale))
		beam.angle = angle
		Susie.EnsureMinSpeed(beam)
		
		beam.destscale = 3*beam.scale/2
		beam.fuse = beam.info.reactiontime
		if not teams
		and beam.flags2 & MF2_STRONGBOX
			beam.color = Susie.RED_BUSTER_COLOR
		else
			if beam.flags2 & MF2_STRONGBOX
				beam.colorized = true
			end
			beam.color = player.skincolor
		end
	end
	
	if red
		S_StartSound(mo, sfx_redba)
	else
		S_StartSound(mo, sfx_rudeba)
	end
end

// ensures an object is traveling at least as fast as its speed in the direction of its angle
function Susie.EnsureMinSpeed(beam)
	local targetSpeed = FixedMul(beam.info.speed, beam.scale)
	local angle = R_PointToAngle2(0, 0, beam.momx, beam.momy) - beam.angle
	local speed = FixedHypot(beam.momx, beam.momy)
	local speedAtAngle = FixedMul(speed, cos(angle))
	
	if speedAtAngle < targetSpeed
		P_Thrust(beam, beam.angle, targetSpeed - speedAtAngle)
	end
end

// spawns a dust ring around @player
function Susie.DustRing(player)
	local mo = player.mo
	local momx = 3*mo.momx/4
	local momy = 3*mo.momy/4
	local scale = mo.scale
	local radius = FixedDiv(mo.radius, scale)
	local angle = 0
	
	for i = 1, 8
		local x = P_ReturnThrustX(mo, angle, radius)
		local y = P_ReturnThrustY(mo, angle, radius)
		local dust = P_SpawnMobjFromMobj(mo, 2*x, 2*y, 0, MT_SPINDUST)
		dust.state = S_DC_DUST
		dust.spritexscale = 2*$
		dust.spriteyscale = dust.spritexscale
		dust.momx = FixedMul(x, scale) + momx
		dust.momy = FixedMul(y, scale) + momy
		angle = $ + ANGLE_45
		while P_RandomChance(FRACUNIT/2)
		and dust.valid
			DC.AdvanceState(dust, 1)
		end
	end
	S_StartSound(mo, sfx_dcbrst)
end

// shakes the screen
function Susie.StartQuake(player)
	if not Susie.GetReach(player)
		return
	end
	
	local mo = player.mo
	local range = FixedMul(Susie.SHOCKWAVE_RANGE, mo.scale)
	
	S_StartSound(mo, sfx_dcslam)
	
	Susie.axX = mo.x + FixedMul(cos(player.drawangle), Susie.reach.h)
	Susie.axY = mo.y + FixedMul(sin(player.drawangle), Susie.reach.h)
	searchBlockmap("objects", Susie.TryQuake, mo,
		Susie.axX - range, Susie.axX + range,
		Susie.axY - range, Susie.axY + range)
	
	DC.StartQuake(player, 24*mo.scale, 3)
end

// makes objects jump in response to Susie's ax landing nearby
// Susie.axX and Susie.axY must be set before this can be called
function Susie.TryQuake(mo, item)
	// item must have one of these properties
	if not (item.flags & (MF_ENEMY|MF_MONITOR|MF_PUSHABLE|MF_SPECIAL))
		return
	end
	
	// item must not have any of these properties
	if item.flags & (MF_SCENERY|MF_NOTHINK|MF_NOGRAVITY|MF_SPRING|MF_BOSS|MF_NOSECTOR|MF_AMBIENT)
	or not P_IsObjectOnGround(item)
	or item.type == MT_TARGET
	or item.player
		return
	end
	
	local range = FixedMul(Susie.SHOCKWAVE_RANGE, mo.scale)
	local dist
	
	local dist = FixedHypot(FixedHypot(Susie.axX - item.x, Susie.axY - item.y),
		(mo.z + (mo.height >> 1)) - (item.z + (item.height >> 1)))
	
	if dist > range
		return
	end
	
	if item.type == MT_MINUS
		item.state = item.info.missilestate
		return
	end
	
	local factor = FRACUNIT
	
	if mo.player.deltaChars.flags & DC.FLAGS.RED_BUSTER
		dist = 0
	end
	
	P_SetObjectMomZ(item, DC.MobjAdjustMomZ(item, FixedMul(FRACUNIT - FixedDiv(dist, range), Susie.SHOCKWAVE_POWER)), true)
end

// returns true if @player is in a state that could attack @item
function Susie.CanAttack(player, item)
	local state = player.mo.state
	
	// states that can damage enemies at any angle
	if Susie.InCrashState(player)
	or state == S_SUSIE_AXCRASH_LANDING
	or state == S_SUSIE_ATTACK
	or state == S_SUSIE_RUDEBUSTER
		return true
	end
	
	return false
end

function Susie.InCrashState(player)
	local state = player.mo.state
	
	return state == S_SUSIE_AXCRASH
	or state == S_SUSIE_REDCRASH
end

// returns true if @player can bust fofs
// if @x and @y are given, takes the coordinates into account
function Susie.CanBust(player, x, y)
	local state = player.mo.state
	
	if Susie.InCrashState(player)
	or state == S_SUSIE_AXCRASH_LANDING
	or state == S_SUSIE_ATTACK
	or state == S_SUSIE_RUDEBUSTER
		return true
	end
	
	return false
end

// spawns Susie's damage effect on @item
function Susie.SpawnDamageEffect(mo, item)
	local effect = P_SpawnMobjFromMobj(item, 0, 0, 0, MT_THOK)
	effect.state = S_DC_SLAM
	effect.spritexscale = 2*$
	effect.spriteyscale = effect.spritexscale
	S_StartSound(mo, sfx_susie6)
end

// reverts monitor/enemy bounces
function Susie.RevertObjectBounce(player, item)
	local mo = player.mo
	local delta = player.deltaChars
	
	// revert vertical bounces
	if delta.flags & DC.FLAGS.FALLING
	and P_MobjFlip(mo)*mo.momz > 0
		mo.momz = -$
	end
	
	// revert horizontal bounces
	if item.info.spawnhealth > 1
		mo.momx = -$
		mo.momy = -$
	end
end

// "kills" a destructable
function Susie.KillDestructable(item, mo)
	local player = mo.player
	
	//Run fake hook for Susie killing a destructable
	//This is used for dialog purposes
	DC.RunHook(MT_NULL, "SusieDestructable", item, mo)
	
	//Set hitlag and sfx
	if item.type ~= MT_EGGROBO1
		S_StartSound(mo, sfx_dcslic)
		if player
			DC.SetHitlag(player, 4)
		end
	end
		
	// just kill the egg shield normally
	if item.type == MT_EGGSHIELD
		P_KillMobj(item)
		return
		
	//Kill entire claw when destroying claw
	elseif item.type == MT_CRUSHCLAW
		local chain = item.target
		local chainnext
		while (chain)
			chainnext = chain.target
			P_RemoveMobj(chain)
			chain = chainnext
		end
		S_StopSound(item)
		P_KillMobj(item, mo, mo)
		return
	end
	
	//Add score
	player = $ or (mo.flags & MF_MISSILE and mo.target and mo.target.player)
	if player and item.type ~= MT_SALOONDOOR
	and (not DC.MobjIsCactus(item) or item.spawnpoint)
		P_SpawnMobj(item.x, item.y, item.z + (item.height / 2), MT_SCORE).state = $
		P_AddPlayerScore(player, 100)
	end
	
	//Kill the eggrobo like a crawla
	if item.type == MT_EGGROBO1
		Susie.SpawnDamageEffect(mo, item)
		item.type = MT_BLUECRAWLA
		item.spritexscale = $*2
		item.spriteyscale = $*2
		A_ForceStop(item)
		P_KillMobj(item)
		return
	end
	
	// replace the destructable with particles
	if item.flags & MF_NOTHINK or item.type == MT_EGGMOBILE4_MACE
		local part = P_SpawnMobjFromMobj(item, 0, 0, 0, MT_PARTICLE)
		part.height = item.height
		P_SetMobjStateNF(part, item.state)
		part.angle = item.angle
		
		//Remove the object, unless you're a CEZ3 mace
		if item.type == MT_EGGMOBILE4_MACE
			item.flags = ($ & MF_NOBLOCKMAP) | MF_NOCLIP | MF_NOCLIPHEIGHT | MF_NOCLIPTHING
			item.state = S_INVISIBLE
			item.health = 0
		else
			P_RemoveMobj(item)
		end
		item = part
	else
		S_StopSound(item)
	end
	
	// kill it!!
	item.flags = ($ & MF_NOBLOCKMAP) | MF_NOCLIP | MF_NOCLIPHEIGHT | MF_NOCLIPTHING
	item.health = 0
	item.tics = -1
	item.fuse = TICRATE
	item.momx = $ + mo.momx
	item.momy = $ + mo.momy
	searchBlockmap("objects", function(prev, item) Susie.DestructableSearch(mo, prev, item) end, item)
	P_SetObjectMomZ(item, DC.MobjAdjustMomZ(item, 12*FRACUNIT), true)
	P_Thrust(item, P_RandomFixed() << (32 - FRACBITS), 4*item.scale)
	DC.AddDestructable(item)
end

function Susie.DestructableSearch(mo, prev, item)
	if not item.health or not DC.MobjIsDestructable(item)
	or ((prev.x ~= item.x or prev.y ~= item.y)
	and not (item.type == MT_SALOONDOOR or item.type == MT_SALOONDOORCENTER))
		return
	end
	
	if prev.eflags & MFE_VERTICALFLIP
		if item.z + item.height > prev.z + prev.height
		or item.z + item.height < prev.z
			return
		end
	else
		if item.z < prev.z
		or item.z > prev.z + prev.height
			return
		end
	end
	
	Susie.KillDestructable(item, mo)
end

// --------------
// hook functions
// --------------

function Susie.AbilitySpecial(player, button)
	if player.skin ~= Susie.GetSkinIndex()
	or not DC.CheckStatus(player)
	or player.charability ~= skins[player.skin].ability
	or button == BT_SPIN and player.pflags & PF_SPINDOWN
		return
	end
	
	if player.pflags & PF_THOKKED
		return true
	end
			
	Susie.StartAxCrash(player, button)
end

function Susie.PlayerCanDamage(player, item)
	if player.skin ~= Susie.GetSkinIndex()
	or not DC.CheckStatus(player)
		return
	end
	
	if Susie.CanAttack(player, item)
	or player.deltaChars.flags & DC.FLAGS.CRASHING
		return true
	end
end

function Susie.PlayerMobjCollide(mo, item)
	local player = mo.player
	
	if not item.valid
	or not player
	or player.skin ~= Susie.GetSkinIndex()
	or not Susie.CanAttack(player, item)
	or mo.z > item.z + item.height
	or mo.z + mo.height < item.z
	or not item.health
		return
	end
	
	Susie.AxSpecial(player, item)
end

function Susie.PlayerMobjLineCollide(mo, line)
	local player = mo.player
	
	if not player
	or player.skin ~= Susie.GetSkinIndex()
	or not Susie.CanBust(player)
		return
	end
	
	local x, y = DC.ClosestPointOnLine(line, mo.x, mo.y)
	
	if not Susie.CanBust(player, x, y)
		return
	end
	
	DC.MobjBustFOFsThroughLine(mo, line)
end

//Well isn't this interesting?
function Susie.AridCanyonLineCollide(mo, line)
	if mapheaderinfo[gamemap].lvlttl == "Arid Canyon"
	and mapheaderinfo[gamemap].actnum == 2
	and mo and mo.valid and mo.player and gamemap == 14
	and line and line.valid and (#line == 7444 or #line == 7318 or #line == 7282)
		local player = mo.player
		if line.frontside.midtexture == 0
		and line.backside.midtexture == 0
			return false
		elseif player.skin == Susie.GetSkinIndex() and Susie.CanBust(player)
		and line.frontside.midtexture == R_CheckTextureNumForName("ACMINES5")
		and line.backside.midtexture == line.frontside.midtexture
			line.frontside.midtexture = 0
			line.backside.midtexture = 0
			if #line == 7318 and lines[7282] and lines[7282].valid
				lines[7282].frontside.midtexture = 0
				lines[7282].backside.midtexture = 0
			elseif #line == 7282 and lines[7318] and lines[7318].valid
				lines[7318].frontside.midtexture = 0
				lines[7318].backside.midtexture = 0
			end
			
			for i = 0, 15
				local fa = i*ANGLE_22h
				local ns = mo.radius
				local wood = P_SpawnMobjFromMobj(mo, 0,0, player.height/2, MT_WOODDEBRIS)
				wood.momz = FixedMul(sin(fa),ns)
				P_InstaThrust(wood, player.drawangle+ANGLE_90,
				FixedMul(cos(fa),ns))
			end
			
			S_StartSound(mo, sfx_wbreak)
			DC.SetHitlag(player, 4)
			return false
		end
	end
end

function Susie.PlayerMobjDamage(mo)
	local player = mo.player
	if player and player.skin == Susie.GetSkinIndex()
	and player.powers[pw_carry] ~= CR_NIGHTSMODE
	and P_RandomChance(Susie.HURT_SOUND_CHANCE)
		S_StartSound(mo, sfx_susieh)
	end
end

function Susie.MobjDamage(item, inf, src, damage, damageType)
	local mo = inf or src
	local player = DC.Valid(mo) and mo.player
	
	if not player
	or player.skin ~= Susie.GetSkinIndex()
	or not DC.CheckStatus(player)
	or not item.valid
	or not Susie.CanAttack(player, item)
		return
	end
	
	if mo == inf
	or inf == nil
		Susie.RevertObjectBounce(player, item)
	end
	Susie.SpawnDamageEffect(mo, item)
end

function Susie.ShouldDamage(item, inf, src, damage, damageType)
	if damage > 1
	or not item.valid
		return
	end
	
	local mo = inf or src
	
	// Red Buster Susie/ax?
	if DC.Valid(mo)
	and mo.state == S_SUSIE_REDCRASH
	// Red Buster projectile?
	or DC.Valid(inf)
	and inf.type == MT_SUSIE_PROJECTILE
	and inf.flags2 & MF2_STRONGBOX
		P_DamageMobj(item, inf, src, 2, damageType)
		return false
	end
end

function Susie.FangDeath(mobj, inf, mo, damageType)
	local player = mo.player
	if not (player and player.skin == Susie.GetSkinIndex()) then return end
	
	//Don't cause an infinite loop!
	if mobj.susiefangdie
		return
	end
	
	//Do the normal death, then instantly skip to fang flying towards the sign
	mobj.susiefangdie = true
	P_KillMobj(mobj, inf, mo, damageType)
	A_Boss5FindWaypoint(mobj, 2, 0)
	mobj.state = S_FANG_FLEEBOUNCE2
	mobj.frame = states[mobj.info.painstate].frame
	
	//Susie winner!
	S_StartSound(mo, sfx_bsnipe)
	DC.SetHitlag(player, 6)
	
	return true
end

function Susie.AxERZ2Shortcut(player, sector, bottom, top)
	if sector.floorheight ~= sector.ceilingheight
		return
	end
	
	// this map doesn't have slopes anyway
	if sector.floorheight < bottom
		return
	end
	
	/*if player.mo.state ~= S_SUSIE_REDCRASH
		return
	end*/
	
	if player.deltaChars.flags & DC.FLAGS.SECRET_COOLDOWN
		return
	end
	
	local map = mapheaderinfo[gamemap]
	
	if not map
	or map.actnum ~= 2
	or map.lvlttl:lower() ~= "egg rock"
		return
	end
	
	local lineNum = P_FindSpecialLineFromTag(403, sector.tag)
	
	if lineNum == -1
		return
	end
	
	local control = lines[lineNum].frontsector
	
	if control == nil
		return
	end
	
	player.deltaChars.flags = $ | DC.FLAGS.SECRET_COOLDOWN
	
	if DC.secretHP > 0
		DC.secretHP = $ - 1
		DC.SetHitlag(player, 6)
		S_StartSound(player.mo, sfx_dcslic)
		return
	end
	
	sector.floorheight = control.floorheight
	DC.CrumbleSector(sector, MT_ROCKCRUMBLE7)
	DC.TryRunHook(sector.tag, "SusieBustSector", player, sector)
end	

// -----
// hooks
// -----

DC.RegisterCustomHook("SusieDestructable")
DC.RegisterCustomHook("SusieAxSector")
DC.RegisterCustomHook("SusieBustSector")

DC.AddHook("AbilitySpecial", function(player)
	return Susie.AbilitySpecial(player, BT_JUMP)
end)
DC.AddHook("JumpSpinSpecial", function(player)
	return Susie.AbilitySpecial(player, BT_USE)
end)
DC.AddHook("PlayerCanDamage", Susie.PlayerCanDamage)

DC.AddHook("MobjCollide", Susie.PlayerMobjCollide, MT_PLAYER)
DC.AddHook("MobjMoveCollide", Susie.PlayerMobjCollide, MT_PLAYER)
DC.AddHook("MobjLineCollide", Susie.AridCanyonLineCollide, MT_PLAYER)
DC.AddHook("MobjDamage", Susie.PlayerMobjDamage, MT_PLAYER)
DC.AddHook("MobjDeath", Susie.FangDeath, MT_FANG)

DC.AddHook("ShouldDamage", Susie.ShouldDamage)
DC.AddHook("MobjDamage", Susie.MobjDamage)

DC.AddHook("SusieAxSector", Susie.AxERZ2Shortcut, 495)

DC.AddMember(Susie)