/*
SusieAxAngle preference

v1
*/

local AA = {}

DeltaChars.AxAngle = AA

// settings
AA.options = {
	"ground",
	"air",
	"buster"
}

for i, option in ipairs(AA.options)
	AA[option:upper()] = 1 << (i - 1)
end

AA.NONE = 0
AA.OFF = AA.NONE
AA.NO = AA.NONE
AA["0"] = AA.NONE

AA.ALL = (1 << #AA.options) - 1
AA.ON = AA.ALL
AA.YES = AA.ALL
AA["1"] = AA.ALL

// aliases
AA.GROUNDED = AA.GROUND
AA.CRASH = AA.AIR
AA.AERIAL = AA.AIR
AA.PROJECTILE = AA.BUSTER
AA.SHOOT = AA.BUSTER
AA.DEFAULT = AA.NONE

AA.NAME = "dc_susieaxangle"
AA.DESCRIPTION = "Enter any combination of options to choose which ax attacks Susie will use facing the camera direction instead of her input direction:"
AA.FILENAME = "client/DeltaChars/SusieAxAngle.dat"
AA.DEFAULTCONTENTS = (tostring(AA.DEFAULT).."\n"):rep(2)

AA.PLAYER_VARIABLE = "dc_susieAxAngle"

AA.configPlayers = {}

// functions

local function valid(mo)
	return mo and mo.valid
end

AA.GetOptions = function()
	local ret = "\n\tnone"
	for i, option in ipairs(AA.options)
		ret = $ .. "\n\t" .. option
	end
	ret = $ .. "\n\tall"
	return ret
end

AA.OptionsToString = function(value)
	local ret = "Currently enabled options:"
	
	if not value
		return ret .. " none"
	end
	
	if value == AA.ALL
		return ret .. " all"
	end
	
	for i, option in ipairs(AA.options)
		if value & (1 << (i - 1))
			ret = $ .. " " .. option
		end
	end
	
	return ret
end

AA.Command = function(player, ...)
	local printPlayer = player == secondarydisplayplayer and consoleplayer or player
	
	if ... == nil
		local commandExample = AA.NAME .. (splitscreen and 2 or "") .. " <options>\n"
		CONS_Printf(printPlayer,
			commandExample .. AA.DESCRIPTION .. AA.GetOptions())
		CONS_Printf(printPlayer, AA.OptionsToString(player[AA.PLAYER_VARIABLE]))
		return
	end
	
	local value = 0
	local args = {...}
	local silent = false
	local raw = false
	local save = true
	local validArgument = false
	
	// parse flags
	for i = #args, 1, -1
		local arg = args[i]:upper()
		
		if arg == "-SILENT"
			silent = true
			table.remove(args, i)
		elseif arg == "-RAW"
			raw = true
			table.remove(args, i)
		elseif arg == "-NOSAVE"
			save = false
			table.remove(args, i)
		end
	end
	
	// parse arguments
	for i, arg in ipairs(args)
		local n
		
		if raw
			n = tonumber(arg)
		else
			n = tonumber(AA[arg:upper()])
		end
		
		if n ~= nil
			value = $ | n
			validArgument = true
		end
	end
	
	if validArgument
		player[AA.PLAYER_VARIABLE] = value
		if save
		and (player == consoleplayer or player == secondarydisplayplayer)
			AA.WriteToFile(value, player == secondarydisplayplayer and 2 or 1)
		end
	end
	
	if silent
		return
	end
	
	if not validArgument
		CONS_Printf(printPlayer, "No valid options found!")
	end
	CONS_Printf(printPlayer, AA.OptionsToString(player[AA.PLAYER_VARIABLE]))
end

AA.WriteToFile = function(val, line)
	local n = 0
	local text
	local file = io.openlocal(AA.FILENAME)
	
	if file
		text = file:read("*a")
		file:close()
	else
		text = AA.DEFAULTCONTENTS
	end
	
	line = $ or 0
	while line > 1
		line = $ - 1
		n = text:find('\n', n + 1)
	end
	
	local fin = text:find('\n', n + 1)
	
	file = io.openlocal(AA.FILENAME, "w+")
	file:write(text:sub(1, n), val, text:sub(fin))
	file:close()
end

AA.ReadFromFile = function(line)
	local file = io.openlocal(AA.FILENAME)
	
	if not file // no file? use the default
		return AA.DEFAULT
	end
	
	while line > 1
		line = $ - 1
		file:read()
	end
	
	local val = tonumber(file:read())
	
	file:close()
	
	if val == nil
		return AA.DEFAULT
	else
		return val
	end
end

// called in DeltaChars.ThinkFrame()
AA.CheckConfig = function()
	if isserver ~= (consoleplayer == server) // the client is player 1 while the gamestate is being received, avoid sending commands during this time
		return
	end
	
	for i, player in ipairs({consoleplayer, secondarydisplayplayer})
		if not valid(AA.configPlayers[i])
		and valid(player)
			AA.configPlayers[i] = player
			if player[AA.PLAYER_VARIABLE] == nil
				local text = AA.NAME
				if i == 2
					text = $..2
				end
				text = $ .. " -silent -raw -nosave " .. AA.ReadFromFile(i)
				CommandQueue_Add(text)
			end
		end
	end
end

COM_AddCommand(AA.NAME..2, AA.Command, COM_SPLITSCREEN)
COM_AddCommand(AA.NAME, AA.Command)